program lensedmap

  use healpix_types
  use healmod
  use misc_utils
  use fitstools
  use paramfile_io
  use extension

  implicit none

  integer(i4b)  :: status, pol
  integer(i4b)  :: nside,nlmax
  integer(i4b)  :: sigma, kappa
  character(len=filenamelen) :: paramfile
  character(len=filenamelen) :: cmbalmfile,potalmfile
  character(len=filenamelen) :: lensedcmbmapfile
  real(dp), dimension(:), allocatable :: thmap,phmap,thnewmap,phnewmap
  real(dp), dimension(:), allocatable :: cosrotmap, sinrotmap
  real(dp), dimension(:,:), allocatable :: potmap,potmapderth,potmapderph
  real(dp), dimension(:,:), allocatable :: lensedcmbmap
  complex(dpc), dimension(:,:,:), allocatable :: cmbalm,potalm

  call getarg(1,paramfile)
  open(1,file=paramfile,status='old') 
  read(1,'(a)') cmbalmfile
  read(1,'(a)') potalmfile
  read(1,*) nside
  read(1,*) nlmax
  read(1,*) pol
  read(1,*) sigma
  read(1,*) kappa
  read(1,'(a)') lensedcmbmapfile
  close(1)

  allocate(potalm(1:1,0:nlmax,0:nlmax),stat = status)
  if (status /= 0) stop 
  potalm(:,:,:)=0.d0

  call fitsalm2asciialm(0,nlmax,potalmfile,potalm)

  allocate(potmap(0:12*nside*nside-1,1:1),stat = status)
  if (status /= 0) stop 
  potmap(:,:)=0.d0
  allocate(potmapderth(0:12*nside*nside-1,1:1),stat = status)
  if (status /= 0) stop 
  potmapderth(:,:)=0.d0
  allocate(potmapderph(0:12*nside*nside-1,1:1),stat = status)
  if (status /= 0) stop 
  potmapderph(:,:)=0.d0

  call asciialm2asciimapder(0,nlmax,potalm,nside, &
                         &   potmap,potmapderth,potmapderph)

  deallocate(potalm)

  allocate(thmap(0:12*nside*nside-1),stat=status)
  if (status /= 0) stop
  thmap(:)=0.d0
  allocate(phmap(0:12*nside*nside-1),stat=status)
  if (status /= 0) stop
  phmap(:)=0.d0

  call direction(nside,thmap,phmap)

  allocate(thnewmap(0:12*nside*nside-1),stat=status)
  if (status /= 0) stop
  thnewmap(:)=0.d0
  allocate(phnewmap(0:12*nside*nside-1),stat=status)
  if (status /= 0) stop
  phnewmap(:)=0.d0

  allocate(cosrotmap(0:12*nside*nside-1),stat=status)
  if (status /= 0) stop
  cosrotmap(:)=0.d0
  allocate(sinrotmap(0:12*nside*nside-1),stat=status)
  if (status /= 0) stop
  sinrotmap(:)=0.d0

  call change_direction(nside,thmap,phmap,potmapderth,potmapderph, &
                        & thnewmap,phnewmap,cosrotmap,sinrotmap)

  deallocate(thmap)
  deallocate(phmap)

  allocate(cmbalm(1:2*pol+1,0:nlmax,0:nlmax),stat = status)
  if (status /= 0) stop 
  cmbalm(:,:,:)=0.d0

  call fitsalm2asciialm(pol,nlmax,cmbalmfile,cmbalm)

  allocate(lensedcmbmap(0:12*nside*nside-1,1:2*pol+1),stat = status)
  if (status /= 0) stop 
  lensedcmbmap(:,:)=0.d0  

  call simulate_nfft_map(pol,nlmax,cmbalm,nside,sigma,kappa, &
       & thnewmap,phnewmap,lensedcmbmap)

  deallocate(cmbalm)

  if (pol .eq. 1) call rotate_map(nside,cosrotmap,sinrotmap,lensedcmbmap)

  deallocate(thnewmap)
  deallocate(phnewmap)
  deallocate(cosrotmap)
  deallocate(sinrotmap)

  call asciimap2fitsmap(pol,nside,lensedcmbmap,'!'//lensedcmbmapfile)

  deallocate(lensedcmbmap)

end program lensedmap














