FUNCTION INSTRUMENTAL_NOISE, channel, polar=polar

  IF NOT KEYWORD_SET(polar) THEN polar=0B

  CASE channel.inst OF

;;=============================================================================================================================
;; PSM_IDEAL INSTRUMENT
;;=============================================================================================================================
     'PSM_IDEAL': BEGIN
        
        COMMON obs, obser

        ;; Get the beam of the PSM_IDEAL channel, and compare with the beam of the requested noise realisation
        ;;-----------------------------------------------------------------------------------------------------
        i = FIX(PSM_STR_REPLACE(channel.name,'band',''))-1
        obsbeam = GET_BEAM_STRUCT('GAUSSIAN', beamsize=obser.psm_ideal.resolution(i))
        norebeam = BEAMS_EQUAL(obsbeam,channel.beam)

        ;; Get to the appropriate seed chain
        ;;-----------------------------------
        PSM_SEEDSTATUS, verbosity=0, exists=seedchainexists, seedchain=previousseedchain
        SET_PSM_SEEDCHAIN, 'psm_ideal_noise'

        ;; First do the noise map at the resolution of the PSM instrument
        ;;----------------------------------------------------------------

        ;; Generate a set of random numbers of the appropriate dimension
        IF NOT KEYWORD_SET(polar) THEN map = RANDOMN(GETSEED(),channel.pix.npix) ELSE map = RANDOMN(GETSEED(),channel.pix.npix,3)
        
        ;; Then scale it to match the expected noise level per pixel
        pixelsize = NSIDE2PIXSIZE(channel.pix.nside,/deg)
        factor_T = obser.psm_ideal.t_noise_level[i] / pixelsize
        map[*,0] = map[*,0] * factor_T
        IF polar NE 0 THEN BEGIN
           factor_P = obser.psm_ideal.p_noise_level[i] / pixelsize
           map[*,1] = map[*,1] * factor_P 
           map[*,2] = map[*,2] * factor_P 
        ENDIF
        
        ;; Put the noise in the right units
        noiseunits = obser.psm_ideal.noise_level_units[i]
        obsunits = obser.psm_ideal.channel_units[i]
        map = TEMPORARY(map) * CONVERSION_FACTOR(PSM_STR_REPLACE(noiseunits,'/deg',''),obsunits,channel.band)

        ;; Now rebeam the map if needed
        IF norebeam NE 0 THEN BEGIN
           CHANGE_MAP_RESOLUTION, TEMPORARY(map), map, beam_in=obsbeam, beam_out=channel.beam, lmax=3*channel.pix.nside-1L, $
                                  ordering_in=channel.pix.ordering, ordering_out=channel.pix.ordering, nside_out=channel.pix.nside
        ENDIF

        ;; Get back to the previous seed chain
        ;--------------------------------------
        IF seedchainexists THEN SET_PSM_SEEDCHAIN, previousseedchain
  
     END

;;=============================================================================================================================
;; HFI_IDEAL INSTRUMENT
;;=============================================================================================================================
     'HFI_IDEAL': BEGIN
        COMMON obs, obser
        duration_months = obser.hfi_ideal.duration_months        
        map = BLUEBOOK_NOISE(PSM_STR_REPLACE(channel.name,'GHz',''), channel.obs_units, polar=polar, $
                             nside=channel.pix.nside, ordering=channel.pix.ordering)

     END

;;=============================================================================================================================
;; HFI_BLUEBOOK INSTRUMENT
;;=============================================================================================================================
     'HFI_BLUEBOOK': BEGIN
        COMMON obs, obser
        duration_months = obser.hfi_bluebook.duration_months        
        map = BLUEBOOK_NOISE(PSM_STR_REPLACE(channel.name,'GHz',''), channel.obs_units, polar=polar, $
                             nside=channel.pix.nside, ordering=channel.pix.ordering)

     END

;;=============================================================================================================================
;; HFI_RIMO INSTRUMENT
;;=============================================================================================================================
     'HFI_RIMO': BEGIN
        ;COMMON obs, obser
        ;duration_months = obser.hfi_bluebook.duration_months        
        ;map = BLUEBOOK_NOISE(PSM_STR_REPLACE(channel.name,'GHz',''), channel.obs_units, polar=polar, $
        ;                     nside=channel.pix.nside, ordering=channel.pix.ordering)
        STOP
     END

;;=============================================================================================================================
;; LFI_IDEAL INSTRUMENT
;;=============================================================================================================================
     'LFI_IDEAL': BEGIN
        COMMON obs, obser
        duration_months = obser.lfi_ideal.duration_months        
        map = BLUEBOOK_NOISE(PSM_STR_REPLACE(channel.name,'GHz',''), channel.obs_units, polar=polar, $
                             nside=channel.pix.nside, ordering=channel.pix.ordering)

     END

;;=============================================================================================================================
;; LFI_BLUEBOOK INSTRUMENT
;;=============================================================================================================================
     'LFI_BLUEBOOK': BEGIN
        COMMON obs, obser
        duration_months = obser.lfi_bluebook.duration_months        
        map = BLUEBOOK_NOISE(PSM_STR_REPLACE(channel.name,'GHz',''), channel.obs_units, polar=polar, $
                             nside=channel.pix.nside, ordering=channel.pix.ordering)

     END

;;=============================================================================================================================
;; WMAP INSTRUMENT
;;=============================================================================================================================
     'WMAP': BEGIN
        
        ;; Get the beam of the WMAP channel, and compare with the beam of the requested noise realisation
        ;;------------------------------------------------------------------------------------------------
        wmapbeam = GET_BEAM_STRUCT('INSTR', instr='WMAP', version=channel.version, channel=channel.name)
        norebeam = BEAMS_EQUAL(wmapbeam,channel.beam)

        ;; Do the noise map differently as a function of whether it has to be 'rebeamed' or not
        ;;--------------------------------------------------------------------------------------
        IF norebeam THEN BEGIN
           ;; If norebeam, get the WMAP noise in the appropriate pixelisation directly
           map = WMAP_NOISE(channel.name, version=channel.version, pix=channel.pix, polar=polar)
        ENDIF ELSE BEGIN
           ;; Otherwise, do the WMAP noise at its original nside,
           ;; rebeam it, and then put it in the appropriate pixelisation
           map = WMAP_NOISE(channel.name, version=channel.version, pix=pix, polar=polar)
           CHANGE_MAP_RESOLUTION, TEMPORARY(map), map, beam_in=wmap_beam, beam_out=channel.beam, lmax=3*pix.nside-1L, $
                                  ordering_in=pix.ordering, ordering_out=channel.pix.ordering, nside_out=channel.pix.nside
        ENDELSE

        ;; Convert to channel units if necessary
        ;;---------------------------------------
        conv = channel.conversion.kcmb2units*1e-3
        IF conv NE 1 THEN map = TEMPORARY(map) * conv

     END


     ELSE: STOP

  ENDCASE

  RETURN, map

END	 
