FUNCTION READ_OBS_PARAM, table, carefulness=carefulness, verbosity=verbosity


  ;; Carefulness and Verbosity
  ;;---------------------------
  cv = { $
       carefulness:carefulness, $                       ; {0,1,2} Sets the amount of carefulness during the run
       verbosity:verbosity}                             ; {0,1,2} Sets the amount of info given while the PSM runs

  max_freq_nb = 64

  ;; Global observation parameters
  ;;-------------------------------
  global = {$
           precision:'single', $                        ; {'single','double'}
           fields:'T', $                                ;
           ;obs_pix:'sky', $                             ; {'sky','instr'}
           obs_res:'sky' $                              ; {'sky','instr'}
           }
  tags = ['PRECISION','FIELDS','OBS_RES'];,'OBS_PIX']
  FILL_STRUCT, table, global, tags=tags, carefulness=cv.carefulness

  ;; Tasks to be done in the observation
  ;;-------------------------------------
  what = {$
         strong_sources_to_cat:'no', $                  ; whether to write catalogue(s) of strong sources for each band
         strong_sources_to_map:'yes', $                 ; whether to write map(s) of strong sources for each band
         group_galaxy:'yes', $                          ; whether to group galaxy components in observed bands
         group_faint_ps:'yes', $                        ; whether to group faint PS components (including FIRB) in observed bands
         group_strong_ps:'yes', $                       ; whether to group strong PS components
         what_obs:'bandinteg' $                         ; {'bandinteg','fullobs'} whether to do just bandintegration, or full observations
         }  
  FILL_STRUCT, table, what, carefulness=cv.carefulness

  ;; List of instruments
  ;;---------------------
  wh = WHERE(STRUPCASE(STRTRIM(table[0,*],2) EQ 'INSTRUMENT'), ninst)
  IF ninst NE 0 THEN BEGIN
     instruments = STRARR(ninst)
     instruments[0:ninst-1] = REFORM(table[1,wh])
  ENDIF ELSE BEGIN
     instruments = ['']
  ENDELSE
  list = STRTRIM(instruments,2)

  ;;-------------------------------------------------------------------------------------
  ;; PSM_IDEAL instrument
  ;;-------------------------------------------------------------------------------------
  wh = WHERE(list EQ 'PSM_IDEAL', nwh)
  IF nwh NE 0 THEN BEGIN
     howmany = {obs_frequencies: FLTARR(max_freq_nb)}
     FILL_STRUCT, table, howmany, carefulness=cv.carefulness
     wh = WHERE(howmany.obs_frequencies NE 0., nb)
     IF nb NE 0 THEN BEGIN
        dummy = {psm_ideal_noise: 'none'}
        FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, TAGS=['PSM_IDEAL_NOISE']
        dummy = { $
                nfreq: FIX(nb), $
                obs_frequencies: FLTARR(nb), $                    ; List of frequencies of psm coadded maps (GHz)
                obs_resolution: DBLARR(nb), $                     ; one value per frequency
                obs_stokes: REPLICATE('T',nb), $                  ; 'T' or 'TQU' or 'QU'
                noise: dummy.psm_ideal_noise, $                   ; 
                T_noise_level: STRARR(nb),$                       ; noise level (per square degree patch) for T
                P_noise_level: STRARR(nb),$                       ; noise level (per square degree patch) for Q and U
                noise_level_units: REPLICATE('uK_RJ/deg',nb), $   ;
                obs_units: REPLICATE('mK_RJ',nb), $               ; output units for all coadded observation maps produced for instrument PSM_IDEAL
                pix:'sky' $
                }
        FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = nb, /strict_dim, $
                     TAGS=['OBS_FREQUENCIES','OBS_RESOLUTION','OBS_STOKES','T_NOISE_LEVEL','P_NOISE_LEVEL']       
        FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = nb, /strict_dim, $
                     TAGS=['NOISE_LEVEL_UNITS','OBS_UNITS']    
        psminst = { $
                  nfreq: dummy.nfreq, $
                  frequencies: dummy.obs_frequencies, $           ; List of frequencies of psm coadded maps (GHz)
                  resolution: dummy.obs_resolution, $             ; one value per frequency
                  stokes: dummy.obs_stokes, $                     ; 'T' or 'TQU' or 'QU'
                  noise: dummy.noise, $                           ; 
                  T_noise_level: dummy.T_noise_level,$            ; noise level (per square degree patch) for T
                  P_noise_level: dummy.P_noise_level,$            ; noise level (per square degree patch) for Q and U
                  noise_level_units: dummy.noise_level_units, $   ;
                  channel_units: dummy.obs_units, $               ; output units for all coadded observation maps produced for instrument PSM_IDEAL
                  pix:'sky' $
                }
     ENDIF ELSE psminst = {nfreq:0}
  ENDIF ELSE psminst = {nfreq:0}

  ;;-------------------------------------------------------------------------------------
  ;; HFI_IDEAL instrument
  ;;-------------------------------------------------------------------------------------
  wh = WHERE(list EQ 'HFI_IDEAL', nwh)
  IF nwh NE 0 THEN BEGIN
     ;; output units and pixelisation for all coadded observation maps produced for instrument HFI_IDEAL
     hfi = {hfi_units: [REPLICATE('K_CMB',3),REPLICATE('K_CMB',3)], hfi_pix:'sky'} 
     FILL_STRUCT, table, hfi, carefulness=cv.carefulness, expected_dim = 6, /strict_dim, $
                  TAGS=['HFI_UNITS']        
     FILL_STRUCT, table, hfi, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, $
                  TAGS=['HFI_PIX']        
     dummy = {hfi_ideal_noise: 'none', hfi_ideal_duration:14.}
     FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, TAGS=['HFI_IDEAL_NOISE','HFI_IDEAL_DURATION']
     hfi_ideal = { $
                 nfreq: 6, $
                 frequencies: [100.,143.,217.,353.,545.,857.], $
                 noise: dummy.hfi_ideal_noise, $
                 units: hfi.hfi_units, $
                 pix: hfi.hfi_pix, $
                 duration_months: dummy.hfi_ideal_duration $
                 }
  ENDIF ELSE hfi_ideal = {nfreq:0}

  ;;-------------------------------------------------------------------------------------
  ;; LFI_IDEAL instrument
  ;;-------------------------------------------------------------------------------------
  wh = WHERE(list EQ 'LFI_IDEAL', nwh)
  IF nwh NE 0 THEN BEGIN
     ;; output units and pixelisation for all coadded observation maps produced for instrument LFI_IDEAL
     lfi = {lfi_units: REPLICATE('K_CMB',3), lfi_pix:'sky'} ; 
     FILL_STRUCT, table, lfi, carefulness=cv.carefulness, expected_dim = 3, /strict_dim, $
                  TAGS=['LFI_UNITS']        
     FILL_STRUCT, table, lfi, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, $
                  TAGS=['LFI_PIX']        
     dummy = {lfi_ideal_noise: 'none', lfi_ideal_duration:14.}
     FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, TAGS=['LFI_IDEAL_NOISE','LFI_IDEAL_DURATION']
     lfi_ideal = { $
                 nfreq: 3, $
                 frequencies: [30.,44.,70.], $
                 noise: dummy.lfi_ideal_noise, $
                 units: lfi.lfi_units, $         
                 pix: lfi.lfi_pix, $
                 duration_months: dummy.lfi_ideal_duration $
                 }
  ENDIF ELSE lfi_ideal = {nfreq:0}

  ;;-------------------------------------------------------------------------------------
  ;; IRAS_IDEAL instrument
  ;;-------------------------------------------------------------------------------------
  wh = WHERE(list EQ 'IRAS_IDEAL', nwh)
  ;; IF nwh NE 0 THEN BEGIN
  ;;    ;; output units and pixelisation for all coadded observation maps produced for instrument IRAS_IDEAL
  ;;    iras = {iras_units: REPLICATE('MJy/sr',4), iras_pix:'sky'} ; 
  ;;    FILL_STRUCT, table, iras, carefulness=cv.carefulness, expected_dim = 4, /strict_dim, $
  ;;                 TAGS=['IRAS_UNITS']        
  ;;    FILL_STRUCT, table, iras, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, $
  ;;                 TAGS=['IRAS_PIX']        
  ;;    iras_ideal = { $
  ;;                 nfreq: 4, $
  ;;                 wavelengths: [12.,25.,60.,100.], $
  ;;                 units: iras.iras_units, $         
  ;;                 pix: iras.iras_pix $
  ;;                 }
  ;; ENDIF ELSE iras_ideal = {nfreq:0}
  IF nwh NE 0 THEN BEGIN
     ;; output units and pixelisation for all coadded observation maps produced for instrument IRAS_IDEAL
     iras = {iras_units: REPLICATE('MJy/sr',2), iras_pix:'sky'} ; 
     FILL_STRUCT, table, iras, carefulness=cv.carefulness, expected_dim = 2, /strict_dim, $
                  TAGS=['IRAS_UNITS']        
     FILL_STRUCT, table, iras, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, $
                  TAGS=['IRAS_PIX']        
     iras_ideal = { $
                  nfreq: 2, $
                  wavelengths: [100.,60], $
                  noise: 'none', $
                  units: iras.iras_units, $         
                  pix: iras.iras_pix $
                  }
  ENDIF ELSE iras_ideal = {nfreq:0}

  ;;-------------------------------------------------------------------------------------
  ;; IRAS_RIMO instrument
  ;;-------------------------------------------------------------------------------------
  wh = WHERE(list EQ 'IRAS_RIMO', nwh)
  IF nwh NE 0 THEN BEGIN
     ;; output units and pixelisation for all coadded observation maps produced for instrument IRAS_RIMO
     iras = {iras_units: REPLICATE('MJy/sr',2), iras_pix:'sky'} ; 
     FILL_STRUCT, table, iras, carefulness=cv.carefulness, expected_dim = 2, /strict_dim, $
                  TAGS=['IRAS_UNITS']        
     FILL_STRUCT, table, iras, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, $
                  TAGS=['IRAS_PIX']        
     iras_rimo = { $
                  nfreq: 2, $
                  wavelengths: [100.,60], $
                  noise: 'none', $
                  units: iras.iras_units, $         
                  pix: iras.iras_pix $
                  }
  ENDIF ELSE iras_rimo = {nfreq:0}

  ;;-------------------------------------------------------------------------------------
  ;; HFI_BLUEBOOK instrument
  ;;-------------------------------------------------------------------------------------

  wh = WHERE(list EQ 'HFI_BLUEBOOK', nwh)
  IF nwh NE 0 THEN BEGIN
     ;; output units and pixelisation for all coadded observation maps produced for instrument HFI_BLUEBOOK
     hfi = {hfi_units: [REPLICATE('K_CMB',3),REPLICATE('K_CMB',3)], hfi_pix:'sky'} 
     FILL_STRUCT, table, hfi, carefulness=cv.carefulness, expected_dim = 6, /strict_dim, $
                  TAGS=['HFI_UNITS']        
     FILL_STRUCT, table, hfi, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, $
                  TAGS=['HFI_PIX']        
     dummy = {hfi_bluebook_noise: 'none', hfi_bluebook_duration:14.}
     FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, TAGS=['HFI_BLUEBOOK_NOISE','HFI_BLUEBOOK_DURATION']
     hfi_bluebook = { $
                    nfreq: 6, $
                    frequencies: [100.,143.,217.,353.,545.,857.], $
                    noise: dummy.hfi_bluebook_noise, $
                    units: hfi.hfi_units, $   
                    pix: hfi.hfi_pix, $
                    duration_months: dummy.hfi_bluebook_duration $
                    }
  ENDIF ELSE hfi_bluebook = {nfreq:0}

  ;;-------------------------------------------------------------------------------------
  ;; LFI_BLUEBOOK instrument
  ;;-------------------------------------------------------------------------------------

  wh = WHERE(list EQ 'LFI_BLUEBOOK', nwh)
  IF nwh NE 0 THEN BEGIN
     ;; output units and pixelisation for all coadded observation maps produced for instrument LFI_IDEAL
     lfi = {lfi_units: REPLICATE('K_CMB',3), lfi_pix:'sky'} ; 
     FILL_STRUCT, table, lfi, carefulness=cv.carefulness, expected_dim = 3, /strict_dim, $
                  TAGS=['LFI_UNITS']        
     FILL_STRUCT, table, lfi, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, $
                  TAGS=['LFI_PIX']        
     dummy = {lfi_bluebook_noise: 'none', lfi_bluebook_duration:14.}
     FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, TAGS=['LFI_BLUEBOOK_NOISE','LFI_BLUEBOOK_DURATION']
     lfi_bluebook = { $
                    nfreq: 3, $
                    frequencies: [30.,44.,70.], $
                    noise: dummy.lfi_bluebook_noise, $
                    units: lfi.lfi_units, $   
                    pix: lfi.lfi_pix, $
                    duration_months: dummy.lfi_bluebook_duration $
                    }
  ENDIF ELSE lfi_bluebook = {nfreq:0}

  ;;-------------------------------------------------------------------------------------
  ;; HFI_RIMO instrument
  ;;-------------------------------------------------------------------------------------

  wh = WHERE(list EQ 'HFI_RIMO', nwh)
  IF nwh NE 0 THEN BEGIN
     det_list = {hfi_rimo_100_det:STRARR(20),hfi_rimo_143_det:STRARR(20),hfi_rimo_217_det:STRARR(20),$
                hfi_rimo_353_det:STRARR(20),hfi_rimo_545_det:STRARR(20),hfi_rimo_857_det:STRARR(20)}
     FILL_STRUCT, table, det_list, carefulness=cv.carefulness
     freqlist = NUMBER2STRING([100.,143.,217.,353.,545.,857.])
     detectors = [''] 
     dummy = {hfi_rimo_version:'20110412',hfi_units:['K_CMB','K_CMB','K_CMB','K_CMB','K_CMB','K_CMB'], hfi_pix:'sky', $
              hfi_rimo_noise:'none', hfi_rimo_obs_start:'', hfi_rimo_obs_stop:''}
     FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = 6, /strict_dim, TAGS=['HFI_UNITS']
     FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, TAGS=['HFI_PIX']   
     FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, TAGS=['HFI_RIMO_VERSION'];,'HFI_RIMO_NOISE']
     FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, TAGS=['HFI_RIMO_OBS_START','HFI_RIMO_OBS_STOP']
     units = ['']
     ndet = 0
     list0 = ['F100','100_1a','100_1b','100_2a','100_2b','100_3a','100_3b','100_4a','100_4b']
     list1 = ['F143','143_1a','143_1b','143_2a','143_2b','143_3a','143_3b','143_4a','143_4b','143_5','143_6','143_7','143_8']
     list2 = ['F217','217_1','217_2','217_3','217_4','217_5a','217_5b','217_6a','217_6b','217_7a','217_7b','217_8a','217_8b']
     list3 = ['F353','353_1','353_2','353_3a','353_3b','353_4a','353_4b','353_5a','353_5b','353_6a','353_6b','353_7','353_8']
     list4 = ['F545','545_1','545_2','545_3','545_4']
     list5 = ['F857','857_1','857_2','857_3','857_4']
     FOR i=0,5 DO BEGIN
        ok = EXECUTE('detlist = det_list.hfi_rimo_'+freqlist[i]+'_det')
        ok = EXECUTE('listi=list'+NUMBER2STRING(i))
        wh = WHERE(detlist EQ 'all',nwh)
        IF nwh NE 0 THEN BEGIN
           detectors = [detectors,listi]
           units = [units,REPLICATE(dummy.hfi_units[i],N_ELEMENTS(listi))]
           ndet = ndet+N_ELEMENTS(listi)
        ENDIF ELSE BEGIN
           detin = BYTARR(N_ELEMENTS(listi))
           FOR d=0, N_ELEMENTS(listi)-1 DO BEGIN
              wh = WHERE(detlist EQ listi[d],nwh)
              IF nwh NE 0 THEN BEGIN
                 detectors = [detectors, listi[d]]
                 units = [units,dummy.hfi_units[i]]
                 ndet = ndet+1
              ENDIF
           ENDFOR
        ENDELSE
     ENDFOR
     
     IF ndet GE 1 THEN BEGIN
        detectors = detectors[1:*]
        units = units[1:*]
        hfi_rimo = {version:dummy.hfi_rimo_version, $
                    nfreq: FIX(ndet), $
                    frequencies: detectors, $
                    noise: dummy.hfi_rimo_noise, $
                    channel_units: dummy.hfi_units, $
                    pix: dummy.hfi_pix, $
                    obs_start: dummy.hfi_rimo_obs_start, $
                    obs_stop: dummy.hfi_rimo_obs_stop $
                   }
     ENDIF ELSE BEGIN
        hfi_rimo = {nfreq:0}
     ENDELSE
  ENDIF ELSE hfi_rimo = {nfreq:0}


  ;;-------------------------------------------------------------------------------------
  ;; LFI_RIMO instrument
  ;;-------------------------------------------------------------------------------------

  wh = WHERE(list EQ 'LFI_RIMO', nwh)
  IF nwh NE 0 THEN BEGIN
     det_list = {lfi_rimo_030_det:STRARR(20),lfi_rimo_044_det:STRARR(20),lfi_rimo_070_det:STRARR(20)}
     FILL_STRUCT, table, det_list, carefulness=cv.carefulness
     freqlist = '0'+NUMBER2STRING([30,44,70])
     detectors = [''] 
     dummy = {lfi_rimo_version:'03032011_DX6',lfi_units:['K_CMB','K_CMB','K_CMB'], lfi_pix:'sky', lfi_rimo_noise:'none'}
     FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = 3, /strict_dim, TAGS=['LFI_UNITS']
     FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, TAGS=['LFI_PIX']        
     FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, TAGS=['LFI_RIMO_VERSION'];,'LFI_RIMO_NOISE']
     units = ['']
     ndet = 0
     list0 = ['F030', '030-28S', '030-28M', '030-27S', '030-27M']
     list1 = ['F044', '044-26S', '044-26M', '044-25S', '044-25M', '044-24S', '044-24M']
     list2 = ['F070', '070-23S', '070-23M', '070-22S', '070-22M', '070-21S', '070-21M', '070-20S', '070-20M', '070-19S', '070-19M', '070-18S', '070-18M']
     FOR i=0,2 DO BEGIN
        ok = EXECUTE('detlist = det_list.lfi_rimo_'+freqlist[i]+'_det')
        ok = EXECUTE('listi=list'+NUMBER2STRING(i))
        wh = WHERE(detlist EQ 'all',nwh)
        IF nwh NE 0 THEN BEGIN
           detectors = [detectors,listi]
           units = [units,REPLICATE(dummy.lfi_units[i],N_ELEMENTS(listi))]
           ndet = ndet+N_ELEMENTS(listi)
        ENDIF ELSE BEGIN
           detin = BYTARR(N_ELEMENTS(listi))
           FOR d=0, N_ELEMENTS(listi)-1 DO BEGIN
              wh = WHERE(detlist EQ listi[d],nwh)
              IF nwh NE 0 THEN BEGIN
                 detectors = [detectors, listi[d]]
                 units = [units,dummy.lfi_units[i]]
                 ndet = ndet+1
              ENDIF
           ENDFOR
        ENDELSE
     ENDFOR
     IF ndet GE 1 THEN BEGIN
        detectors = detectors[1:*]
        units = units[1:*]
        lfi_rimo = {version:dummy.lfi_rimo_version, $
                    nfreq: FIX(ndet), $
                    frequencies: detectors, $
                    noise: dummy.lfi_rimo_noise, $
                    channel_units: dummy.lfi_units, $
                    pix: dummy.lfi_pix}
     ENDIF ELSE BEGIN
        lfi_rimo = {nfreq:0}
     ENDELSE
  ENDIF ELSE lfi_rimo = {nfreq:0}

  ;;-------------------------------------------------------------------------------------
  ;; WMAP instrument
  ;;-------------------------------------------------------------------------------------

  wh = WHERE(list EQ 'WMAP', nwh)
  IF nwh NE 0 THEN BEGIN
     det_list = {wmap_det:STRARR(20)}
     FILL_STRUCT, table, det_list, carefulness=cv.carefulness
     freqlist = ['K','Ka','Q','V','W']
     detectors = [''] 
     dummy = {wmap_version:'7yr',wmap_units:['mK_CMB','mK_CMB','mK_CMB','mK_CMB','mK_CMB'], wmap_pix:'sky', wmap_noise:'none'}
     FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = 5, /strict_dim, TAGS=['WMAP_UNITS']
     FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, TAGS=['WMAP_PIX']        
     FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, TAGS=['WMAP_VERSION']
     FILL_STRUCT, table, dummy, carefulness=cv.carefulness, expected_dim = 1, /strict_dim, TAGS=['WMAP_NOISE']
     units = ['']
     ndet = 0
     wh = WHERE(det_list.wmap_det EQ 'alldet', nwh)
     IF nwh NE 0 THEN detectors=['K', 'Ka', 'Q1', 'Q2', 'V1', 'V2', 'W1', 'W2', 'W3', 'W4']
     wh = WHERE(det_list.wmap_det EQ 'allfreq', nwh)
     IF nwh NE 0 THEN detectors=[detectors, ['K', 'Ka', 'Q', 'V', 'W']]
     possibledet = ['K', 'Ka', 'Q1', 'Q2', 'Q', 'V1', 'V2', 'V', 'W1', 'W2', 'W3', 'W4', 'W']
     FOR i=0, N_ELEMENTS(det_list.wmap_det)-1 DO BEGIN
        wh = WHERE(possibledet EQ (det_list.wmap_det)[i], nwh)
        IF nwh EQ 1 THEN detectors=[detectors,possibledet[wh[0]]]
     ENDFOR
     detectors=detectors[PSM_UNIQ(detectors)]
     wh = WHERE(STRLEN(detectors) NE 0, ndet)
     IF ndet NE 0 THEN BEGIN
        detectors = detectors[wh] 
        wmap = { version:dummy.wmap_version, $
                 nfreq:FIX(ndet), $
                 frequencies: detectors, $
                 channel_units: dummy.wmap_units, $
                 noise: dummy.wmap_noise, $
                 pix: dummy.wmap_pix}
     ENDIF ELSE wmap = {nfreq:0}
    
  ENDIF ELSE wmap = {nfreq:0}


  ;; Groups of components in observations
  ;;--------------------------------------
  wh = WHERE(STRUPCASE(STRTRIM(table[0,*],2) EQ 'OBS_COADD'), ngroup)
  IF ngroup NE 0 THEN BEGIN
     groups = STRARR(ngroup)
     groups[0:ngroup-1] = REFORM(table[1,wh])
  ENDIF ELSE BEGIN
     groups = ['']
  ENDELSE
  obs_groups = STRTRIM(groups,2)

  obser = {global:global,what:what,groups:obs_groups,inst:list,$
           psm_ideal:psminst,hfi_ideal:hfi_ideal,lfi_ideal:lfi_ideal,hfi_bluebook:hfi_bluebook,lfi_bluebook:lfi_bluebook, $
           hfi_rimo:hfi_rimo,lfi_rimo:lfi_rimo, wmap:wmap, iras_ideal:iras_ideal, iras_rimo:iras_rimo}
     

  RETURN, obser

END
