FUNCTION MODEL_TO_DATA_CAT, model_cat, band_list, whichsrc=whichsrc, name=name, id=id, nopolar=nopolar, polar=polar, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'MODEL_TO_DATA_CAT: function to create a point source data catalog from a model catalogue'
     PSM_INFO_MESSAGE, 'SYNTAX: result = MODEL_TO_DATA_CAT(model_cat, band_list, name=, id=, /nopolar, /polar)'
     PSM_INFO_MESSAGE, "model_cat is the input model catalogue"
     PSM_INFO_MESSAGE, "band_list: array of structures or of pointers to structures (bands in which the flux is given)"
     PSM_INFO_MESSAGE, "set 'whichsrc' to an array of indices to observe only a fraction of the sources"
     PSM_INFO_MESSAGE, "input keyword 'name' is the catalogue name (default empty string)"
     PSM_INFO_MESSAGE, "set 'id' to the id key of the catalogue (default: generated by the code)"
     PSM_INFO_MESSAGE, "set /polar for a polarised catalogue, /nopolar for an unpolarised catalogue"
     PSM_INFO_MESSAGE, "if none is set, the polarisation status is read from the model"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  IF model_cat.polarised THEN BEGIN
     IF NOT KEYWORD_SET(nopolar) THEN BEGIN
        ispol = 1B
     ENDIF ELSE BEGIN
        ispol = 0B
     ENDELSE
  ENDIF ELSE BEGIN
     IF NOT KEYWORD_SET(polar) THEN BEGIN
        ispol = 0B
     ENDIF ELSE BEGIN
        ispol = 1B
     ENDELSE
  ENDELSE

  IF NOT KEYWORD_SET(whichsrc) THEN BEGIN

     data_cat = CREATE_PS_DATA_CAT(model_cat.nsources, band_list, name=name, id=id, polarised=ispol)
     ok = EXECUTE("data_cat.name = model_cat.name")
     FOR i=1, data_cat.nobs DO BEGIN
        band = band_list[i-1]

        *data_cat.objectname = *model_cat.objectname
        *data_cat.lon.value = *model_cat.lon.value
        data_cat.lon.unit = model_cat.lon.unit
        data_cat.lon.info = model_cat.lon.info
        *data_cat.lat.value = *model_cat.lat.value
        data_cat.lat.unit = model_cat.lat.unit
        data_cat.lat.info = model_cat.lat.info

        OBSERVE_MODEL_CAT, model_cat, band, result, nopolar=nopolar, polar=polar
        ok = EXECUTE("*data_cat.obs"+NUMBER2STRING(i)+".iflux = result[*,0]")
        ok = EXECUTE("data_cat.obs"+NUMBER2STRING(i)+".unit = 'mJy'")
        IF model_cat.polarised THEN IF ispol THEN BEGIN
           ok = EXECUTE("*data_cat.obs"+NUMBER2STRING(i)+".qflux = result[*,1]")
           ok = EXECUTE("*data_cat.obs"+NUMBER2STRING(i)+".uflux = result[*,2]")
        ENDIF
     ENDFOR

  ENDIF ELSE BEGIN

     data_cat = CREATE_PS_DATA_CAT(N_ELEMENTS(whichsrc), band_list, name=name, id=id, polarised=ispol)
     ok = EXECUTE("data_cat.name = model_cat.name")
     FOR i=1, data_cat.nobs DO BEGIN
        band = band_list[i-1]

        *data_cat.objectname = (*model_cat.objectname)[whichsrc]
        *data_cat.lon.value = (*model_cat.lon.value)[whichsrc]
        data_cat.lon.unit = model_cat.lon.unit[whichsrc]
        data_cat.lon.info = model_cat.lon.info[whichsrc]
        *data_cat.lat.value = (*model_cat.lat.value)[whichsrc]
        data_cat.lat.unit = model_cat.lat.unit[whichsrc]
        data_cat.lat.info = model_cat.lat.info[whichsrc]

        OBSERVE_MODEL_CAT, model_cat, band, result, nopolar=nopolar, polar=polar, whichsrc=whichsrc
        ok = EXECUTE("*data_cat.obs"+NUMBER2STRING(i)+".iflux = result[*,0]")
        ok = EXECUTE("data_cat.obs"+NUMBER2STRING(i)+".unit = 'mJy'")
        IF model_cat.polarised THEN IF ispol THEN BEGIN
           ok = EXECUTE("*data_cat.obs"+NUMBER2STRING(i)+".qflux = result[*,1]")
           ok = EXECUTE("*data_cat.obs"+NUMBER2STRING(i)+".uflux = result[*,2]")
        ENDIF
     ENDFOR
     

  ENDELSE

  RETURN, data_cat
END
