; Copyright 2009 Jacques Delabrouille, CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Function to create a catalogue of sources (with data in a set of bands), with its header
; 
; @categories ps
; 
; @param name_cat {in}{required}{type=string} Name of the catalogue
; @param nsources {in}{required}{type=long64} Number of sources in the catalogue 
; @param band {in}{required}{type=array of string} Bands in which the flux is given 
; @keyword polarised {in}{optional}{type=boolean} Set this keyword to make a catalogue of polarised sources (default = not polarised)
; @keyword usenan {in}{optional}{type=boolean} Set this keyword to use IEEE 'NaN' and 'Inf' as initialisation values (default = use 0 values)
; @keyword psm_hdr {out}{optional}{type=structure} PSM Header of the catalogue
;
; @history <p>30/03/2009: First version, Jacques Delabrouille</p>
; @history <p>17/04/2009: polar added in the header, Jacques Delabrouille</p>
; @history <p>18/04/2009: full set of fields and units now in the header, Jacques Delabrouille</p>
; @copyright Jacques Delabrouille, CNRS (2009)
;-
FUNCTION CREATE_OLD_PS_DATA_CAT, name_cat, nsources, band, polarised=polarised, usenan=usenan, psm_hdr=psm_hdr

ndata = N_ELEMENTS(band)


IF NOT KEYWORD_SET(usenan) THEN BEGIN
   db = 0d0 
   fla = FLTARR(ndata)
ENDIF ELSE BEGIN
   db = DOUBLE('NaN')
   fla = REPLICATE(FLOAT('NaN'),ndata)
ENDELSE


  IF NOT KEYWORD_SET(polarised) THEN BEGIN
     source = { name:'', glon_rad:db, glat_rad:db, iflux:fla, ierr:fla}
     units = ['unitless','rad','rad','unknown','unknown']
  ENDIF ELSE BEGIN
     source = { name:'', glon_rad:db, glat_rad:db, iflux:fla, ierr:fla, qflux:fla, qerr:fla, uflux:fla, uerr:fla}
     units = ['unitless','rad','rad','unknown','unknown','unknown','unknown','unknown','unknown']
  ENDELSE

  fields = TAG_NAMES(source)
  IF NOT KEYWORD_SET(polarised) THEN polar=0B ELSE polar=1B
 
  psm_hdr = {datatype:'PS_DATA_CAT',cat_name:name_cat,nsources:LONG64(nsources),ndata:FIX(ndata),band:band,polar:BYTE(polar),fields:fields,units:units}


  RETURN, REPLICATE(source,nsources)

END


  
  

