; Copyright 2009 Jacques Delabrouille, CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Software to write a PSM point source model catalogue in the appropriate fits file format
;
; @categories ps
;
; @param filename {in}{required}{type=string} Name of the file to be written
; @param cat {in}{required}{type=array of structures} The catalogue to be written
; @keyword psm_hdr {in}{required}{type=structure} The PSM header of the catalogue
; @uses NUMBER2STRING (PSM), MWRFITS, HEADFITS, MODFITS, SXADDPAR, SXPAR (ASTRON)
; @author Jacques Delabrouille
; @history <p>15/04/2009: First version, Jacques Delabrouille</p>
; @history <p>28/04/2009: Minor reorganisation of the PSM header, Jacques Delabrouille</p>
; @copyright Jacques Delabrouille, CNRS, 2009
;-
PRO WRITE_OLD_PS_MODEL_CAT, filename, cat, psm_hdr=psm_hdr


  psm_datatype = psm_hdr.datatype
  IF psm_datatype NE 'PS_MODEL_CAT' THEN MESSAGE, 'DATATYPE IS NOT WHAT EXPECTED'

; Remove existing file if any, write catalog
  FILE_DELETE, filename, /allow_nonexistent
  MWRFITS, cat, filename, /CREATE

; Read the header of the FITS extension
  hd=HEADFITS(filename, exten=1)

; Prepare the inclusion of a PSM header at the end
  array = ['COMMENT','COMMENT  *** PSM header ***','COMMENT']
  ln = STRLEN(hd[0])
  array = PSM_STRING_TO_LENGTH(array,ln)
  nhd = N_ELEMENTS(hd)
  hd = [hd[0:nhd-2],array,hd[nhd-1]]

; Include the psm_header information in the catalogue header
  tagnames = TAG_NAMES(psm_hdr)
  ntags = N_ELEMENTS(tagnames)
  FOR i=0, ntags-1 DO BEGIN
     CASE tagnames[i] OF
        'FIELD':                ;do nothing (already written in TTYPE1, TTYPE2, ...)
        'INFO':                 ;do nothing (see below)
        'UNITS':                ;do nothing (see below)
        ELSE: BEGIN
           IF NOT ISARRAY(psm_hdr.(i)) THEN BEGIN
              cmd = "SXADDPAR, hd, '"+STRTRIM(tagnames[i],2)+"', '"+STRTRIM(psm_hdr.(i),2)+"'"
              ok=EXECUTE(cmd)
           ENDIF ELSE BEGIN
              nel = N_ELEMENTS(psm_hdr.(i))
              FOR j=0, nel-1 DO BEGIN
                 type = SIZE((psm_hdr.(i))[j],/type)
                 IF type EQ 1 THEN BEGIN
                                ; problem of conversion of byte to string in execute command
                    value = FIX((psm_hdr.(i))[j])
                    res=EXECUTE("SXADDPAR, hd, '"+tagnames[i]+NUMBER2STRING(j+1)+"', value")
                 ENDIF ELSE BEGIN
                    res=EXECUTE("SXADDPAR, hd, '"+tagnames[i]+NUMBER2STRING(j+1)+"', psm_hdr."+tagnames[i]+'['+NUMBER2STRING(j)+']')
                 ENDELSE
              ENDFOR           
           ENDELSE
        ENDELSE
     ENDCASE
  ENDFOR

; Write units for all fields (TUNIT)
  nfields = SXPAR(hd, 'TFIELDS')
  IF NFIELDS EQ N_ELEMENTS(psm_hdr.units) THEN BEGIN
     FOR i=1, NFIELDS DO BEGIN
        name = 'TUNIT'+NUMBER2STRING(i)
        after = 'TFORM'+NUMBER2STRING(i)
        SXADDPAR, hd, name, STRTRIM(psm_hdr.units[i-1],2), after=after
     ENDFOR
  ENDIF

; Describe the fields in the catalogue header : name+units
  tags = tag_names(psm_hdr)
  wh = where(tags EQ 'INFO',nwh)
  IF nwh EQ 1 THEN BEGIN
     ninfo = N_ELEMENTS(psm_hdr.info)
     FOR i=0, ninfo-1 DO BEGIN
        cmd = "SXADDPAR, hd, 'TTYPE"+STRTRIM(STRING(i+1),1)+"', SXPAR(hd,'TTYPE"+STRTRIM(STRING(i+1),1)+"'), '"+STRTRIM((psm_hdr.info)[i],2)+" ["+STRTRIM(STRING((psm_hdr.units)[i]),2)+"]'"
        ok=EXECUTE(cmd)
     ENDFOR
  ENDIF
  MODFITS, filename, 0, hd, exten=1

END
