PRO PLANCK_POINTING, ringpointing, after_date=after_date, before_date=before_date, help=help

  IF (KEYWORD_SET(help) NE 0 OR (N_PARAMS() LT 1)) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PLANCK_POINTING: procedure to load ring pointing information for Planck'
     PSM_INFO_MESSAGE, 'SYNTAX: PLANCK_POINTING, ringpointing, after_date=, before_date='
     PSM_INFO_MESSAGE, "'after_date' default value is '2009-14-07 12:00:00'"
     PSM_INFO_MESSAGE, "'before_date' default value is '2011-14-07 12:00:00'"
     PSM_INFO_MESSAGE, "'ringpointing' is the output ring pointing structure."
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  LATEST_POINTING_FILE, file, type=type

  IF KEYWORD_SET(after_date) EQ 0 THEN after_date = '2009-08-14 12:00:00' ; 14 august 2009
  IF KEYWORD_SET(before_date) EQ 0 THEN before_date = '2011-08-14 12:00:00' ; 14 august 2011
 
  CASE STRUPCASE(type) OF
     'PPL': BEGIN
        STOP                    ; Requires putting together different orbit files
        READ_PPL_FILE, ringpointing, file=file, after_date=after_date, before_date=before_date
     END
     'PLAN': BEGIN
        ok = TEST_PSM_DATA(file, /get)
        IF NOT ok THEN MESSAGE, 'Input plan file missing'
        READCOL, file, elon, elat, t0, duration, format='D,D,A,L', skipline=2       
        date = PSM_STR_REPLACE(t0, 'T', ' ')
        date = PSM_STR_REPLACE(date, 'Z', '')    
        nd = N_ELEMENTS(date)
        modified_jd = DBLARR(nd)
        FOR i=0L, nd-1 DO modified_jd[i] = DATE_CONV(date[i],'MODIFIED')
        after_jd = DATE_CONV(after_date,'MODIFIED')
        before_jd = DATE_CONV(before_date,'MODIFIED')
        wh = WHERE((modified_jd GT after_jd) AND (modified_jd+duration/24d/3600d LT before_jd), nwh)
        IF nwh GT 0 THEN BEGIN
           dtor = !DPI/180d0
           ring = {ringnum:0L, elon_rad:0d0, elat_rad:0d0, glon_rad:0d0, glat_rad:0d0, date:'', mjd:0d0, duration:0d0}
           ringpointing = REPLICATE(ring,nwh)
           ringpointing.ringnum = LINDGEN(nwh)
           ringpointing.elon_rad = elon[wh]*dtor
           ringpointing.elat_rad = elat[wh]*dtor
           ringpointing.date = date[wh]
           ringpointing.mjd = modified_jd[wh]
           ringpointing.duration = duration[wh]
           EULER, ringpointing.elon_rad, ringpointing.elat_rad, glon, glat, 5, /radian
           ringpointing.glon_rad = glon
           ringpointing.glat_rad = glat
        ENDIF ELSE MESSAGE, 'No ring begins after '+after_date + ' and ends before '+before_date
     END
  ENDCASE

END
