FUNCTION CHECK_FITSHDR, hdr_in

; Check that the header is defined

  IF UNDEFINED(hdr_in) THEN BEGIN
     PSM_INFO_MESSAGE, 'CHECK_FITSHDR: Input header undefined'
     IF PSM_CAREFULNESS() GE 2 THEN BEGIN
        PSM_INFO_MESSAGE, 'Stopping the processing, type .c to continue'
        STOP
        RETURN, 0
     ENDIF
  ENDIF

; Check that the header is an array of string

  s = SIZE(hdr_in,/type)
  IF s NE 7 THEN BEGIN
     STOP
  ENDIF

; Check that the header comprises an end line at the end

  hdr = hdr_in

  wh = WHERE(STRMID(hdr,0,8) EQ 'END     ',nwh)
  endlinenotok = 0
  IF nwh NE 1 THEN endlinenotok = 1 ELSE IF wh[0] NE N_ELEMENTS(hdr)-1 THEN endlinenotok = 1

  IF endlinenotok THEN BEGIN
     IF PSM_CAREFULNESS() GE 3 THEN BEGIN
        PSM_INFO_MESSAGE, 'CHECK_FITSHDR: Warning, input header does not contain a single END line at the end'        
        PSM_INFO_MESSAGE, 'Stopping the processing, type .c to add an END line and continue'
        STOP
        RETURN, 0
     ENDIF
     hdr = [hdr, PSM_STRING_TO_LENGTH('END',80,character=' ')]
  ENDIF


; Check that the header does not contain multiply defined keywords

  nel = N_ELEMENTS(hdr)
  ok = 1

  listpar = ['']

  ;; Check for duplicates
  FOR i=0, nel-1 DO BEGIN
     iscomment = (STRMID(hdr[i],0,8) EQ 'COMMENT ') 
     isend =  (STRMID(hdr[i],0,8) EQ 'END     ') 
     ishistory =  (STRMID(hdr[i],0,8) EQ 'HISTORY ')
     isreference =  (STRMID(hdr[i],0,8) EQ 'REFERENC')
     IF NOT (iscomment OR isend OR ishistory OR isreference) THEN BEGIN
        parname = STRTRIM(STRMID(hdr[i],0,8), 2)
        ok = EXECUTE('v = SXPAR(hdr, parname, count=c, /silent)')
        IF NOT ok THEN STOP
        IF c GT 1 THEN BEGIN
           listpar = [listpar,parname]
           ok = 0
        ENDIF
     ENDIF
  ENDFOR

  listpar = listpar(PSM_UNIQ(listpar))
  np = N_ELEMENTS(listpar)
  IF np GE 2 THEN FOR p=1, np-1 DO PSM_DEBUG_MESSAGE, 'Keyword '+listpar[p]+ ' occurs more than once in FITS header'

  RETURN, ok

END
