FUNCTION PSM_BAND_HDR, band, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSM_BAND_HDR: function to generate PSM band header information array'
     PSM_INFO_MESSAGE, 'SYNTAX: result = PSM_BAND_HDR(band)'
     PSM_INFO_MESSAGE, "band is a structure describing the instrumental band of the observation,"
     PSM_INFO_MESSAGE, "or optionally the value of the central frequency of a 'DIRAC' band"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  hd = [PSMHDRBLOCK_HEADLINE('band')] & hd=[hd]

  IF DEFINED(band) THEN BEGIN
     bandshptype = SIZE(band,/type)
     IF bandshptype LE 5 THEN BEGIN
        bdshp = 'DIRAC'
        bdnuc = band
     ENDIF
     IF bandshptype EQ 8 THEN BEGIN
        bdshp = band.shape
        IF STRUPCASE(bdshp) EQ 'DIRAC' THEN bdnuc = band.nu_c
     ENDIF
     SXADDPAR, hd, 'BD_SHP', bdshp, "Band shape (e.g. 'DIRAC', 'TOPHAT', 'INSTR')"
     CASE bdshp OF
        'DIRAC': BEGIN
           SXADDPAR, hd, 'BD_CNU', bdnuc, "Band central frequency"
        END
        'TOPHAT': BEGIN
           SXADDPAR, hd, 'BD_LNU', band.nu_min, "Band lower frequency"
           SXADDPAR, hd, 'BD_UNU', band.nu_max, "Band upper frequency"
        END
        'TABLE': BEGIN
           SXADDPAR, hd, 'BD_NPTS', N_ELEMENTS(band.nu), "Number of points of tabulated band"
           PSM_DEBUG_MESSAGE, 'Implement Table bands', routine='PSM_BAND_HDR'
           STOP
        END
        'INSTR': BEGIN
           SXADDPAR, hd, 'BD_INSTR', band.instr, "Instrument for the specified band"
           SXADDPAR, hd, 'BD_VERS', band.version, "Version for the specified band"
           SXADDPAR, hd, 'BD_CHAN', band.channel, "Channel for the specified band"
        END
        ELSE: MESSAGE, 'Band type not implemented'
     ENDCASE

  ENDIF ELSE BEGIN
     bdshp = 'NO_BAND'
     SXADDPAR, hd, 'BD_SHP', bdshp
  ENDELSE

  hd = hd[0:N_ELEMENTS(hd)-2]

RETURN, hd

END
