PRO READ_MAP, fitsfile, map, signal=signal, hdr=hdr, ordering=ordering, pointer=pointer, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'READ_MAP: procedure to read a map'
     PSM_INFO_MESSAGE, 'SYNTAX: READ_MAP, fitsfile, map, signal=, hdr=, ordering=, /pointer'
     PSM_INFO_MESSAGE, "fitsfile is a file to be read"
     PSM_INFO_MESSAGE, "map is the output map array (or pointer, see below)"
     PSM_INFO_MESSAGE, "signal is the signal to be read: can be 'T' or 'TQU' or 'ALL'"
     PSM_INFO_MESSAGE, "hdr is the map header contained in the fitsfile"
     PSM_INFO_MESSAGE, "ordering is the healpix ordering of the map in the fitsfile"
     PSM_INFO_MESSAGE, "set /pointer to return a pointer rather than a memory array"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  PSM_DEBUG_MESSAGE, 'To be implemented seriously', routine='READ_MAP', infolimit=2, stoplimit=3

  ok = FILE_TEST(fitsfile)
  IF NOT OK THEN BEGIN
     PSM_INFO_MESSAGE, 'Cannot find input file: '+fitsfile
     GOTO, closing
  ENDIF

  IF KEYWORD_SET(signal) NE 0 THEN BEGIN
     CASE STRUPCASE(signal) OF 
        'T': read_polar = 0
        'TQU': BEGIN
           IF ISPOLARISED(fitsfile,/map) THEN read_polar = 1 ELSE BEGIN
              MESSAGE, 'Cannot read TQU from file ' + fitsfile +': No polarisation available'
           ENDELSE
        END
        'ALL': read_polar = ISPOLARISED(fitsfile,/map)
     ENDCASE
  ENDIF ELSE BEGIN
     read_polar = ISPOLARISED(fitsfile,/map)
  ENDELSE

  IF read_polar THEN BEGIN
     READ_TQU, fitsfile, map 
  ENDIF ELSE BEGIN
     READ_FITS_MAP, fitsfile, map
     map = map[*,0]
  ENDELSE
  IF KEYWORD_SET(pointer) NE 0 THEN map = PTR_NEW(map,/no_copy)

;  IF (KEYWORD_SET(hdr) NE 0) OR (KEYWORD_SET(ordering) NE 0) THEN BEGIN
     hdr = HEADFITS(fitsfile,exten=1)
     ordering = STRTRIM(SXPAR(hdr, 'ORDERING'),2)
;  ENDIF

closing:
END
