FUNCTION GETPSMDIR, whichdir, psm_output_dir=psm_output_dir, exists=exists, create=create, help=help

  IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() EQ 0)) THEN BEGIN
     helpme:
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'GETPSMDIR: function which returns the path to a specific PSM directory'
     PSM_INFO_MESSAGE, 'SYNTAX: result = GETPSMDIR(whichdir, psm_output_dir=, exists=, create=)'
     PSM_INFO_MESSAGE, "'whichdir' is a directory name among the following:"
     PSM_INFO_MESSAGE, "'psmroot', 'psmsoft', 'psmdata', 'output', 'private', 'info', 'ancillary', "
     PSM_INFO_MESSAGE, "'components', 'cosmo', 'skyinbands', 'observations', 'figures',"
     PSM_INFO_MESSAGE, "'dipole', 'cmb', 'co', 'firb', 'freefree', 'ps', 'synchrotron', 'sz', 'thermaldust', etc."
     PSM_INFO_MESSAGE, "When called outside of a PSM run, specify the output directory with 'psm_output_dir'."
     PSM_INFO_MESSAGE, "'exists' is an ouptput keyword, returning 1 if the directory exists and 0 otherwise."
     PSM_INFO_MESSAGE, "set '/create' to create the directory if it does not exist."
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  ON_ERROR, 2

  COMMON psm, param

  ;; Cases which do not require output to be defined
  ;;------------------------------------------------
  CASE STRLOWCASE(whichdir) OF
     'psmroot': direc = FIX_SEPARATOR(!PSMROOT)
     'psmsoft': direc = FIX_SEPARATOR(!PSMROOT)+'Soft/'
     'psmdata': direc = FIX_SEPARATOR(!PSMROOT)+'Data/'
     'dx8':     direc = FIX_SEPARATOR(!PSMROOT)+'Data/ancillary/observations/Planck/DX8/'
     ELSE: GOTO, continue_getpsmdir
  ENDCASE

  RETURN, direc
  
  continue_getpsmdir:

  IF DEFINED(param) THEN BEGIN
     IF KEYWORD_SET(psm_output_dir) NE 0 THEN outdir = FIX_SEPARATOR(psm_output_dir) $
     ELSE outdir = FIX_SEPARATOR(param.run.output_directory)
  ENDIF ELSE BEGIN
     IF PSM_CAREFULNESS() LT 2 THEN BEGIN
        IF KEYWORD_SET(psm_output_dir) NE 0 THEN BEGIN
           outdir = FIX_SEPARATOR(psm_output_dir)
        ENDIF ELSE BEGIN
           MESSAGE, "PSM output dir not defined -- no directory name for '"+whichdir+"'", /info
           RETURN, ''
        ENDELSE
     ENDIF
  ENDELSE
  
  ;; Cases which do not require param to be defined
  ;;------------------------------------------------
  CASE STRLOWCASE(whichdir) OF
     'psmroot': direc = FIX_SEPARATOR(!PSMROOT)
     'psmsoft': direc = FIX_SEPARATOR(!PSMROOT)+'Soft/'
     'psmdata': direc = FIX_SEPARATOR(!PSMROOT)+'Data/'
     'output':  direc = outdir
     'private': direc = outdir + FIX_SEPARATOR('.psm')
     ELSE: ;do nothing
  ENDCASE

  IF DEFINED(direc) THEN BEGIN
     exists = FILE_TEST(direc, /directory)
     IF KEYWORD_SET(create) NE 0 THEN IF NOT exists THEN FILE_MKDIR, direc
     exists = FILE_TEST(direc, /directory)
     RETURN, direc
  ENDIF

  ;; Cases for which param is needed
  ;;---------------------------------

  IF NOT DEFINED(param) THEN BEGIN
     RESTORE_PSM_PARAM, psm_output_dir=psm_output_dir
  ENDIF

  IF DEFINED(param) THEN BEGIN
     compdir = outdir + FIX_SEPARATOR(param.names.component_basename)
     ancildir = outdir + FIX_SEPARATOR(param.names.ancillary_basename)
     obsdir = outdir + FIX_SEPARATOR(param.names.observation_basename)
  ENDIF

  CASE STRLOWCASE(whichdir) OF
     'ancillary': direc = ancildir
     'cmb': direc = compdir + FIX_SEPARATOR(param.names.cmb_basename)
     'co': direc = compdir + FIX_SEPARATOR(param.names.co_basename)
     'components': direc = compdir
     'cosmo': direc = outdir + FIX_SEPARATOR('cosmo')
     'dipole': direc = compdir + FIX_SEPARATOR(param.names.dipole_basename)
     'faintercscps': direc = GETPSMDIR('ps')
     'faintirps': direc = GETPSMDIR('ps')
     'faintradiops': direc = GETPSMDIR('ps')
     'faintuchii': direc = GETPSMDIR('ps')
     'faintwmapps': direc = GETPSMDIR('ps')
     'figures': direc = outdir + FIX_SEPARATOR('figures')
     'firb': direc = compdir + FIX_SEPARATOR(param.names.firb_basename)
     'freefree': direc = compdir + FIX_SEPARATOR(param.names.freefree_basename)
     'info': direc = outdir + FIX_SEPARATOR(param.names.info_basename)
     'irps': direc = GETPSMDIR('ps')
     'kineticsz': direc = compdir + FIX_SEPARATOR(param.names.sz_basename)
     'observations': direc = obsdir     
     'ps': direc = compdir + FIX_SEPARATOR(param.names.ps_basename)
     'radiops': direc = GETPSMDIR('ps')
     'skyinbands': direc = outdir + FIX_SEPARATOR('skyinbands')
     'spindust': direc = compdir + FIX_SEPARATOR(param.names.spindust_basename)
     'strongercscps': direc = GETPSMDIR('ps')
     'strongirps': direc = GETPSMDIR('ps')
     'strongradiops': direc = GETPSMDIR('ps')
     'stronguchii': direc = GETPSMDIR('ps')
     'strongwmapps': direc = GETPSMDIR('ps')
     'synchrotron': direc = compdir + FIX_SEPARATOR(param.names.synchrotron_basename)
     'sz': direc = compdir + FIX_SEPARATOR(param.names.sz_basename)
     'thermaldust': direc = compdir + FIX_SEPARATOR(param.names.thermaldust_basename)
     'thermalsz': direc = compdir + FIX_SEPARATOR(param.names.sz_basename)
     'uchii': direc = GETPSMDIR('ps')
     'visu': direc = outdir + FIX_SEPARATOR(param.names.visu_basename)
     'zle': direc = compdir + FIX_SEPARATOR(param.names.zle_basename)
     ELSE: MESSAGE, 'Case not implemented'
  ENDCASE

  exists = FILE_TEST(direc, /directory)
  IF KEYWORD_SET(create) NE 0 THEN IF NOT exists THEN FILE_MKDIR, direc
  exists = FILE_TEST(direc, /directory)

  RETURN, direc

END
