FUNCTION READ_PSM_PARAM, table, carefulness=carefulness, verbosity=verbosity, debug=debug

;;-----------------------------------------------------------------------------------
;;    PSM parameters
;;-----------------------------------------------------------------------------------

  ;; Global parameters (needed for historical reasons)
  ;;---------------------------------------------------
  gp = { $
       precision:'single', $                                     ; {'single','double'}
       fields:'T', $                                             ;
       sky_resolution:15., $                                     ;
       lmax:1536, $                                              ;
       seed:0L, $
       getdata:1}
  FILL_STRUCT, table, gp
  dummy = {sky_lmax:1536}
  FILL_STRUCT, table, dummy
  gp.lmax = dummy.sky_lmax

  todo = {sky_task:'new', $
          dipole_model:'', $
          cmb_model:'', $
          gal_model:'', $
          sz_model:'', $
          ps_model:'', $
          firb_model:'', $
          zle_model:'', $
          write_ancillary:'no', $
          obs_task:'none'}
  FILL_STRUCT, table, todo
          
  existing = 'existing'

  tasks = {sky_task:todo.sky_task, $
           update_dipole:((todo.dipole_model NE existing) OR (STRLOWCASE(todo.sky_task) EQ 'new')), $
           update_cmb:((todo.cmb_model NE existing) OR (STRLOWCASE(todo.sky_task) EQ 'new')), $
           update_gal:((todo.gal_model NE existing) OR (STRLOWCASE(todo.sky_task) EQ 'new')), $
           update_sz:((todo.sz_model NE existing) OR (STRLOWCASE(todo.sky_task) EQ 'new')), $
           update_ps:((todo.ps_model NE existing) OR (STRLOWCASE(todo.sky_task) EQ 'new')), $
           update_firb:((todo.firb_model NE existing) OR (STRLOWCASE(todo.sky_task) EQ 'new')), $
           update_zle:((todo.zle_model NE existing) OR (STRLOWCASE(todo.sky_task) EQ 'new')), $
           write_ancillary:todo.write_ancillary, $
           obs_task:todo.obs_task $
          }


  ;; PSM run info parameters
  ;;-------------------------
   
  PSM_ID_KEYGEN, psm_id                                          ; Generate an identification for this run of the PSM

  outdir = FIX_SEPARATOR(!psmroot)+'PSM_OUTPUT'

  run_info = { $                                                 ; Parameters for the behaviour of the PSM run (mostly fixed)
             output_directory:outdir, $                          ; Path towards the directory where to store results
             clear_all:'no', $                                   ; Whether output subdirectories are emptied from existing files
             carefulness:carefulness, $                          ; {0,1,2} Sets the amount of carefulness during the run
             verbosity:verbosity, $                              ; {0,1,2} Sets the amount of info given while the PSM runs
             debug:debug, $
             visu:2, $                                           ; {0,1,2} Sets the amount of display (maps, plots) given on screen while the PSM runs
             output_visu:'ps', $                                 ; 'screen', 'png', 'ps'
             ;; The remaining fields are not set by the PSM user --------------------------------------------------------
             psm_version:PSM_VERSION(), $                        ; Version
             psm_id:psm_id, $                                    ; Identification key for the current PSM run
             psm_minfreq:1e9, $                                  ; Frequency range of PSM validity
             psm_maxfreq:3e12, $                                 ; Frequency range of PSM validity
             run_start_date:'', $                                ;
             psm_cite_file:'', $                                 ; File for citations
             psm_biblio_file:'' $                                ; File for bibliographic information
             }
  tags = ['OUTPUT_DIRECTORY','CLEAR_ALL','VISU','OUTPUT_VISU','RUN_START_DATE'] ;
  FILL_STRUCT, table, run_info, tags=tags, carefulness=carefulness

  run_info.output_directory = FIX_SEPARATOR(run_info.output_directory)
  run_info.psm_biblio_file = run_info.output_directory+'psminfo/psm_bibliography.txt'
  run_info.psm_cite_file = run_info.output_directory+'psminfo/psm_citations.txt'

  ;; PSM names info parameters (centralised and hard-coded here, and fixed)
  ;;------------------------------------------------------------------------

  name_info = {$
              component_basename:'components', $                        ;
              observation_basename:'observations', $                    ;
              ;; dipole component info
              dipole_basename:'dipole', $                               ;
              dipole_alm_file:'dipole_alm.fits', $                      ;
              dipole_map_file:'dipole_map.fits', $                      ;
              ;; cmb component info
              cmb_basename:'cmb', $                                     ;
              cmb_alm_file:'cmb_alm.fits', $                            ;
              cmb_map_file:'cmb_map.fits', $                            ;
              cmb_cl_file:'cmb_cl.fits', $                              ;
              cmb_unlensed_alm_file: 'cmb_unlensed_alm.fits', $         ;
              cmb_unlensed_map_file: 'cmb_unlensed_map.fits', $         ;
              cmb_unlensed_cl_file: 'cmb_unlensed_cl.fits', $           ;
              cmb_lensed_alm_file: 'cmb_lensed_alm.fits', $             ;
              cmb_lensed_map_file: 'cmb_lensed_map.fits', $             ;
              cmb_lensed_cl_file: 'cmb_lensed_cl.fits', $               ;
              ;; lensing potential and matter power spectrum
              potential_alm_file: 'potential_alm.fits', $               ;
              potential_map_file: 'potential_map.fits', $               ;
              ;; galactic component info
              synchrotron_basename:'synchrotron', $                     ;
              freefree_basename:'freefree', $                           ;
              spindust_basename:'spindust', $                           ;
              thermaldust_basename:'thermaldust', $                     ;
              co_basename:'co', $                                       ;
                                ; synchrotron_map_file:'synchrotron_map.fits', $ 
                                ; freefree_map_file:'freefree_map.fits', $ 
                                ; spindust_map_file:'spindust_map.fits', $ 
                                ; thermaldust_map_file:'thermaldust_map.fits', $ 
              ;; sz component info
              sz_basename:'sz', $                                 ;
              thermalsz_basename:'thermalsz', $                   ;
              kineticsz_basename:'kineticsz', $                   ;
              polarsz_basename:'polarsz', $                       ;
              thermalsz_alm_file:'thermalsz_alm.fits', $ 
              thermalsz_map_file:'thermalsz_map.fits', $ 
              kineticsz_alm_file:'kineticsz_alm.fits', $ 
              kineticsz_map_file:'kineticsz_map.fits', $ 
              ;; point source component info
              ps_basename:'ps', $                                 ;
              irps_basename:'irps', $                             ;
              radiops_basename:'radiops', $                       ;
              wmapps_basename:'wmapps', $                         ;
              ercscps_basename:'ercscps', $                       ;
              uchii_basename:'uchii', $                           ;
              coldcore_basename:'coldcore', $                     ;
              firb_basename:'firb', $                             ;
              ;; Zodiacal light info
              zle_basename:'zle', $                               ;
              ;; ancillary data info
              ancillary_basename:'ancillary', $                   ;
              radiops_ancil_file:'radiops_ancil_cat.fits', $      ;
              irps_ancil_file:'irps_ancil_cat.fits', $            ;
              ;; other psm base names
              info_basename:'psminfo', $                          ;
              visu_basename:'figures'  $                          ;
             }

  ;; Set final psm parameter structure
  ;;-----------------------------------
  parameters = {global:gp, tasks:tasks, run:run_info, names:name_info}
  RETURN, parameters

END


