PRO PSM_SKYOBS, param_table, check_param=check_param

  COMMON psm, param
  COMMON sky, model

  IF OBS_TASK() NE 'none' THEN BEGIN

     PSM_INFO_TITLE, "OBSERVING THE SKY EMISSION"
     
     ;; Read observation parameters
     ;;-----------------------------
     COMMON obs, obser
     obser = READ_OBS_PARAM(param_table, carefulness=PSM_CAREFULNESS(), verbosity=PSM_VERBOSITY())
     IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_SUBTITLE, "OBSERVATION PARAMETERS SUCCESFULLY LOADED"
     IF KEYWORD_SET(check_param) NE 0 THEN BEGIN
        PRTSTRUCT, obser, name='OBSER'
        GOTO, closing
     ENDIF ELSE BEGIN
        IF PSM_VERBOSITY() GE 2 THEN PRTSTRUCT, obser, name='OBSER'
     ENDELSE

     ;; Restore sky model
     ;;------------------
     RESTORE_SKY_MODEL, model, output_dir=use_output_dir
     IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_SUBTITLE, "MODEL PARAMETERS SUCCESFULLY RELOADED FOR SKY OBSERVATION"
     IF PSM_VERBOSITY() GE 2 THEN PRTSTRUCT, model, name='MODEL'

     CASE obser.what.what_obs OF 
        'bandinteg': makobs = 0B
        'fullobs': makobs = 1B
     ENDCASE

     ;; Observe the sky
     ;;-----------------
     wh = WHERE(obser.inst NE '', nwh)

     IF nwh NE 0 THEN BEGIN

        ;; Make main sky-in-bands directory 
        ;;----------------------------------
        dir = GETPSMDIR('skyinbands')
        exists = FILE_TEST(dir,/directory)
        
        CASE OBS_TASK() OF
           'new': BEGIN
              IF exists THEN BEGIN
                 IF PSM_CAREFULNESS() GE 2 THEN BEGIN
                    PSM_INFO_MESSAGE, 'Directory for band-integrated sky already exists. Stopping the PSM run'
                    PSM_INFO_MESSAGE, 'Type .c to erase '+dir+' and continue'
                 ENDIF
                 FILE_DELETE, dir, /recursive
              ENDIF ELSE FILE_MKDIR, GETPSMDIR('skyinbands')
           END
           'update': BEGIN
              IF exists THEN BEGIN
                 ;; do nothing
              ENDIF ELSE FILE_MKDIR, GETPSMDIR('skyinbands')
           END
        ENDCASE

        ;; Make main observation directory 
        ;;---------------------------------
        IF makobs THEN BEGIN  
           IF OBS_TASK() EQ 'new' THEN BEGIN
              obsdir = GETPSMDIR('observations')
              exists = FILE_TEST(obsdir,/directory)
              IF exists THEN BEGIN
                 IF PSM_CAREFULNESS() GE 2 THEN BEGIN
                    PSM_INFO_MESSAGE, 'Observations directory already exists. Stopping the PSM run'
                    PSM_INFO_MESSAGE, 'Type .c to erase '+obsdir+' and continue'
                 ENDIF
                 FILE_DELETE, obsdir, /recursive
              ENDIF ELSE FILE_MKDIR, GETPSMDIR('observations')
           ENDIF
        ENDIF
        
        ;; What to group
        ;;---------------
        CASE obser.what.group_galaxy OF
           'yes': group_gal=1
           'no': group_gal=0
        ENDCASE
        CASE obser.what.group_strong_ps OF
           'yes': group_strong_ps=1
           'no': group_strong_ps=0
        ENDCASE
        CASE obser.what.group_faint_ps OF
           'yes': group_faint_ps=1
           'no': group_faint_ps=0
        ENDCASE
        
        ;; Observe the sky in the list of instrumental bands
        ;;---------------------------------------------------
        instrument = ALL_INSTRUMENTS()
        INDEX_BANDS, instrument, ibands
        
        ind = PSM_UNIQ(ibands)
        nb = N_ELEMENTS(ind)
        bandlist = PTRARR(nb,/allocate_heap)
        FOR b=0,nb-1 DO bandlist[b] = PTR_NEW(CHANNELPARAM(instrument,'band',ind[b]+1))
        bandnames = STRARR(nb)
        FOR b=0,nb-1 DO bandnames[b] = CHANNELPARAM(instrument,'name',ind[b]+1)
        banddirnames = STRARR(nb)
        FOR b=0,nb-1 DO banddirnames[b] = CHANNELPARAM(instrument,'inst',ind[b]+1) + '/' + CHANNELPARAM(instrument,'name',ind[b]+1)
        bandunits = STRARR(nb)
        FOR b=0,nb-1 DO bandunits[b] = CHANNELPARAM(instrument,'obs_units',ind[b]+1)

        SKY_IN_BANDS, bandlist=bandlist, banddirnames=banddirnames, bandnames=bandnames, bandunits=bandunits, $
                      group_gal=group_gal, group_strong_ps=group_strong_ps, group_faint_ps=group_faint_ps


        ;; Make observations
        ;;-------------------
        IF makobs THEN BEGIN

           groups = obser.groups
           OBSERVE_SKY, groups, instrument
        
        ENDIF 
     
     ENDIF ELSE BEGIN
        
        PSM_INFO_MESSAGE, 'No instrument defined for observing the sky'
        
     ENDELSE
     
     
  ENDIF
  
  closing:

END
