PRO PSM_SEEDINIT, main_seed=main_seed, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, ''
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSM_SEEDINIT: procedure to initialize seeds used in PSM simulations and software'
     PSM_INFO_MESSAGE, 'SYNTAX: PSM_SEEDINIT, main_seed='
     PSM_INFO_MESSAGE, 'Set main_seed to the value of the main seed (any positive long integer)'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF


  COMMON psm, param

  seed_exists = 0

  IF DEFINED(param) THEN BEGIN
     ;; Define a default main seed if param is defined
     IF NOT KEYWORD_SET(main_seed) THEN main_seed = param.global.seed 

     ;; Check whether a psm_seed status is saved in the psm private directory
     seed_exists = FILE_TEST(GETPSMDIR('private')+'psm_seeds.sav')
  ENDIF ELSE BEGIN
  ENDELSE

  ;; If seed exists, then restore it, and use it to define the psm_seed
  IF seed_exists THEN BEGIN
     RESTORE, GETPSMDIR('private')+'psm_seeds.sav'
     shiftseed = N_ELEMENTS(psm_seeds.chain_used)
     
     ;; Updating psm_seed: set values of unused chains to new value set by main_seed
     new_chain_value = ULONG(LINDGEN(shiftseed)) + ULONG(main_seed*shiftseed)
     wh_not_used = WHERE(psm_seeds.chain_used EQ 0, nwh)
     IF nwh NE 0 THEN psm_seeds.chain_value[wh_not_used] = new_chain_value[wh_not_used]

  ENDIF 

  IF NOT seed_exists THEN BEGIN
     shiftseed=1024

     ;; Create basic structure for seed info
     ;;--------------------------------------
     IF KEYWORD_SET(main_seed) EQ 0 THEN BEGIN
        IF DEFINED(param) THEN main_seed = param.global.seed ELSE main_seed=1      
     ENDIF
     current = {chain:0,name:'psm',value:ULONG(main_seed*shiftseed)}
     psm_seeds = {current:current,chain_used:BYTARR(shiftseed),chain_name:STRARR(shiftseed),chain_value:ULONG(LINDGEN(shiftseed)+current.value)}
     
     ;; Seeds chains reserved for PSM developer
     ;;-----------------------------------------
     
     psm_seeds.chain_used[0:255] = 1B
     
     ;; PSM main
     psm_seeds.chain_name[0] = 'psm'
     
     ;; CMB setup and pipe
     psm_seeds.chain_name[10] = 'cmb-setup'
     psm_seeds.chain_name[11] = 'cmb-pipe'
     psm_seeds.chain_name[12] = 'cmb-dipole'
     
     ;; Galaxy setup and galactic component pipes
     psm_seeds.chain_name[15] = 'galaxy-setup'
     psm_seeds.chain_name[16] = 'synchrotron-pipe'
     psm_seeds.chain_name[17] = 'freefree-pipe'
     psm_seeds.chain_name[18] = 'spindust-pipe'
     psm_seeds.chain_name[19] = 'thermaldust-pipe'
     psm_seeds.chain_name[20] = 'co-pipe'
     
     ;; SZ setup and galactic component pipes
     psm_seeds.chain_name[25] = 'sz-setup'
     psm_seeds.chain_name[26] = 'sz-pipe'
     
     ;; PS setup and ps component pipes
     psm_seeds.chain_name[30] = 'ps-setup'
     psm_seeds.chain_name[31] = 'ps-pipe-radio'
     psm_seeds.chain_name[32] = 'ps-pipe-ir'
     psm_seeds.chain_name[33] = 'ps-pipe-uchii'

     ;; FIRB
     psm_seeds.chain_name[40] = 'firb-pipe'
     
     ;; PSM_IDEAL INSTRUMENT AND NOISE
     psm_seeds.chain_name[60] = 'psm_ideal-noise'

     ;; WMAP INSTRUMENT AND NOISE
     psm_seeds.chain_name[70] = 'wmap-noise'

  ENDIF
    
     ;; Define the system variable
     ;;----------------------------
     DEFSYSV, '!psm_seeds', psm_seeds
     

END
