PRO GAUSSIAN_CMB, cmb

  COMMON psm, param
  COMMON sky, model

  cmbdir = GETPSMDIR('cmb')

  ;; Filenames for the theory alm and map 
  ;;--------------------------------------
  cmb_unlensedalm_file = cmbdir + param.names.cmb_unlensed_alm_file
  cmb_unlensedmap_file = cmbdir + param.names.cmb_unlensed_map_file
  cmb_lensedalm_file = cmbdir + param.names.cmb_lensed_alm_file
  cmb_lensedmap_file = cmbdir + param.names.cmb_lensed_map_file
  potential_alm_file = cmbdir + param.names.potential_alm_file 
  potential_map_file = cmbdir + param.names.potential_map_file

  ;; Read unlensed and lensed Cl
  ;;-----------------------------
  cmb_lensedcl_file = cmbdir + 'cmb' + '_lensed_cl.fits' 
  cmb_unlensedcl_file = cmbdir + 'cmb' + '_unlensed_cl.fits' 
  PSM_DEBUG_MESSAGE, 'Fix problem with the format of the fits file', routine='GAUSSIAN_CMB', infolimit=3, stoplimit=3
  ;stop
  READ_CL, cmb_lensedcl_file, lensedcl, hdr=hdrlensed
  READ_CL, cmb_unlensedcl_file, unlensedcl, hdr=hdrunlensed

  ;; Create cmb component structure
  ;;-----------------------------------
  cmb_id = STRTRIM(SXPAR(hdrunlensed,'PSM_CPID'),2)
  cmb = COMPONENT_STRUCT('cmb',name='cmb',id=cmb_id)
  cmb.polarised = (model.global.sky_fields NE 'T')
  cmb.e1.ampl.unit = model.units.cmb
  *cmb.e1.numin.value = 0.
  *cmb.e1.numax.value = FLOAT('inf')

  polar = FIX(cmb.polarised)

  ;; Select the cl to be used to generate the CMB map and the name of
  ;; the output cmb alm 
  ;;------------------------------------------------------------------

  CASE model.cmb.cmb_lensing OF
     'cl': BEGIN
        cl = lensedcl
        alm_writefile = cmb_lensedalm_file
     END
     ELSE: BEGIN
        cl = unlensedcl
        alm_writefile = cmb_unlensedalm_file
     END
  ENDCASE

  ;;--------------------------------------------------------------------------------------------------------------------;;
  ;; Convolve cl with beam and pixel window function
  ;;--------------------------------------------------------------------------------------------------------------------;;
  ;; Get gaussian beam 
  IF NOT DEFINED(skyres) THEN skyres = SKY_RESOLUTION()
  bl = GAUSSBEAM(skyres, MAX(cl.L), 3)

  ;; Get pixel window function 
  IF model.global.sky_pixwindow NE 0 THEN BEGIN
    tmpwl = HEALPIXWINDOW(SKY_NSIDE(),3)
    tmpwl = tmpwl(*,0:2*polar)
    nwl=N_ELEMENTS(tmpwl[*,0])-1 < MAX(cl.L)
    wl = REPLICATE(0.d0, MAX(cl.L)+1, 2*polar+1)
    wl(0:nwl,*) = tmpwl(0:nwl,*)    
    DELVARX, tmpwl
    bl=bl[cl.L,*]*wl[cl.L,*]
  ENDIF

  ;; Apply beam to fields containing T
  wht = WHERE(cl.field EQ 'T', nwht, complement=whnt, ncomplement=nwhnt)
  IF nwht EQ 1 THEN BEGIN
     cmd = 'cl.c_'+NUMBER2STRING(wht[0]+1)+'_'+NUMBER2STRING(wht[0]+1)+' = cl.c_'+NUMBER2STRING(wht[0]+1)+'_'+NUMBER2STRING(wht[0]+1)+'*bl[cl.L,0]*bl[cl.L,0]' & IF PSM_VERBOSITY() GE 1 THEN PRINT, cmd
     ok = EXECUTE(cmd) & IF NOT ok THEN STOP
     IF nwhnt GT 0 THEN BEGIN
        FOR ii=0, nwhnt-1 DO BEGIN
           IF wht[0] LT whnt[ii] THEN BEGIN
              cmd = 'cl.c_'+NUMBER2STRING(wht[0]+1)+'_'+NUMBER2STRING(whnt[ii]+1)+' = cl.c_'+NUMBER2STRING(wht[0]+1)+'_'+NUMBER2STRING(whnt[ii]+1)+'*bl[cl.L,0]' & IF PSM_VERBOSITY() GE 1 THEN PRINT, cmd
              ok = EXECUTE(cmd) & IF NOT ok THEN STOP
           ENDIF ELSE BEGIN
              cmd = 'cl.c_'+NUMBER2STRING(whnt[ii]+1)+'_'+NUMBER2STRING(wht[0]+1)+' = cl.c_'+NUMBER2STRING(whnt[ii]+1)+'_'+NUMBER2STRING(wht[0]+1)+'*bl[cl.L,0]' & IF PSM_VERBOSITY() GE 1 THEN PRINT, cmd
              ok = EXECUTE(cmd) & IF NOT ok THEN STOP
           ENDELSE
        ENDFOR
     ENDIF
  ENDIF

  IF polar THEN BEGIN

     ;; Apply beam to fields containing E
     wht = WHERE(cl.field EQ 'E', nwht, complement=whnt, ncomplement=nwhnt)
     IF nwht EQ 1 THEN BEGIN
        cmd = 'cl.c_'+NUMBER2STRING(wht[0]+1)+'_'+NUMBER2STRING(wht[0]+1)+' = cl.c_'+NUMBER2STRING(wht[0]+1)+'_'+NUMBER2STRING(wht[0]+1)+'*bl[cl.L,1]*bl[cl.L,1]' & IF PSM_VERBOSITY() GE 1 THEN PRINT, cmd
        ok = EXECUTE(cmd) & IF NOT ok THEN STOP
        IF nwhnt GT 0 THEN BEGIN
           FOR ii=0, nwhnt-1 DO BEGIN
              IF wht[0] LT whnt[ii] THEN BEGIN
                 cmd = 'cl.c_'+NUMBER2STRING(wht[0]+1)+'_'+NUMBER2STRING(whnt[ii]+1)+' = cl.c_'+NUMBER2STRING(wht[0]+1)+'_'+NUMBER2STRING(whnt[ii]+1)+'*bl[cl.L,1]' & IF PSM_VERBOSITY() GE 1 THEN PRINT, cmd
                 ok = EXECUTE(cmd) & IF NOT ok THEN STOP
              ENDIF ELSE BEGIN
                 cmd = 'cl.c_'+NUMBER2STRING(whnt[ii]+1)+'_'+NUMBER2STRING(wht[0]+1)+' = cl.c_'+NUMBER2STRING(whnt[ii]+1)+'_'+NUMBER2STRING(wht[0]+1)+'*bl[cl.L,1]' & IF PSM_VERBOSITY() GE 1 THEN PRINT, cmd
                 ok = EXECUTE(cmd) & IF NOT ok THEN STOP
              ENDELSE
           ENDFOR
        ENDIF
     ENDIF
     
     ;; Apply beam to fields containing B
     wht = WHERE(cl.field EQ 'B', nwht, complement=whnt, ncomplement=nwhnt)
     IF nwht EQ 1 THEN BEGIN
        cmd = 'cl.c_'+NUMBER2STRING(wht[0]+1)+'_'+NUMBER2STRING(wht[0]+1)+' = cl.c_'+NUMBER2STRING(wht[0]+1)+'_'+NUMBER2STRING(wht[0]+1)+'*bl[cl.L,2]*bl[cl.L,2]' & IF PSM_VERBOSITY() GE 1 THEN PRINT, cmd
        ok = EXECUTE(cmd) & IF NOT ok THEN STOP
        IF nwhnt GT 0 THEN BEGIN
           FOR ii=0, nwhnt-1 DO BEGIN
              IF wht[0] LT whnt[ii] THEN BEGIN
                 cmd = 'cl.c_'+NUMBER2STRING(wht[0]+1)+'_'+NUMBER2STRING(whnt[ii]+1)+' = cl.c_'+NUMBER2STRING(wht[0]+1)+'_'+NUMBER2STRING(whnt[ii]+1)+'*bl[cl.L,2]' & IF PSM_VERBOSITY() GE 1 THEN PRINT, cmd
                 ok = EXECUTE(cmd) & IF NOT ok THEN STOP
              ENDIF ELSE BEGIN
                 cmd = 'cl.c_'+NUMBER2STRING(whnt[ii]+1)+'_'+NUMBER2STRING(wht[0]+1)+' = cl.c_'+NUMBER2STRING(whnt[ii]+1)+'_'+NUMBER2STRING(wht[0]+1)+'*bl[cl.L,2]' & IF PSM_VERBOSITY() GE 1 THEN PRINT, cmd
                 ok = EXECUTE(cmd) & IF NOT ok THEN STOP
              ENDELSE
           ENDFOR
        ENDIF
     ENDIF  
     
  ENDIF

  ;;--------------------------------------------------------------------------------------------------------------------;;
  ;; Get constraining alm and Nl if necessary
  ;;--------------------------------------------------------------------------------------------------------------------;;

  IF model.cmb.cmb_constrained EQ 'yes' THEN BEGIN

     ;; Get NILC5 alm and noise power spectrum, at the resolution of
     ;; the PSM sky, with the proposer window function
     ;;--------------------------------------------------------------
     lmaxnilc = (SKY_LMAX() < 1000)
     almobs = NILC5_ALM(beam_out=SKY_RESOLUTION(),lmax=lmax,pixwin_out=(model.global.sky_pixwindow) * SKY_NSIDE())
     nlnilc = NILC5_NL(LINDGEN(lmaxnilc+1L),beam_out=SKY_RESOLUTION(),pixwin_out=(model.global.sky_pixwindow) * SKY_NSIDE())
     nlobs = CL_STRUCTURE(nfields=1,fieldnames='T',units='uK_CMB',lmin=0, lmax=lmaxnilc, double=double)
     nlobs.c_1_1[*] = nlnilc
  ENDIF
  

  ;;--------------------------------------------------------------------------------------------------------------------;;
  ;; Generate CMB alm 
  ;;--------------------------------------------------------------------------------------------------------------------;;

  ;; Set sky resolution
  skyres = SKY_RESOLUTION()
  ;; Set sky nside
  skynside = SKY_NSIDE()
  ;; Set sky lmax
  IF model.cmb.cmb_lensing EQ 'ilens' THEN skylmax = SKY_LMAX()+300L ELSE skylmax = SKY_LMAX()
  ;; Set sky pixelisation
  skypix = SKY_PIX()

  CASE model.cmb.cmb_cl_source OF

     'CAMB': BEGIN
        scalarclfile = GETPSMDIR('cosmo')+'camb/camb_scalCls.txt'
        tensorclfile = GETPSMDIR('cosmo')+'camb/camb_tensCls.txt'
        lensedscalarclfile = GETPSMDIR('cosmo')+'camb/camb_lensedCls.txt'
        cmboutputscale = 0
     END

     'CLASS': BEGIN
     END

     'standard-LCDM': BEGIN
     END

  ENDCASE

  CASE skypix.pixtype OF


  ;; Filenames for the theory alm and map 
  ;;--------------------------------------
  ;; cmb_unlensedalm_file = cmbdir + param.names.cmb_unlensed_alm_file
  ;; cmb_unlensedmap_file = cmbdir + param.names.cmb_unlensed_map_file
  ;; cmb_lensedalm_file = cmbdir + param.names.cmb_lensed_alm_file
  ;; cmb_lensedmap_file = cmbdir + param.names.cmb_lensed_map_file
  ;; potential_alm_file = cmbdir + param.names.potential_alm_file 
  ;; potential_map_file = cmbdir + param.names.potential_map_file

  ;; Read unlensed and lensed Cl
  ;;-----------------------------
  ;; cmb_lensedcl_file = cmbdir + 'cmb' + '_lensed_cl.fits' 
  ;; cmb_unlensedcl_file = cmbdir + 'cmb' + '_unlensed_cl.fits' 
  ;; PSM_DEBUG_MESSAGE, 'Fix problem with the format of the fits file', routine='GAUSSIAN_CMB', infolimit=3, stoplimit=3
  ;; READ_CL, cmb_lensedcl_file, lensedcl, hdr=hdrlensed
  ;; READ_CL, cmb_unlensedcl_file, unlensedcl, hdr=hdrunlensed

    'HEALPIX': BEGIN 

       IF model.cmb.cmb_constrained EQ 'no' THEN BEGIN
          PSM_CL2ALM, cl, alm, lmin=2, lmax=MAX(cl.L) < skylmax, double=double
          ;; READ_CAMB_CL, scalarclfile, scalarcl, cmboutputscale=0

       ENDIF ELSE BEGIN
          PSM_CL2ALM, cl, alm, lmin=2, lmax=MAX(cl.L) < skylmax, almobs=almobs, nlobs=nlobs, double=double
       ENDELSE
       tags = cl.field
       nalm = N_ELEMENTS(alm[*,0,0])
       IF PRECISION_ISDOUBLE() THEN almcmb = DBLARR(nalm,2,2*polar+1) ELSE almcmb = FLTARR(nalm,2,2*polar+1)

       wht = WHERE(tags EQ 'T', nwh)
       IF nwh EQ 1 THEN almcmb[*,*,0] = alm[*,*,wht[0]]
       IF polar THEN BEGIN
          whe = WHERE(tags EQ 'E', nwh)
          IF nwh EQ 1 THEN almcmb[*,*,1] = alm[*,*,whe[0]]
          whb = WHERE(tags EQ 'B', nwh)
          IF nwh EQ 1 THEN almcmb[*,*,2] = alm[*,*,whb[0]]
       ENDIF

       WRITE_CMB_ALM, TEMPORARY(almcmb), alm_writefile, cmb
       
       ;;---------------------------------------;;
       ;; Write potential files if necessary
       ;;---------------------------------------;;
       IF model.cmb.cmb_lensing EQ 'ilens' THEN BEGIN
          ;; Extract lensing potential
          IF PRECISION_ISDOUBLE() THEN almpot = DBLARR(nalm,2) ELSE almpot = FLTARR(nalm,2)
          whp = WHERE(tags EQ 'P', nwh)
          IF nwh EQ 1 THEN almpot[*,*] = alm[*,*,whp[0]]
          DELVARX, alm
          ;; Write unlensed cmb map
          cmb_unlensedalm_file = cmbdir + param.names.cmb_unlensed_alm_file
          cmb_unlensedmap_file = cmbdir + param.names.cmb_unlensed_map_file          
          PSM_ALM2MAP, cmb_unlensedalm_file, cmb_unlensedmap_file, hpx_nside=skynside, polar=polar, double=double 
          ;; Write lensing potential alm and map files
          basehdr = PSM_BASE_HDR(datatype='comp', dataform='alm', /setval)
          almhdr = PSM_ALM_HDR(lmax=MAX(cl.L) < skylmax, pxwin=0)
          beamhdr = PSM_BEAM_HDR(0)
          MK_PSMHDR, hdr, basehdr, almhdr, beamhdr
          WRITE_ALM, almpot, potential_alm_file, xhdr1=hdr
          DELVARX, almpot
          PSM_ALM2MAP, potential_alm_file, potential_map_file, hpx_nside=skynside, double=double
       ENDIF ELSE BEGIN
          *cmb.e1.ampl.value = alm_writefile
       ENDELSE
    END ;; End of case of healpix maps

  ENDCASE
  ;;--------------------------------------------------------------------------------------------------------------------;;
  ;; 
  ;;--------------------------------------------------------------------------------------------------------------------;;  
END
