;FUNCTION COVOLUME, z_array, cosmo=cosmo
;+
; NAME:
;    COVOLUME
; ONE LINE HELP:
;    Compute the covolume dV/dz/dOmega (Mpc^3)
; PURPOSE:
;    Compute the covolume dV/dz/dOmega (Mpc^3)
; CALLING SEQUENCE:
;    cov_array = COVOLUME(z_array, cosmo=cosmo)
; INPUT:
;    z_array : an array of redshift values
;
; OUTPUT:
;    cov_array : the array of covolumes corresponding to z_array
;    
; KEYWORDS:
;    cosmo : the structure containing the cosmological parameters
; EXAMPLE:
;    
; RESTRICTIONS:
;    Not fully tested yet. Report any bugs to Jacques Delabrouille
;    j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;    HUBBLE_AT_Z
;    ANG_DIST
;
; REVISION HISTORY
;    Written, Jean-Baptiste Melin, September 6 2001
;    Keyword cosmo introduced, JBM, Sept 27 2001
;    Works with quintessence models, Matthieu Roman, April 2012
;-   

FUNCTION COVOLUME, z_array, cosmo=cosmo

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
        print,'CALL IS : COVOLUME(z_array)'
        print,'Keyword : cosmo=cosmo'
     GOTO, closing
ENDIF

c_light = 2.9979246d+05 ; in km/s
hb = HUBBLE_AT_Z(z_array, cosmo=cosmo)
ad = ANG_DIST(z_array, cosmo=cosmo)

RETURN, c_light*(1+z_array)^2/hb*ad^2

closing:
END
