;FUNCTION PRESS_SCHECHTER, delta_c, sigma, z, cosmo=cosmo
;         
;+
; NAME:
;  PRESS_SCHECHTER
; ONE LINE HELP:
;  give M/rho0*dNclu/(d_covolume*dln(sigma^(-1))) using the Press-Schechter formula
; PURPOSE:
;  give M/rho0*dNclu/(d_covolume*dln(sigma^(-1))) using the Press-Schechter formula
; CALLING SEQUENCE:
;  result = PRESS_SCHECHTER(delta_c, sigma, z, cosmo=cosmo)
;
; INPUT:
;  delta_c : the critical density contrast at redshift z
;  sigma : fluctuation amplitude for scale M and redshift z
;  z : the redshift of computation
; OUTPUT:
;  M/rho0*dNclu/(d_covolume*dln(sigma^(-1)))
;    
; KEYWORDS:
;  cosmo : the structure containing the cosmological parameters
; EXAMPLES:
;  
; RESTRICTIONS:
;  Not fully tested yet. Report any bugs to Jacques Delabrouille
;  j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;  No procedure called 
; REVISION HISTORY
;  Written, Jean-Baptiste Melin, September 10 2001
;  Missing ' added in first print, Antoine Chamballu, 2006
;-

FUNCTION PRESS_SCHECHTER, delta_c, sigma, z, cosmo=cosmo

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
    print,'  CALL IS : PRESS_SCHECHTER(delta_c, sigma, z)'
    print,'  KEYWORD : cosmo=cosmo'
    GOTO, closing
ENDIF

nu = delta_c/sigma

nu_lim = 30.

nu = nu < nu_lim 

RETURN, SQRT(2/!pi)*nu*EXP(-nu^2/2.)

closing:
END
