FUNCTION CORRSKY_SPH, nside, Sn, Sx, Slim=Slim
;Slim in Jy
;outputmap in DT/T

  COMMON psm, param
  COMMON sky, model

;; Define parameters for convenience
;;-----------------------------------
  gp = param.global
  psp = model.ps
  firbp = model.firb
  names = param.names
  runp = param.run
  pixp = model.mapping.pix


  nu=353.
  IF NOT keyword_set(Slim) THEN BEGIN
     Slim=100.
  ENDIF
  
;; Compute the density map
;;-------------------------

  ;nlmax   = 3.*nside-1.
  ;nmmax   = nlmax
  npixtot = 12.*nside*nside
  pix_size_arcmin = 360.*60./SQRT(npixtot*!PI)  
  

  clname = firbp.firb_files.cl_sph_input

  almname = PSM_TMPFILENAME(extension='.fits')
;STRMID(clname,0,STRPOS(clname,'.fits'))+'_alm.fits'
  
  almfile_exists = FILE_TEST(almname)
  IF almfile_exists THEN SPAWN, 'rm '+almname
  
  t0 = SYSTIME(1)


  tab = mrdfits(clname,1)
  lmax = SKY_LMAX()
  IF lmax GT n_elements(tab.temperature) THEN BEGIN
     ell = LINDGEN(n_elements(tab.temperature))
     cl = tab.temperature
  ENDIF ELSE BEGIN
     ell = LINDGEN(lmax+1)
     cl = (tab.temperature)[0:lmax]
  ENDELSE
  GENERATE_X_LM, ell, cl, alm,my_seed=GETSEED(chain_name='firb-pipe'), write_fits_file = almname
   
  ;SPAWN, 'rm map_den.fits'
  ;outname='map_den.fits'
  PSM_ALM2MAP, almname, map, hpx_nside=nside, /double
  ;ALM2MAP_CXX, almname, 0., nlmax, nside, outname='map_den.fits'
  SPAWN, 'rm '+almname
  e_t = SYSTIME(1)-t0
  
  ;READ_FITS_MAP, 'map_den.fits', map
  ;SPAWN, 'rm map_den.fits'
  
;; Read the spheroids counts at 857GHz
;;-------------------------------------
;; Spheroids(lapi)

  GET_LUN, unit

  OPENR, unit,  firbp.firb_files.lapi_353GHz
; !PSMROOT+'data/ancillary/models/firb/JGN-2005/lapi_353GHz.dat'

  i=0L
  WHILE NOT EOF(unit) DO BEGIN
     READF,unit,a
     i=i+1
  ENDWHILE

  POINT_LUN, unit, 0
  c = FLTARR(2,i)
  READF, unit, c
  CLOSE, unit
  FREE_LUN, unit


  hor = c[0,*]
  bin = ABS(hor[1]-hor[2])          ;
  ver = 10.^c[1,*]*bin              ;
  c = [hor,ver]


  c[1,*]=c[1,*]*4.*!PI
  w = WHERE(c[0,*] GT Sn AND c[0,*] LT Sx)
  ntot = TOTAL(c[1,w], /double)

  map = (TEMPORARY(map)+1.)*ntot/npixtot
  map_floor=floor(map)
  r=randomu(getseed(),n_elements(map))
  w=where(r lt (map-map_floor))
  map_floor[w]=map_floor[w]+1 

  map = TEMPORARY(map_floor)


  ;map = RANDOM_ROUND(TEMPORARY(map))
  


  ntot = TOTAL(map, /double)


;; Density to Fluxes
;;-------------------
  PRINT,''
  PRINT,'+---------------+'
  PRINT,'!   FLUXASOC    !'
  PRINT,'+---------------+'
  PRINT,'  nsources=',ntot

  t0 = SYSTIME(1)
  map = FLUXASOC(TEMPORARY(map), c, Sn, Sx, Slim=Slim)
  PRINT,'  e_time(Healpix)=', round(e_t), ' sec'
  PRINT,'  e_time(FLUXASOC)=', round(systime(1)-t0), ' sec'


;; Flux to Temperature (Delta_T/T)
;;---------------------------------

  pixsize = 4*!PI/npixtot 
  factor = CONVERSION_FACTOR('Jy/sr','K/KCMB',nu*1e9)     ; nu in Hz
  factor = factor/pixsize                                 ; because initial map is Jy, not Jy/sr
  map = TEMPORARY(map) * factor

  ;beam = 4*!PI/npixtot
  ;f = nu/56.8
  ;map = TEMPORARY(map)/(24.8*beam*1.e6*(2.*f^2./(exp(f/2.)-exp(-f/2.)))^2.)/2.725

RETURN, map

END
