FUNCTION FIRB_DNDSDZ_SPHEROIDS, population, which_counts, cosmo=cosmo, compute=compute


  IF (population NE 'LENSED') AND (population NE 'UNLENSED') THEN BEGIN
     print, 'case not implemented. Please use either "UNLENSED" or "LENSED" population'
     STOP
  ENDIF
  

  ;; choose one reference file
  ;;--------------------------
  CASE which_counts OF 
     '250micron': BEGIN
        model='lapi'
        ;; ref_freq = 3.*(10.^14)/250.
        ref_freq = !phyconst.c / 250.e-6
        which='250micron'
        flux_units='mJy'
        fname = GETPSMDIR('psmdata')+'ancillary/models/firb/GC-2012/'+'fluxfunc_'+which+'_extended.sav'
     END
     '350micron': BEGIN
        model='lapi'
        ;; ref_freq = 3.*(10.^14)/250.
        ref_freq = !phyconst.c / 350.e-6
        which='350micron'
        flux_units='mJy'
        fname = GETPSMDIR('psmdata')+'ancillary/models/firb/GC-2012/'+'fluxfunc_'+which+'_extended.sav'
     END
     'Cai350micron': BEGIN
        model='cai2012'
        ;; ref_freq = 3.*(10.^14)/250.
        ref_freq = !phyconst.c / 350.e-6
        which='350um857GHz_sph_ste'
        flux_units='mJy'
        fname = GETPSMDIR('psmdata')+'ancillary/models/firb/GC-2012/'+'fluxfunc_'+which+'_extended.sav'
     END
  ENDCASE



  IF NOT KEYWORD_SET(compute) THEN BEGIN

     ;; test if extrapolated counts have already been computed
     ;;---------------------------------------------------
     ok = TEST_PSM_DATA(fname, /get)
     IF NOT ok THEN MESSAGE, 'File not available: '+fname
     restore, fname

  ENDIF ELSE BEGIN

     ;; read counts
     ;;------------
     CASE model OF
        'lapi': BEGIN
           fname = GETPSMDIR('psmdata')+'ancillary/models/firb/GC-2012/'+'fluxfunc_'+which+'.txt'
           ok = TEST_PSM_DATA(fname, /get)
           IF NOT ok THEN MESSAGE, 'File not available: '+fname
           
           rnz=27
           nS=31
           nskip=9
           array=dblarr(rnz, nS)
           log_n=dblarr(rnz, nS)
           rfluxes=dblarr(nS)  
           redshifts=findgen(rnz)*.25+1
           
           For i=0, rnz-1 DO BEGIN
              readcol, fname, skipline=nskip+(nS+4)*i, numline=nS, logS, logNn, logNl
              rfluxes=10.^(logS)
              IF population eq 'UNLENSED' THEN logN=logNn ELSE logN=logNl
              
              logNn=0
              logNl=0
              array[i,*]=(10.^logN)/rfluxes       
              
              wh=where(finite(logN) eq 0, nwh)
              IF nwh ne 0 THEN BEGIN
                 wh=where(finite(logN) ne 0, nwh)
                 atemp=interpol(logN[wh],rfluxes[wh],rfluxes)
                 logN=atemp
              ENDIF     
              log_n[i,*]=logN
              
        
           ENDFOR
        END
        'cai2012': BEGIN
           fname = GETPSMDIR('psmdata')+'ancillary/models/firb/GC-2012/July2012/fluxfunc_test/'+'fluxfunc_'+which+'.dat'
           ok = TEST_PSM_DATA(fname, /get)
           IF NOT ok THEN MESSAGE, 'File not available: '+fname
           
           rnz=25
           nS=31
           nskip=8
           array=dblarr(rnz, nS)
           log_n=dblarr(rnz, nS)
           rfluxes=dblarr(nS)  
           redshifts=findgen(rnz)*.25+1           

          
           For i=0, rnz-1 DO BEGIN
              readcol, fname, skipline=nskip+(nS)*i, numline=nS, logS, logN
              rfluxes=10.^(logS)
              
              logNn=0
              logNl=0
              array[i,*]=(10.^logN)/rfluxes       
              
              wh=where((finite(logN) eq 0), nwh) ; OR (logN le -200)
              IF nwh ne 0 THEN BEGIN
                 wh=where((finite(logN) ne 0), nwh) ; AND (logN gt -200)
                 atemp=interpol(logN[wh],rfluxes[wh],rfluxes)
                 logN=atemp
              ENDIF     
              log_n[i,*]=logN
              
        
           ENDFOR

        END
     ENDCASE
;;;;;;;;;;;;;;;;;;;;;
     
;;;special value for null counts
  
;whn=where(finite(log_n) eq 0, nwh)
;IF nwh ne 0 THEN log_n[whn]=-300
;znn=min(redshifts[whn])
  
;whl=where(finite(log_l) eq 0, nwh)
;IF nwh ne 0 THEN log_l[whl]=-300
;zll=min(redshifts[whl])
  
;;;;;;;;;;;
  
  
     ;; arbitrary values for flux and z number of bins
     ;;-----------------------------------------------
     bnflux=30000.
     nz=200.
     
     nflux=bnflux/10.
     lfmin=min(logS)
     
     logflux=(findgen(nflux))*(max(logS)-lfmin)/(nflux-1)+lfmin
     blogflux=(findgen(bnflux))*(max(logS)-lfmin)/(bnflux-1)+lfmin
     
     fluxes=10.^logflux  ;(findgen(nflux))*(max(rfluxes)-flmin)/(nflux-1)+flmin
     bfluxes=10.^blogflux ;(findgen(bnflux))*(max(rfluxes)-flmin)/(bnflux-1)+flmin
     
     
;; z range from 1 to 8
;;--------------------
     z_tot=findgen(nz+1)*(7./nz)+1 ;;nz+1 to compute a volume for last shell
     z=z_tot[0:nz-1]

     z_counts=dblarr(nz, nflux)
     bz_counts=dblarr(nz, bnflux)
     dndlgsdz=dblarr(nz, bnflux)
     
;----------test------------
;     nz_counts=dblarr(nz, nS)
;     
;     For i=0, nS-1 DO BEGIN
;        nz_counts[*,i]=interpol(log_n[*,i], redshifts, z)
;    ;; ;nz_counts[i,*]=(10.^bz_counts[i,*])/bfluxes;
;
;     ENDFOR
;     For i=0, nz-1 DO BEGIN
;        dndlgsdz[i,*]=interpol(nz_counts[i,*],logS, blogflux)
;        bz_counts[i,*]=(10.^dndlgsdz[i,*])/(bfluxes*alog(10))
;        dndlgsdz[i,*]=(10.^temporary(dndlgsdz[i,*]))
;     ENDFOR

  ;z_counts=MIN_CURVE_SURF(log_n,xvalues=redshifts, yvalues=logS, xout=z, yout=logflux)  

biliin, redshifts, logS, log_n, z, logflux, z_counts


 ; z_counts=TRI_SURF(log_n, redshifts, logS, GS=[z[1]-z[0], logflux[1]-logflux[0]])  
   ; z_counts=MIN_CURVE_SURF(log_n,xvalues=redshifts, yvalues=10.^logS, xout=z, yout=fluxes) 
  For i=0, nz-1 DO BEGIN
     bz_counts[i,*]=interpol(z_counts[i,*],logflux, blogflux)
truc=bz_counts
     dndlgsdz[i,*]=(10.^bz_counts[i,*])
     bz_counts[i,*]=(10.^bz_counts[i,*])/(bfluxes*alog(10))
  ENDFOR
  ;;counts_units:'Mpc-3'  



     ;save, dndlgdsdz, bzcounts, z_tot, filename=fname

  ENDELSE

  
  ;; comoving volume of all micro-shells and convert counts from Mpc-3
  ;; to sr-1.dz-1
  ;;-------------------------------------------------------------------
  nz = N_ELEMENTS(z_tot)-1
  r_borders = COM_DIST(z_tot, cosmo=cosmo)
  z_vol = dblarr(nz)


  FOR i=0, nz-1 DO BEGIN
     ;; simplification by 4pi in volume and dndlgdz
     z_vol[i]=(1./3)*(r_borders[i+1])^3-(1./3)*(r_borders[i])^3.
     dndlgsdz[i,*]=dndlgsdz[i,*]*z_vol[i]/((z_tot[i+1]-z_tot[i]))
     bz_counts[i,*]=bz_counts[i,*]*z_vol[i]/((z_tot[i+1]-z_tot[i]))
  ENDFOR

  counts_units='sr-1.dz-1.mJy-1'

  counts={population:population, fluxes:bfluxes, lg_fluxes:blogflux, flux_units:flux_units, lg_flux:'log10(fluxes)', redshifts:z, dndsdz:bz_counts, dndlgsdz:dndlgsdz , counts_units:counts_units,nuref:ref_freq}
  
  return, counts
  
END
