FUNCTION FIRB_POP_SHELLS, nside, counts, densmap, zborders, flux_max=flux_max

  t0=systime(1)

  ;; Limiting number of sources to switch from one method of source distribution to the other
  slimit = 1e6 
  
  tmap = dblarr(nside2npix(nside))
  map_at_z = dblarr(nside2npix(nside))

  ;; define number of flux bins
  nflux=100

  fluxmin=min(counts.fluxes)
     
  IF DEFINED(flux_max) THEN BEGIN
     whg=where(counts.fluxes lt flux_max, ncflux)
     fluxmax=TEMPORARY(flux_max)
  ENDIF ELSE BEGIN
     ncflux = N_ELEMENTS(counts.fluxes)
     fluxmax=max(counts.fluxes)
  ENDELSE

  last_ind=-1

  zmin=zborders[0]
  zmax=zborders[1]
  j=0L

  whz = WHERE((counts.redshifts ge zmin) AND (counts.redshifts lt zmax),nwh)
  
  IF nwh NE 0 THEN BEGIN
     imin=min(whz)
     IF imin NE 0 THEN whz=[imin-1,whz]
     nz = N_ELEMENTS(whz)
     
     N_dz=dblarr(nz)
     
     WHILE j le nflux DO BEGIN
        
        ;; Width of the flux bin
        ;dnflux=floor((ncflux-1.)*1./nflux)>5
        dflux = (fluxmax-fluxmin)/nflux
        
        ;; starting and ending indixes of the current flux bin
        ;ideb=dnflux*j
        ;ifin=(j+1)*dnflux<ncflux-1
        good_ind= WHERE((counts.fluxes ge (fluxmin+dflux*j)) AND (counts.fluxes le (fluxmin+dflux*(j+1))) AND (counts.fluxes le fluxmax))
        ideb=min(good_ind)
        ifin=max(good_ind)

        IF ifin eq -1 THEN BEGIN
           ideb=last_ind+1
           ifin=(ifin>(ideb+3))<ncflux-1
        ENDIF ELSE BEGIN
           IF ideb le last_ind THEN BEGIN
              ideb=last_ind+1
              ifin=(ifin>(ideb+3))<ncflux-1
           ENDIF
        ENDELSE
        
        IF last_ind eq (ncflux-1)THEN BEGIN
           ;;Game is over. Go home.
           ideb = -1
           ifin = -2
        ENDIF ELSE BEGIN
           last_ind=ifin
        ENDELSE

        IF (ideb lt 0) AND (ifin ge 0) THEN STOP
        IF ifin gt 0 THEN IF counts.fluxes[ifin] GT fluxmax THEN STOP

        IF ideb LT ifin THEN BEGIN                     
           FOR iz= 0, nz-1 DO BEGIN
              N_dz[iz] = int_tabulated(counts.lg_fluxes[ideb:ncflux-1], counts.dndlgsdz[whz[iz],ideb:ncflux-1],/double)
           ENDFOR
           N_restant = 4*!dpi*int_tabulated(counts.redshifts[whz], N_dz,/double)
          
           IF N_restant GT slimit THEN BEGIN
              
              ;; First algorithm using random_dist (for many sources at the same flux)
              ;;----------------------------------------------------------------------
              
              FOR iz= 0, nz-1 DO BEGIN
                 N_dz[iz] = int_tabulated(counts.lg_fluxes[ideb:ifin], counts.dndlgsdz[whz[iz],ideb:ifin],/double)
              ENDFOR
              
              
              N_thisflux = 4*!dpi*int_tabulated(counts.redshifts[whz], N_dz,/double)
              
              N_dlgS=dblarr(ifin-ideb+1)
              FOR i_s= ideb, ifin DO BEGIN
                 N_dlgS[i_s-ideb] = int_tabulated(counts.redshifts[whz], counts.dndlgsdz[whz,i_s],/double)
              ENDFOR
              
              
              flux_j=int_tabulated(counts.lg_fluxes[ideb:ifin], 4*!pi*N_dlgS*counts.fluxes[ideb:ifin],/double)/N_thisflux
              
              
              IF N_thisflux GT 0 THEN tmap = RANDOM_DISTRIBUTION(densmap, N_thisflux, iseed=GETSEED(), /silent, /double) ELSE tmap=0.
              map_at_z = map_at_z + tmap*flux_j       

IF counts.population eq 'UNLENSED' THEN BEGIN
 npix = N_elements(tmap)
fac=(1.e-9)*npix/(4*!dpi)     
              Print, 'flux here', total(tmap*flux_j)*fac, ' MJy'
           ENDIF

IF j eq 0 THEN BEGIN
   print, '----------------------------------------------'
   print, counts.population+' N this shell: '+strtrim(N_restant,2)
   print, '----------------------------------------------'
   print, '----------------------------------------------'
ENDIF              
;               print, strtrim(j,2)+' / '+strtrim(nflux,2)+' -- N restant: '+strtrim(N_restant,2)+'  -- temps: '+strtrim(systime(1)-t0,2), ' sec'
           ENDIF ELSE BEGIN
              
              ;; Second algorithm: group all remaining flux bins and
              ;; use random_draw
              ;;----------------------------------------------------
              
              ;;integrate in flux bin
              ;;---------------------
              FOR iz= 0, nz-1 DO BEGIN
                 N_dz[iz] = int_tabulated(counts.lg_fluxes[ideb:ifin], counts.dndlgsdz[whz[iz],ideb:ifin],/double)
              ENDFOR
              ;;integrate over redshift
              ;;-----------------------
              N_thisflux = 4*!dpi*int_tabulated(counts.redshifts[whz], N_dz,/double)
              
              ;;compute mean flux in the bin
              ;;----------------------------
              N_dlgS=dblarr(ifin-ideb+1)
              FOR i_s= ideb, ifin DO BEGIN
                 N_dlgS[i_s-ideb] = int_tabulated(counts.redshifts[whz], counts.dndlgsdz[whz,i_s],/double)
              ENDFOR
              flux_j=int_tabulated(counts.lg_fluxes[ideb:ifin], 4*!dpi*N_dlgS*counts.fluxes[ideb:ifin],/double)/N_thisflux
              IF flux_j EQ 0 THEN STOP
              
              IF N_thisflux ne 0 THEN BEGIN
                 N_thisflux=randomn(GETSEED(), poisson=N_thisflux, /double)
                 IF N_thisflux ne 0 THEN nvalues=replicate(flux_j, N_thisflux) ELSE nvalues=-1
              ENDIF ELSE BEGIN
                 nvalues=-1
              ENDELSE

              
              N_group=N_thisflux
              
              For k=1L, nflux-j DO BEGIN
                 ;ideb=dnflux*(j+k)
                 ;ifin=(j+k+1)*dnflux<ncflux-1
                 
                 good_ind= WHERE((counts.fluxes ge (fluxmin+dflux*(j+k))) AND (counts.fluxes le (fluxmin+dflux*(j+k+1))) AND (counts.fluxes le fluxmax))
                 ideb=min(good_ind)
                 ifin=max(good_ind)
                 IF ifin eq -1 THEN BEGIN
                    ideb=last_ind+1
                    ifin=(ifin>(ideb+3))<ncflux-1
                 ENDIF ELSE BEGIN
                    IF ideb le last_ind THEN BEGIN
                       ideb=last_ind+1
                       ifin=(ifin>(ideb+3))<ncflux-1
                    ENDIF
                 ENDELSE

                
                 IF last_ind eq (ncflux-1) THEN BEGIN
                    ;;Game is over. Go home.
                    ideb = -1
                    ifin = -2
                 ENDIF ELSE BEGIN
                    last_ind=ifin
                 ENDELSE

                 IF (ideb lt 0) AND (ifin ge 0) THEN STOP
                 IF ifin gt 0 THEN IF counts.fluxes[ifin] GT fluxmax THEN STOP



                 IF ideb lt ifin THEN BEGIN
                    IF (N_elements(nvalues) ne N_group) AND (N_group ne 0) THEN STOP
                    
 ;                   IF floor(k/10)*10 eq k then print,'shell '+strtrim(zmin,2)+' to '+strtrim(zmax,2)+': bin flux n '+strtrim(k,2)

                    ;;integrate in flux bin
                    ;;---------------------
                    FOR iz= 0, nz-1 DO BEGIN
                       N_dz[iz] = int_tabulated(counts.fluxes[ideb:ifin], counts.dndsdz[whz[iz],ideb:ifin],/double)
                    ENDFOR
                    ;;integrate over redshift
                    ;;-----------------------
                    N_thisflux = 4*!dpi*int_tabulated(counts.redshifts[whz], N_dz,/double)
                    
                    ;;compute mean flux in the bin
                    ;;----------------------------
                    N_dlgS=dblarr(ifin-ideb+1)
                    FOR i_s= ideb, ifin DO BEGIN
                       N_dlgS[i_s-ideb] = int_tabulated(counts.redshifts[whz], counts.dndlgsdz[whz,i_s],/double)
                    ENDFOR
                    flux_j=int_tabulated(counts.lg_fluxes[ideb:ifin], 4*!dpi*N_dlgS*counts.fluxes[ideb:ifin],/double)/N_thisflux
                    IF flux_j EQ 0 THEN STOP
      
                    IF N_thisflux ne 0 THEN N_thisflux=randomn(GETSEED(), poisson=N_thisflux, /double)
                    
                    IF N_thisflux GT 0 THEN BEGIN
                       temp_nvalues = replicate(flux_j, N_thisflux)
                       nvalues = [nvalues, temp_nvalues]
                       N_group = N_group+N_thisflux
                    ENDIF
                 ENDIF
              ENDFOR
              wh=WHERE(nvalues gt 0, nwh)
              IF (n_elements(nvalues) NE N_group) AND (nwh ne 0) THEN STOP
              IF nwh gt 0 THEN BEGIN
                 nvalues = nvalues[wh]
                 tmap = random_draw(densmap, N_group, values=nvalues, /draw, iseed=GETSEED())
                 map_at_z = map_at_z+tmap
              ENDIF

              ;; stop the parent loop
              j = nflux
              
              print, ' One more shell done '+strtrim(zmin,2)+' to '+strtrim(zmax,2)+'  ----------------------'
           ENDELSE
        ENDIF
;mollview, map_at_z


        j=j+1
     ENDWHILE
     IF last_ind ne (ncflux-1) THEN STOP

     IF counts.flux_units ne 'mJy' THEN STOP
     ;; convert from mJy to MJy/sr
     npix = N_elements(map_at_z)
print, '------------------------'
print, total(map_at_z), ' mJy
     conv_fac = (1.e-9)*npix/(4*!dpi)
     map_at_z = TEMPORARY(map_at_z) * conv_fac
  ENDIF
;wdel


;print, '------------------------'
;print, total(map_at_z), ' MJy/sr'
;stop
;mollview, map_at_z, title=counts.population+' zmin: '+number2string(zmin)+' zmax: '+number2string(zmax)
  RETURN, map_at_z

END

;;---------------------------------------------
;;---------------------------------------------
;;---------------------------------------------

PRO FIRB_GAL_TO_SHELLS, cosmo=cosmo, test=test

  IF DEFINED(cosmo) EQ 0 THEN BEGIN
     print, 'case not implemented. Please use "cosmo" keyword'
     stop
  ENDIF

  t0 = SYSTIME(1)

  ;; 'centers' and borders of shells in redshift
  ;;---------------------------------------------
  RESTORE, file = GETPSMDIR('cosmo')+'class/shells_class.sav'
  z_array = shells_class.z
  dz_array = shells_class.dz
  nz = N_ELEMENTS(z_array)
  zmin = MIN(z_array)
  zmax = MAX(z_array)
  z_borders = FLTARR(nz+1)
  z_borders[0] = (z_array[0] - dz_array[0]) > 0
  z_borders[nz] = z_array[nz-1] + dz_array[nz-1]
  IF nz GE 2 THEN FOR iz=1, nz-1 DO  z_borders[iz] = (dz_array[iz-1]*z_array[iz] + dz_array[iz]*z_array[iz-1]) / (dz_array[iz-1] + dz_array[iz])

  ;; comoving volume of all shells
  ;;-------------------------------

  r_borders = COM_DIST(z_borders, cosmo=cosmo)
  z_vol = dblarr(nz)
  FOR i=0, nz-1 DO z_vol[i]=(4.*!dpi/3)*(r_borders[i+1])^3-(4.*!dpi/3)*(r_borders[i])^3.

  ;; Source counts
  ;;---------------
  counts_sph_lensed = FIRB_DNDSDZ_SPHEROIDS('LENSED', 'Cai350micron', cosmo=cosmo, /compute)

;  counts_sph_lensed_lap = FIRB_DNDSDZ_SPHEROIDS('LENSED', '350micron', cosmo=cosmo, /compute)

  counts_spiral = FIRB_DNDSDZ_LATE_TYPE('SPIRAL')
  counts_starburst = FIRB_DNDSDZ_LATE_TYPE('STARBURST')

  ;; Create FIRB model structure
  ;;-----------------------------

  sph_em_number = INTARR(nz)
  spiral_em_number = INTARR(nz)
  starburst_em_number = INTARR(nz)

  wh_sph = WHERE(z_array GE 1, nz_sph)
  wh_spiral = WHERE(z_array LE 2, nz_spiral)
  wh_starburst = WHERE(z_array LE 2, nz_starburst)

  FOR i=1, nz_sph DO BEGIN & $

  ;   em = EMLAW_PARAMSTRUCT('SUBMMLAPI',/include_nurange,/include_ampl) & $
     em = EMLAW_PARAMSTRUCT('SUBMMCAI',/include_nurange,/include_ampl) & $
     *em.numax.value = FLOAT('inf') & $
     *em.numin.value = 0. & $
     *em.nuref.value = counts_sph_lensed.nuref & $
     *em.ampl.value = GETPSMDIR('firb')+'firb_spheroid_z'+NUMBER2STRING(wh_sph[i-1]+1)+'.fits' & $
     em.ampl.unit = 'MJy/sr' & $
     *em.redshift.value = z_array[wh_sph[i-1]] & $
     ok = EXECUTE('e'+NUMBER2STRING(i)+'=em') & $
  ENDFOR
  sph_em_number[wh_sph] = INDGEN(nz_sph)+1    

  FOR i=1, nz_spiral DO BEGIN & $
     em = EMLAW_PARAMSTRUCT('SPIRALGAL',/include_nurange,/include_ampl) & $
     *em.numax.value = FLOAT('inf') & $
     *em.numin.value = 0. & $
     *em.nuref.value = counts_spiral.nuref & $
     *em.ampl.value = GETPSMDIR('firb')+'firb_spiral_z'+NUMBER2STRING(wh_spiral[i-1]+1)+'.fits' & $
     em.ampl.unit = 'MJy/sr' & $
     *em.redshift.value = z_array[wh_spiral[i-1]] & $
     ok = EXECUTE('e'+NUMBER2STRING(i+nz_sph)+'=em') & $
  ENDFOR
  spiral_em_number[wh_spiral] = INDGEN(nz_spiral) + 1 + nz_sph 

  FOR i=1, nz_starburst DO BEGIN & $
     em = EMLAW_PARAMSTRUCT('STARBURSTGAL',/include_nurange,/include_ampl) & $
     *em.numax.value = FLOAT('inf') & $
     *em.numin.value = 0. & $
     *em.nuref.value = counts_starburst.nuref & $
     *em.ampl.value = GETPSMDIR('firb')+'firb_starburst_z'+NUMBER2STRING(wh_starburst[i-1]+1)+'.fits' & $
     em.ampl.unit = 'MJy/sr' & $
     *em.redshift.value = z_array[wh_starburst[i-1]] & $
     ok = EXECUTE('e'+NUMBER2STRING(i+nz_sph+nz_spiral)+'=em') & $
  ENDFOR
  starburst_em_number[wh_starburst] = INDGEN(nz_starburst) + 1 + nz_sph + nz_spiral

  COMMON psm, param
  cmd = 'comp = COMPONENT_STRUCT('
  eadd_ok=0
  emlist='emlaw_list=['
  FOR i=1, nz_sph+nz_spiral+nz_starburst DO BEGIN
     IF i GT 1 THEN emlist = emlist + ','
     emlist = emlist+'e'+NUMBER2STRING(i)
  ENDFOR

  emlist=emlist+']'
  cmd = cmd+emlist+",name=param.names.firb_basename, class='diffuse')"

print, "-------------------"
print, emlist

  ok = EXECUTE(cmd)
  IF NOT ok THEN STOP

prtstruct, comp

  PSM_ID_KEYGEN, firb_id
  comp.id = firb_id

  ;; read the bias
  ;;------------------------------
  ps_bias = proto_sph_bias(z_array, bias_model='xia_hermes')
  lt_bias = late_type_bias(z_array)

  skypix = SKY_PIX()
  sky_beam=GET_BEAM_STRUCT('GAUSSIAN',beamsize=SKY_RESOLUTION(),sizeunits='arcmin')

  ordering = skypix.ordering
  nside = skypix.nside
  npix = nside2npix(nside)

  ;; Populate all shells sequentially
  ;;----------------------------------
  FOR i=0, nz-1 DO BEGIN

     ;; Read the alm file of density contrast for that shell
     ;;------------------------------------------------------

     READ_ALM, GETPSMDIR('cosmo')+'class/density_contrast_shell'+NUMBER2STRING(i+1)+'.fits', alm
     psm_alm2map, alm, densmap, ordering=ordering, hpx_nside=nside, /overwrite


     ;; Prepare headers for maps
     ;;-------------------------
     basehdr = PSM_BASE_HDR(datatype='comp',dataform='map',/setval)
     maphdr = PSM_MAP_HDR(pixtype='HEALPIX',lmax=SKY_LMAX())
     xhdr = MERGE_FITSHDRS(basehdr,maphdr)
     beamhdr = PSM_BEAM_HDR(sky_beam)
     xhdr = MERGE_FITSHDRS(xhdr,beamhdr)

     ;; Populate proto-spheroids
     ;;-------------------------
     bias = ps_bias[i]


     dopop = where(wh_sph EQ i, ndopop)
     IF ndopop eq 1 THEN BEGIN
        densmap_pop = densmap*bias
        densmap_pop = (TEMPORARY(densmap_pop)+1)>0            
        map_at_z_proto_sph_lensed = FIRB_POP_SHELLS(nside, counts_sph_lensed, densmap_pop, z_borders[i:i+1])
        map_at_z_proto_sph = TEMPORARY(map_at_z_proto_sph_lensed)

        ;; Write the map
        ;;--------------
        ok = EXECUTE('fitsfile = *comp.e'+NUMBER2STRING(dopop[0]+1)+'.ampl.value')
        CHANGE_MAP_RESOLUTION, map_at_z_proto_sph, map_at_z_proto_sph,  beam_out=sky_beam, lmax=SKY_LMAX(), /overwrite
        WRITE_HPXMAP, fitsfile, tstokes=map_at_z_proto_sph, datatype='COMP', psm_dtid=firb_id, ordering=SKY_ORDERING(), units='MJy_sr', lmax=SKY_LMAX(), xhdr=xhdr, coordsys=skypix.coordsys
        DESTROY_DATA, map_at_z_proto_sph
     ENDIF

     ;; Populate spirals and starbusts
     ;;-------------------------------
     bias = lt_bias[i]

     densmap_pop=densmap*bias
;denstest=densmap_pop
     densmap_pop=(TEMPORARY(densmap_pop)+1)>0
;stop

     ;;spiral
     ;;------
     dopop = where(wh_spiral EQ i, ndopop)
     IF ndopop eq 1 THEN BEGIN        
        flux_max=100. ;; mJy
        map_at_z_spiral=FIRB_POP_SHELLS(nside, counts_spiral, densmap_pop,z_borders[i:i+1]);, flux_max=flux_max)
        ;; Write the map
        ;;--------------
        ok = EXECUTE('fitsfile = *comp.e'+NUMBER2STRING(dopop[0]+1+nz_sph)+'.ampl.value')
        CHANGE_MAP_RESOLUTION, map_at_z_spiral, map_at_z_spiral,  beam_out=sky_beam, lmax=SKY_LMAX(), /overwrite
        WRITE_HPXMAP, fitsfile, tstokes=map_at_z_spiral, datatype='COMP', psm_dtid=firb_id, ordering=SKY_ORDERING(), units='MJy_sr', lmax=SKY_LMAX(), xhdr=xhdr, coordsys=skypix.coordsys
        DESTROY_DATA, map_at_z_spiral
     ENDIF

     ;;starburst
     ;;---------
     dopop = where(wh_starburst EQ i, ndopop)
     IF ndopop eq 1 THEN BEGIN
        flux_max=100. ;; mJy
        map_at_z_starburst=FIRB_POP_SHELLS(nside, counts_starburst, densmap_pop, z_borders[i:i+1]);, flux_max=flux_max)  

        ;; Write the map
        ;;--------------
        ok = EXECUTE('fitsfile = *comp.e'+NUMBER2STRING(dopop[0]+1+nz_sph+nz_spiral)+'.ampl.value')
        CHANGE_MAP_RESOLUTION, map_at_z_starburst, map_at_z_starburst,  beam_out=sky_beam, lmax=SKY_LMAX(), /overwrite
        WRITE_HPXMAP, fitsfile, tstokes=map_at_z_starburst, datatype='COMP', psm_dtid=firb_id, ordering=SKY_ORDERING(), units='MJy_sr', lmax=SKY_LMAX(), xhdr=xhdr, coordsys=skypix.coordsys
        DESTROY_DATA, map_at_z_starburst
     ENDIF

     
  ENDFOR

  ;; Write the component structure
  IF FIRB_MODEL() NE 'no_firb' THEN BEGIN
     SAVE_COMP, comp
     PRTSTRUCT, comp, name='FIRB', file=GETPSMDIR('firb')+'firb.txt'
     DESTROY_DATA, comp
  ENDIF


  ;; PRINT,'TOTAL  time=', round(systime(1)-t0), ' sec'

closing:

END
