FUNCTION psm_thermal_dust,pix_num,freq,polQ=polQ,polU=polU

;+
; NAME:
;    psm_thermal_dust
; CATEGORY:
;    Planck Sky Model
; PURPOSE:
;    Calls PSM_SFD to computes SFD dust emission at given position(s) and frequency and 
;    polarization Q and U if keyword is specified
; CALLING SEQUENCE:
;    vec=psm_thermal_dust(pix_num,freq)
; INPUT:
;    pix_num = array of pixel numbers in the Nside used during
;              initialization (see sky_model_init).
;              Should be in same scheme (NESTED is the default)
;    freq    = frequency at which calculation is to be done (GHz)
; OUTPUT:
;    vec     = SFD contribution at given pixels and frequency (MJy/sr)
; OPTIONAL INPUT:
;    None
; INPUT KEYWORDS:
;    polQ    = if set, the map of Q Stokes parameter is output
;    polU    = if set, the map of U Stokes parameter is output
;    no_polar= if set, no polarization is calculated
; OUTPUT KEYWORDS:
;    POLQ = retrieves polarization Q
;    POLU = retrieves polarization U
; SIDE EFFECTS:
;    None
; EXAMPLES:
;     Nside=256
;     vec=psm_thermal_dust(lindgen(12L*Nside^2),100.)
; RESTRICTIONS:
;    The gal_model_init routine must have been executed before
; PROCEDURES CALLED:
;    None
; REVISION HISTORY:
;    Created by Marc-Antoine Miville-Deschenes (Sept 06, 2011)
;-

IF n_params() EQ 0 THEN BEGIN
  print,'vec=psm_thermal_dust(pix_num,freq [,PolQ=][,PolU=][,/no_polar])'
  goto,exit
ENDIF

output = psm_sfd(pix_num,freq,polQ=polQ,polU=polU)

RETURN, output
exit:


END
