; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	EMISSIONPARAM
;
; PURPOSE:
;	Components in the PSM are defined as a sum of emission
;	processes, each of which defined with a set of
;	emission-specific parameters. This function is an utility that
;	permits to access rapidly the value of a specific parameter of
;	one of the emission processes of a component.
;
; CATEGORY:
;	Sky model
;
; CALLING SEQUENCE:
;	EMISSIONPARAM, Comp, Tag, Iem
;
; INPUTS:
;	Comp:	The input component structure, comprising one or more
;	        emission processe(s).
;	Tag:	The tag for the parameter of interest
;	Iem:	The emission number (starting at 1)
;
; OUTPUTS:
;	Value of a parameter in an component emission process.
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
; 	01/12/2009:	Jacques Delabrouille, First version.
; 	06/01/2012:	JD, doc and PSM license header.
;-
FUNCTION EMISSIONPARAM, comp, tag, iem, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'EMISSIONPARAM: function which returns the value of a tag for an emission process in an component'
     PSM_INFO_MESSAGE, 'SYNTAX: result = EMISSIONPARAM(comp, tag, iem)'
     PSM_INFO_MESSAGE, 'instr is a component structure, part of the sky model'
     PSM_INFO_MESSAGE, "tag is a string, the tag name of the parameter to return (example: 'specind.value')"
     PSM_INFO_MESSAGE, "iem is the emission number (starting at 1)"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  ok = EXECUTE('res = comp.e'+NUMBER2STRING(iem)+'.'+tag)
  RETURN, res

END


