; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Jacques Delabrouille
;   Guillaume Castex
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PSM_ALLCOMP
;
; PURPOSE:
;	This function returns the list of valid PSM component names
;	(some of these components (cold cores, zle) are not
;	implemented yet, but we are thinking about it very
;	seriously).
;
; CATEGORY:
;	Sky model
;
; CALLING SEQUENCE:
;	result = PSM_ALLCOMP()
;
; INPUTS:
;	None
;
; KEYWORD PARAMETERS:
;	None
;
; OUTPUTS:
;	a string array with the name of all valid PSM component names
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	29/10/2009:	Jacques Delabrouille, First version.
; 	21/06/2010:	JD, distinguish between strong and faint sources.
; 	06/07/2010:	JD, CO lines added ('co' component).
; 	01/09/2010:	Guillaume Castex, WMAP sources added.
; 	07/04/2011:	JD, Renamed strong and faint sources
; 	                (underscore '_' removed in names). Not sure it
; 	                is the best decision in my life, though...
; 	28/04/2011:	Guillaume Castex, ERCSC sources added.
; 	06/01/2012:	JD, doc and PSM license header.
;-
FUNCTION PSM_ALLCOMP

;; RETURN, ['dipole','cmb','synchrotron','freefree','thermaldust','spindust','co','thermalsz','kineticsz','polarsz',$
;;          'strongirps','faintirps','strongradiops','faintradiops','stronguchii','faintuchii',$
;;          'strongwmapps','faintwmapps','strongercscps','faintercscps','strongcoldcore','faintcoldcore','firb','zle']
RETURN, ['dipole','cmb','synchrotron','freefree','thermaldust','spindust','co','thermalsz','kineticsz','polarsz',$
         'strongirps','faintirps','strongradiops','faintradiops','stronguchii','faintuchii',$
         'strongwmapps','faintwmapps','strongercscps','faintercscps','firb']

END
