; -----------------------------------------------------------------------------
;
; Copyright (C) 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	SKY_POLARISED
;
; PURPOSE:
;	Obtain the sky simulation in a PSM session is polarised or
;	not. This routine is a convenient interface that gives
;	access to the information while hiding the (non-stable)
;	practical way in which the information is stored internally by
;	the PSM. 
;
; CATEGORY:
;	Sky model
;
; CALLING SEQUENCE:
;	return, SKY_POLARISED()
;
; INPUTS:
;	None
;
; KEYWORD PARAMETERS:
;	None
;
; OUTPUTS:
;	Whether or not the sky is polarised (Byte, 0 or 1)
;
; COMMON BLOCKS:
;	SKY:	model
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
; 	06/10/2010:	Jacques Delabrouille, First version.
; 	06/01/2012:	JD, doc and PSM license header.
;-
FUNCTION SKY_POLARISED

  COMMON sky, model
  CASE model.global.sky_fields OF
     'T': RETURN, 0
     'TP': RETURN, 1
  ENDCASE

END
