; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2011
;   Jacques Delabrouille
;   Guillaume Castex
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; Function to create a catalogue of sources with each an arbitrary number of arbitrary emission processes (e.g. with greybody and powerlaw emission laws)
; 
; @categories ps
; 
; @param nsources {in}{required}{type=long64} Number of sources in the catalogue 
; @param em_law {in}{required}{type=array of string} List of emission laws for each source 
; @history <p>17/04/2009: First version, Jacques Delabrouille</p>
; @history <p>18/04/2009: Added commontags keyword, Jacques Delabrouille</p>
; @history <p>28/04/2009: Minor reorganisation of the PSM header, Jacques Delabrouille</p>
; @history <p>13/02/2010: Completely rewritten, Jacques Delabrouille</p>
; @copyright Jacques Delabrouille, CNRS (2009)
;-
FUNCTION CREATE_PS_MODEL_CAT, nsources, emlaw_list, name=name, id=id, polarised=polarised, polar_model=polar_model, elliptic=elliptic, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'CREATE_PS_MODEL_CAT: function to create a point source model catalog'
     PSM_INFO_MESSAGE, 'SYNTAX: result = CREATE_PS_DATA_CAT(nsources, emlaw_list, name=, id=, /polarised, /elliptic)'
     PSM_INFO_MESSAGE, "nsources is the number of sources in the catalogue"
     PSM_INFO_MESSAGE, "emlaw_list: array of string (emission law names for all emission processes of the sources)"
     PSM_INFO_MESSAGE, "'name' is the catalogue name (default empty string)"
     PSM_INFO_MESSAGE, "set 'id' to the id key of the catalogue (default: generated by the code)"
     PSM_INFO_MESSAGE, "set /polarised for a polarised catalogue"
     PSM_INFO_MESSAGE, "if polarised, polar_model can be 'GLOBAL' (the default)"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF
  
  cat = COMPONENT_STRUCT(emlaw_list=emlaw_list, name=name, id=id, polarised=polarised, elliptic=elliptic, class='pointsource')
  *cat.objectname = STRARR(nsources)
  *cat.lon.value = DBLARR(nsources)
  *cat.lat.value = DBLARR(nsources)
  cat.nsources = nsources

  IF NOT KEYWORD_SET(id) THEN PSM_ID_KEYGEN, id
  cat.id = id

  RETURN, cat

END


  
  

