; -----------------------------------------------------------------------------
;
; Copyright (C) 2007, 2009
;   Jacques Delabrouille
;   Gianfranco de Zotti
;   Joaquin Gonzalez-Nuevo
;   Marcella Massardi
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
;-
;+
; Software to generate the input radiosource catalogues at 5 GHz and 20 GHz
; The generated catalogues include fluxes, spectral indices and polarisation properties
;
; @categories ps
;
; @keyword query {in}{optional}{type=boolean} If set, an online query on Vizier generates a new data catalogue, (which is used to build the input catalogues) 
;
; @uses 
;
; @history <p>06/03/2007: First version, adapting the output file of rs_cat_gen of Marcella Massardi, Joaquin Gonzlez-Nuevo </p>
; @history <p>17/04/2009: Update the documentation + minor cleanup, Jacques Delabrouille </p>
; @copyright Jacques Delabrouille, Joaquin Gonzalez-Nuevo, Marcella Massardi, CNRS, 2007
;-
PRO GENERATE_INPUT_CATALOGUE, query=query

  ;; Query a new catalogue if needed
  ;;---------------------------------
  IF  KEYWORD_SET(query) then begin
     PSM_DEBUG_MESSAGE, 'This part is not working, check it'
     stop
     ;PRINT,'...QUERY process...'
     ;psmroot = FIX_SEPARATOR(!PSMROOT)
     ;RS_CAT_GEN, psmroot+'data/Intermediate/ps/cat_in.txt', psmroot+'data/Intermediate/ps/cat_cc.txt' , psmroot+'data/Intermediate/ps/ps_radio_input_cat'
  ENDIF

  ;; Open unit
  ;;-----------
  psdatafile = GETPSMDIR('psmdata')+'ancillary/models/radiosources/ps_radio_input_cat_out.dat'
  ok = FILE_TEST(psdatafile)
  IF NOT ok THEN GET_PSM_DATA, datafile=psdatafile
  OPENR, unit, psdatafile, /get_lun


  ;; Count number of lines in file 
  ;;-------------------------------
  i = FILE_LINES(GETPSMDIR('psmdata')+'ancillary/models/radiosources/ps_radio_input_cat_out.dat')


  ;; Read data 
  ;;-----------
  point_lun,unit,0
  cat0=fltarr(7,i)
  readf,unit,cat0
  close,unit
  free_lun,unit

  ;; Eliminate outlayers in the spectral indices
  ;;---------------------------------------------
  pscorr_id=WHERE(FINITE(cat0[6,*]) EQ 1,n_pscorr_id)
  md=median(cat0[6,pscorr_id])
  s=stddev(cat0[6,pscorr_id])
  alpha_out=where(cat0[6,pscorr_id] lt md-3*s OR cat0[6,pscorr_id] gt md+3*s,n_alpha_out)
  cat0[6,pscorr_id[alpha_out]]=randomn(getseed(),n_alpha_out)*s+md
  alpha_out=0.

  ;; Duplicate spectral indices for sources observed only at 5 GHz
  ;;---------------------------------------------------------------
  nan_id=WHERE(cat0[4,*] eq 0.,n_nan_id)
  if (n_nan_id gt 0) then begin
     dup_id=floor(randomu(getseed(),n_nan_id)*n_pscorr_id)
     cat0[6,nan_id]=cat0[6,pscorr_id[dup_id]]
  endif
  pscorr_id=0.
  nan_id=0.

  ;; SUMSS problem...
  ;;------------------
  nan_id=WHERE(FINITE(cat0[6,*]) eq 0 AND cat0[5,*] eq 0.84,n_nan_id)
  mu=1.0 & sigma=0.3
  if (n_nan_id ge 0) then begin
     alpha=randomn(getseed(),n_nan_id)*sigma+mu
     cat0[6,nan_id]=alpha
  endif


  ;; Add a spectral index for sources observed at nu<5GHz
  ;; Distribution based on Fomalont et al.(1991)
  ;; (S>50mJy 0.44; S<50mJy 0.8)
  ;;------------------------------------------------------

  nan_id=WHERE(FINITE(cat0[6,*]) eq 0 AND cat0[4,*] gt 50.,n_nan_id)
  ;mu=0.44 & sigma=0.3
  mu=0.8 & sigma=0.3
  if (n_nan_id ge 0) then begin
     alpha=randomn(getseed(),n_nan_id)*sigma+mu
     cat0[6,nan_id]=alpha
  endif
  
  nan_id=WHERE(FINITE(cat0[6,*]) eq 0 AND cat0[4,*] le 50.,n_nan_id)
  mu=0.80 & sigma=0.3
  if (n_nan_id ge 0) then begin
     alpha=randomn(getseed(),n_nan_id)*sigma+mu
     cat0[6,nan_id]=alpha
  endif

  ;; Calculate the flux at 4.85GHz
  ;;-------------------------------
  nan_id=WHERE(FINITE(cat0[2,*]) eq 0,n_nan_id)
  if (n_nan_id ge 0) then begin
     cat0[3,nan_id]=4.85
     cat0[2,nan_id]=cat0[4,nan_id]*(cat0[3,nan_id]/cat0[5,nan_id])^(-cat0[6,nan_id])
  endif

  ;w=where(cat0[2,*] gt 0.)
  ;cat0=cat0[*,w]

  cat=fltarr(6,n_elements(cat0[0,*]))
  cat[0:2,*]=cat0[0:2,*]
  cat[3,*]=cat0[6,*]

  cat0=0.

  ;; From Eq. coord. to Gal. coord.
  ;;--------------------------------
  euler,cat[0,*],cat[1,*],glon,glat,1
  cat[0,*]=glon
  cat[1,*]=glat
  glon=0.
  glat=0.

  name_pol_deg_dist_flat = GETPSMDIR('psmdata')+'ancillary/models/radiosources/pol_deg_dist_flat.dat'
  name_pol_deg_dist_steep = GETPSMDIR('psmdata')+'ancillary/models/radiosources/pol_deg_dist_steep.dat'

  ok = FILE_TEST(name_pol_deg_dist_flat)
  IF NOT ok THEN GET_PSM_DATA, datafile=name_pol_deg_dist_flat
  ok = FILE_TEST(name_pol_deg_dist_steep)
  IF NOT ok THEN GET_PSM_DATA, datafile=name_pol_deg_dist_steep

  ;;... Adding polarization properties...
  ;;--------------------------------------

    ;; Read alpha distribution for flat spectrum sources
    hor=fltarr(200) & ver=fltarr(200)
    i=0
    openr,1,name_pol_deg_dist_flat
    while not eof(1) do begin
       readf,1,a,b
       hor[i]=a & ver[i]=b
       i=i+1
    endwhile
    close,1
    hor=hor[0:i-1] & ver=ver[0:i-1]
    ver=smooth(ver,3.)
    posf=where(cat[3,*] lt 0.5)
    N_elem=n_elements(posf)
    r=rand_num_dist_file(N_elem/0.2,hor,ver)
    r=r[0:N_elem-1]
    cat[4,posf]=r
    posf=0.

    ;; Read alpha distribution for steep spectrum sources
    hor=fltarr(200) & ver=fltarr(200)
    i=0
    openr,1,name_pol_deg_dist_steep
    while not eof(1) do begin
       readf,1,a,b
       hor[i]=a & ver[i]=b
       i=i+1
    endwhile
    close,1
    hor=hor[0:i-1] & ver=ver[0:i-1]
    ver=smooth(ver,20.)
    ver=smooth(ver,2.)
    
    poss=where(cat[3,*] ge 0.5,N_elem)
    if (N_elem gt 0) then begin
       r=rand_num_dist_file(N_elem/0.2,hor,ver)
       r=r[0:N_elem-1]
       
       cat[4,poss]=r
       poss=0.
    endif

    ;; Random Polarization Angle
    cat[5,*] = !PI*randomu(getseed(),n_elements(cat[4,*]))

    ;; Remove source number 1814716 (vanishing 5 GHz and 20 GHz flux)
    keep = LINDGEN(n_elements(cat[4,*]))
    wh = WHERE(keep NE 1814716)
    cat = cat[*,wh]

    mkhdr,hdr,cat
    sxaddpar,hdr,'Columns','Glon Glat S alpha Pol_deg Pol_ang','S=nu^alpha',before='DATE'
    sxaddpar,hdr,'Units','Degree Degree mJy --- % rad',before='DATE'

    writefits,GETPSMDIR('ps')+'rs_input_cat_5GHz.fits',cat,hdr
    

  ;;...frequency extrapolation (for nu>5 GHz) of fluxes using spectral indices...
  ;;------------------------------------------------------------------------------

    ;; Steep
    mu=0.45 & sigma=0.3         ; Data coming from Ricci et al. (2006) (see data_info folder for more information)
    w=where(cat[3,*] ge 0.5)
    if (w[0] ge 0) then begin
       delta_alpha_18=randomn(getseed(),n_elements(w))*sigma+mu
       cat[3,w]=cat[3,w]+delta_alpha_18
    endif

    ;; Flat
    mu=0.24 & sigma=0.3         ; Data coming from Ricci et al. (2006) (see data_info folder for more information)
    w=where(cat[3,*] lt 0.5)
    if (w[0] ge 0) then begin
       delta_alpha_18=randomn(getseed(),n_elements(w))*sigma+mu
       cat[3,w]=cat[3,w]+delta_alpha_18
    endif

    nu0=4.85                    ; GHz , Starting Frequency

    ;; Change in the spectral index for nu>20 GHz
    cat[2,*]=(20./nu0)^(-cat[3,*])*cat[2,*]
    nu0=20.
    mu=0.29 & sigma=0.43        ; Data coming from NEWPS (see data_info folder for more information)
    delta_alpha_61=randomn(getseed(),n_elements(cat[3,*]))*sigma+mu
    cat[3,*]=cat[3,*]+delta_alpha_61

    md=median(cat[3,*])
    s=stddev(cat[3,*])
    alpha_out=where(cat[3,*] lt md-3*s OR cat[3,*] gt md+3*s,n_alpha_out)
    cat[3,alpha_out]=randomn(getseed(),n_alpha_out)*s+md
    alpha_out=0.

    mkhdr,hdr,cat
    sxaddpar,hdr,'Columns','Glon Glat S alpha Pol_deg Pol_ang','S=nu^alpha',before='DATE'
    sxaddpar,hdr,'Units','Degree Degree mJy --- % rad',before='DATE'
    writefits,GETPSMDIR('ps')+'rs_input_cat_20GHz.fits',cat,hdr
    cat=0.

END
