; -----------------------------------------------------------------------------
;
; Copyright (C) 
;   Joaquin Gonzalez-Nuevo
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
;-
PRO generate_input_lensing_catalogue,amplitude_polar,Slim
;+
; NAME:
;    generate_input_lensing_catalogue
; PURPOSE:
;    Generate the input of lensed Spheroids catalogue at 857 GHz
; CALLING SEQUENCE:
;    generate_input_lensing_catalogue
; INPUT PARAMETERS:
;
; OUTPUT:
;    A lensed source catalogue at 857 GHz (with fluxes, spectral idexes and polarization properties)
; EXAMPLES:
;    generate_input_lensing_catalogue
;
; REVISION HISTORY:
; May 8, Developed by Joaquin González-Nuevo
;-

; Read the integrated lensed spheroids counts at 857GHz
; Perrotta et al.

GET_LUN, unit
openr,unit,!PSMROOT+'/data/Intermediate/ps/lensing857GHz.dat'
i=0L
while not eof(unit) do begin
      	readf,unit,a
	i=i+1
endwhile
point_lun,unit,0
c=fltarr(4,i)
readf,unit,c
FREE_LUN, unit

hor=alog10(c[0,*]/1.5); 1.5 is a correction factor to account for the last observational results
bin=abs(hor[1]-hor[2])
a=shift(c[3,*],-1)-c[3,*]
ver=[c[3,0],a[0:n_elements(a)-2]]/(!PI/180.)^2.
c=[hor[0,*],transpose(ver)]

c[1,*]=c[1,*]*4.*!PI
w=where(c[0,*] ge Slim,nbin)
ntot=round(total(c[1,w],/double))
print,ntot

;%% Latitude: cos (theta) random distribution
lat=(90.-randomu(getseed(),3*ntot,1)*180.)
ylat=randomu(getseed(),3*ntot,1)
wlat=where(ylat le cos(lat/180.*!PI))
lat=lat[wlat]
lat=lat[0:ntot-1]

;%% Longitude: random distribution
lon=360*randomu(getseed(),ntot,1)

;%% Fluxes following the counts
id=0
flux=fltarr(ntot*3)
for i=0,nbin-1 do begin
	ns=round(c[1,i])
	if (ns gt 0) then begin
		flux(id:id+ns-1)=c[0,i]-bin*randomu(getseed(),ns,1)
		id=id+ns;
	endif
endfor
flux=10^flux[0:ntot-1]

; Polarization properties
perc_polar = (amplitude_polar * (randomn(getseed(),ntot))^2) < 1
polang_rad = 2.*!PI*randomu(getseed(),ntot)

newcat = replicate({name:"",glon_deg:0.d,glat_deg:0.d, $
                        F_mjy:0.0, $
                        polang_rad:0.0,perc_polar:0.0}$
                       , n_elements(lon))
newcat.glon_deg = lon
newcat.glat_deg = lat
newcat.F_mjy = flux*1000.
newcat.polang_rad= polang_rad
newcat.perc_polar= perc_polar*100.

outfilename = fix_separator(!PSMROOT) + 'data/Intermediate/ps/intermediate_lensing_cat.fits'
mwrfits, newcat, outfilename, /create
;writefits,'./lensing.fits',[[lon],[lat],[flux],[transpose(lenspolar[0,*])],[transpose(lenspolar[1,*])]]

END
