; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2011
;   Jacques Delabrouille
;   Marc Betoule
;   Guillaume Castex
;   Gianfranco de Zotti
;   Joaquin Gonzalez-Nuevo
;   Maude Le Jeune
;   Samuel Leach
;   Marcella Massardi
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; Computes radio sources flux extrapolation and drive the routine to generate a catalog of strongest sources and maps of weakest one.
; 
;
; @categories ps
; 
; @param gp {in}{required}{type=global_parameters} general parameters of the simulation
; @param Slim {in}{required}{type=double} flux limit for strong sources in mJy
; @keyword newcat {in}{optional}{type=string} 
; @keyword nopol {in}{optional}{type=boolean} if set don't take into account
; the polarisation.
; @keyword outputdir {in}{optional}{type=string} where to save the catalog
; @keyword outputcatfile {out}{optional}{type=string} name of the catalog
;
;
; @history <p>19/07/2007: First version, Maude Le Jeune</p>
; @history <p>28/03/2008: Check strong sources existence before making catalog and map, Maude Le Jeune</p>
; @history <p>18/02/2009: Major rewriting, Jacques Delabrouille</p>
;-

PRO JGN_READ_INPUT_RADIOSOURCE_CAT, newcat, polarised=polarised, psm_hdr=psm_hdr


  input_file =  GETPSMDIR('psmdata')+'ancillary/models/radiosources/ps_radio_input_cat_out.dat'

; count number of lines in file ----------------------------------------------------------
  nsources = FILE_LINES(input_file) - 1    ;; Accounts for the fu@?!*ng missing source number 1814716

; open unit ------------------------------------------------------------------------------
  GET_LUN, unit
  OPENR, unit, input_file
  
; initialise the catalog -----------------------------------------------------------------
  freq = [0.84,1.4,4.85]*1e9
  band_list = PTRARR(3,/allocate_heap)
  FOR i=0, 2 DO *band_list[i] = GET_BAND_STRUCT('DIRAC', nu_c=freq[i]) 
  newcat = CREATE_PS_DATA_CAT(nsources, band_list, name='Radio_Ancillary_Cat', id=id, polarised=polarised)
  FOR i=0, 2 DO PTR_FREE, band_list[i]
  *newcat.obs1.iflux = REPLICATE(FLOAT('NAN'),nsources)
  *newcat.obs2.iflux = REPLICATE(FLOAT('NAN'),nsources)
  *newcat.obs3.iflux = REPLICATE(FLOAT('NAN'),nsources)

; read data from radiosource catalogue ---------------------------------------------------

  lon = DBLARR(nsources)
  lat = lon

  j = 0L
  FOR k=0L, nsources-1L DO BEGIN
     a = DBLARR(7)
     READF, unit, a

     ;; Remove source number 1814716 (vanishing 5 GHz and 20 GHz flux)
     
     IF k NE 1814716L THEN BEGIN
        lon[j]=a[0]
        lat[j]=a[1]
        
        IF FINITE(a[2]) THEN BEGIN
           IF FLOAT(a[3]) EQ 4.85 THEN BEGIN
              (*newcat.obs3.iflux)[j] = a[2]
           ENDIF
           IF FLOAT(a[3]) EQ 1.4 THEN BEGIN
              (*newcat.obs2.iflux)[j] = a[2]
           ENDIF
           IF FLOAT(a[3]) EQ 0.84 THEN BEGIN
              (*newcat.obs1.iflux)[j] = a[2]
           ENDIF
        ENDIF
        IF FINITE(a[4]) THEN BEGIN
           IF FLOAT(a[5]) EQ 4.85 THEN BEGIN
              (*newcat.obs3.iflux)[j] = a[4]
           ENDIF
           IF FLOAT(a[5]) EQ 1.4 THEN BEGIN
              (*newcat.obs2.iflux)[j] = a[4]
           ENDIF
           IF FLOAT(a[5]) EQ 0.84 THEN BEGIN
              (*newcat.obs1.iflux)[j] = a[4]
           ENDIF
        ENDIF
        j = j+1L
     ENDIF

  ENDFOR

  ; Rotate coordinates from equatorial to galactic ---------------------------------------
  dtor = !DPI/180D0
  EULER, lon*dtor, lat*dtor, glon_rad, glat_rad, 1, /radian
  (*newcat.lon.value) = TEMPORARY(glon_rad)
  (*newcat.lat.value) = TEMPORARY(glat_rad)

  ; Source names -------------------------------------------------------------------------
  *newcat.objectname = 'radiosource_'+NUMBER2STRING(LINDGEN(nsources))

  newcat.obs1.unit = 'mJy'
  newcat.obs2.unit = 'mJy'
  newcat.obs3.unit = 'mJy'

; close unit -----------------------------------------------------------------------------
  CLOSE, unit
  FREE_LUN, unit

; write the ancillary catalog ------------------------------------------------------------
  COMMON psm, param

  IF param.tasks.write_ancillary EQ 'yes' THEN BEGIN

     dir = GETPSMDIR('ancillary')
     dir_exists = FILE_TEST(dir, /directory)
     IF NOT dir_exists THEN FILE_MKDIR, dir
     
     filename = FIX_SEPARATOR(dir)+'radiops_ancil_cat.sav'
     WRITE_CATALOG, newcat, filename, /sav
  ENDIF

END

;==============================================================================================================================

PRO JGN_RADIOSOURCE_DATA_TO_MODEL, data_cat, model_cat, polarised=polarised

;;;;; NEEDS COMPLETE UPDATE
;;;;; NOT USED FOR THE MOMENT

  nsources = data_cat.nsources
  npowerlaw = 2
  units = 'mJy'
  model_cat = CREATE_RADIOSRC_MODEL_CAT('radiosource_model', nsources, npowerlaw, units=units, polarised=polarised, /usenan)
  
  model_cat.source.name = data_cat.source.name
  model_cat.source.glon_rad = data_cat.source.glon_rad
  model_cat.source.glat_rad = data_cat.source.glat_rad

  model_cat.source.nuref[0:1,*] = [5.,20.]*1e9
  model_cat.source.minrange[0:1,*] = [1.,20.]*1e9
  model_cat.source.maxrange[0:1,*] = [20.,3000.]*1e9


; spectral index and flux(5) for sources measured at two frequencies -----------------------

  freq = [0.84, 1.4, 4.85]
  limit = 20.                                            ;do not trust data below this many mJy for computing a spectral index
  FOR i=0L, nsources-1 DO BEGIN
     ok = WHERE(finite(data_cat.source[i].iflux), nok)
     IF nok GE 2 THEN BEGIN 
        nu = freq[ok]
        flux = data_cat.source[i].iflux(ok)
        
        IF MIN(flux) GE limit THEN BEGIN
           specind = ALOG(flux(1)/flux(0)) / ALOG(nu(1)/nu(0))
           model_cat.source[i].specind[0] = specind
           model_cat.source[i].valueref[0] = flux(0)*(5./nu(0))^specind
           
        ENDIF
     ENDIF
  ENDFOR   

; To eliminate outlayers in the spectral indexes --------------------------------------------

  pscorr_id = WHERE(FINITE(model_cat.source.specind[0]) EQ 1 ,n_pscorr_id)
  md = MEDIAN(model_cat.source[pscorr_id].specind[0])
  s = STDDEV(model_cat.source[pscorr_id].specind[0])
  alpha_out = WHERE(model_cat.source[pscorr_id].specind[0] LT md-3*s OR model_cat.source[pscorr_id].specind[0] GT md+3*s,n_alpha_out)
  model_cat.source[pscorr_id[alpha_out]].specind[0] = RANDOMN(GETSEED(),n_alpha_out)*s+md
  alpha_out = 0B

; Duplicate spectral indexes for sources observed only at 4.85 GHz --------------------------
  
  ind485 = WHERE(freq EQ 4.85)
  nan_id = WHERE(FINITE(data_cat.source.iflux[ind485]) AND (NOT FINITE(model_cat.source.specind[0])), n_nan_id)

  IF (n_nan_id GT 0) THEN BEGIN
     dup_id = FLOOR(RANDOMU(GETSEED(),n_nan_id)*n_pscorr_id)
     model_cat.source[nan_id].specind[0] =  model_cat.source[pscorr_id[dup_id]].specind[0]
     ; Write the 5GHz flux  
     model_cat.source[nan_id].valueref[0] = data_cat.source[nan_id].iflux[ind485] * (5./4.85)^model_cat.source[nan_id].specind[0]
  ENDIF
  
; Free some memory --------------------------------------------------------------------------

  pscorr_id = 0B
  nan_id = 0B

END

;==============================================================================================================================

PRO JGN_READ_MODEL_CAT, data_cat, newmodel_cat, polarised=polarised, psm_hdr=psm_hdr

  nsources = data_cat.nsources
  npowerlaw = 4
  units = 'mJy'

  emlaw_list = ['POWERLAW','POWERLAW','POWERLAW','POWERLAW']
  newmodel_cat = CREATE_PS_MODEL_CAT(nsources, emlaw_list, name=name, id=id, polarised=polarised)

  *newmodel_cat.objectname = *data_cat.objectname

  *newmodel_cat.e1.nuref.value = 4.85e9
  *newmodel_cat.e1.numin.value = 0.
  *newmodel_cat.e1.numax.value = 4.85e9

  *newmodel_cat.e2.nuref.value = 4.85e9
  *newmodel_cat.e2.numin.value = 4.85e9
  *newmodel_cat.e2.numax.value = 20.0e9

  *newmodel_cat.e3.nuref.value = 20.0e9
  *newmodel_cat.e3.numin.value = 20.0e9
  *newmodel_cat.e3.numax.value = 1.0e11

  *newmodel_cat.e4.nuref.value = 1.0e11
  *newmodel_cat.e4.numin.value = 1.0e11
  *newmodel_cat.e4.numax.value = FLOAT('INF')

  ;; model emission below 4.85 GHz
  ;;-------------------------------
  cat5GHz = READFITS(GETPSMDIR('ps')+'rs_input_cat_5GHz.fits')

  IF nsources NE N_ELEMENTS(cat5GHz[0,*]) THEN MESSAGE, 'Number of sources is not what expected'

  dtor = (!DPI/180D0)

  *newmodel_cat.lon.value = *data_cat.lon.value
  *newmodel_cat.lat.value = *data_cat.lat.value

  IF KEYWORD_SET(polarised) NE 0 THEN BEGIN
     *newmodel_cat.pol_fraction = REFORM(cat5GHz[4,*]/100.)
     *newmodel_cat.pol_angle = REFORM(cat5GHz[5,*] + 2*!PI) MOD !PI
  ENDIF

  *newmodel_cat.e1.ampl.value = REFORM(cat5GHz[2,*])
  newmodel_cat.e1.ampl.unit = 'mJy'
  *newmodel_cat.e1.specind.value = -REFORM(cat5GHz[3,*])

  ;; model emission between 20 GHz and 100 GHz
  ;;-------------------------------------------
  cat20GHz = READFITS(GETPSMDIR('ps')+'rs_input_cat_20GHz.fits')

  IF nsources NE N_ELEMENTS(cat20GHz[0,*]) THEN MESSAGE, 'Number of sources is not what expected'

  IF KEYWORD_SET(polarised) NE 0 THEN BEGIN
     IF newmodel_cat.polar_model NE 'GLOBAL' THEN STOP
  ENDIF

  *newmodel_cat.e3.ampl.value = REFORM(cat20GHz[2,*])
  newmodel_cat.e3.ampl.unit = 'mJy'
  *newmodel_cat.e3.specind.value = -REFORM((TEMPORARY(cat20GHz))[3,*])

  ;; model emission between 4.85 GHz and 20 GHz
  ;;--------------------------------------------
  newmodel_cat.e2.ampl.value = newmodel_cat.e1.ampl.value            ; saves memory
  newmodel_cat.e2.ampl.unit = 'mJy'
  logfreqratio = ALOG(20./4.85)
  *newmodel_cat.e2.specind.value = ALOG(*newmodel_cat.e3.ampl.value/*newmodel_cat.e2.ampl.value)/logfreqratio

  ;; model emission above 100 GHz
  ;;------------------------------
  *newmodel_cat.e4.ampl.value = 5.^(*newmodel_cat.e3.specind.value)  *  (*newmodel_cat.e3.ampl.value)
  newmodel_cat.e4.ampl.unit = 'mJy'
  md = MEDIAN(*newmodel_cat.e3.specind.value)
  s = STDDEV(*newmodel_cat.e3.specind.value)
  alpha_out = WHERE((*newmodel_cat.e3.specind.value LT md-2.5*s) OR (*newmodel_cat.e3.specind.value GT md+2.5*s), nout)
  *newmodel_cat.e4.specind.value = *newmodel_cat.e3.specind.value
  IF nout NE 0 THEN (*newmodel_cat.e4.specind.value)[alpha_out] = md

END
  

;==============================================================================================================================
; MAIN PROGRAMME
;==============================================================================================================================

PRO PSM_PS_MULTIFREQ_RADIO, query=query, new_cat=new_cat, polarised=polarised

  COMMON psm, param
  COMMON sky, model

  gp = param.global
  runp = param.run
  names = param.names
  
;; Initial seed for radio point source pipeline
;;----------------------------------------------
  SET_PSM_SEEDCHAIN, 'ps-pipe-radio'

;; Generate the point source data catalogue with data points at 0.84, 1.4 and 4.85 GHz
;;-------------------------------------------------------------------------------------
  IF KEYWORD_SET(new_cat) THEN BEGIN
     IF runp.verbosity GE 1 THEN PSM_INFO_MESSAGE,'Generating new input radiosource catalogue at 5 GHz and 20 GHz'
     IF KEYWORD_SET(query) EQ 0 THEN BEGIN
        IF runp.verbosity GE 1 THEN PSM_INFO_MESSAGE, '     (Using standard initial catalogues at 0.84, 1.4 and 4.85 GHz)'
        IF runp.verbosity GE 2 THEN PSM_INFO_MESSAGE, '     (For a new initial catalogue use /query)'
        IF runp.verbosity GE 2 THEN PSM_INFO_MESSAGE, '     (Need internet and take around 10 hours)'
     ENDIF
     GENERATE_INPUT_CATALOGUE, query=query
  ENDIF

;; Read the input catalog data
;;-----------------------------
  JGN_READ_INPUT_RADIOSOURCE_CAT, data_cat, psm_hdr=data_cat_header

;; Read the input catalog model
;;------------------------------
  IF runp.verbosity GE 1 THEN PSM_INFO_MESSAGE,'Modeling radiosource emission'
  JGN_READ_MODEL_CAT, data_cat, model_cat, polarised=polarised, psm_hdr=model_cat_header
  
;; Read the WMAP catalogue and adjust emission laws to match WMAP observations
;;-----------------------------------------------------------------------------
 
  IF model.ps.include_wmap_sources EQ 'yes' THEN BEGIN
     mcat = PTR_NEW(model_cat)
     dcat = PTR_NEW(data_cat)
     PSM_WMAP_SOURCES, dcat, mcat, polarised=polarised
  ENDIF

;; Free memory
;;-------------

  DESTROY_DATA, data_cat

;; Select which sources are strong and which ones are faint
;;----------------------------------------------------------

  nfreq = N_ELEMENTS(model.ps.strong_ps_limit_freq_ghz)
  source_is_strong = BYTARR(model_cat.nsources)
  
  FOR i=0, nfreq-1 DO BEGIN
     band = GET_BAND_STRUCT('DIRAC',nu_c=1e9*model.ps.strong_ps_limit_freq_ghz[i])
     fluxlimitmJy = 1000*model.ps.strong_ps_limit_flux_jy[i]
     OBSERVE_MODEL_CAT, model_cat, band, result
     wh = WHERE(result GE fluxlimitmJy, nwh)
     IF nwh NE 0 THEN source_is_strong[wh] = 1B
  ENDFOR

  whstrong = WHERE(source_is_strong, nstrong, complement=whfaint, ncomplement=nfaint)   

;; Write the radiosource catalog model
;;-------------------------------------

  emlaw_list = ['POWERLAW','POWERLAW','POWERLAW','POWERLAW']


;;strong catalogue---------------------
IF nstrong GT 0 THEN BEGIN

  newcat1 = CREATE_PS_MODEL_CAT(nstrong, emlaw_list, name='strong'+names.radiops_basename, id=id, polarised=polarised)
  *newcat1.objectname = (*model_cat.objectname)[whstrong]

  IF polarised THEN BEGIN
     *newcat1.pol_fraction = (*model_cat.pol_fraction)[whstrong]
     *newcat1.pol_angle = (*model_cat.pol_angle)[whstrong]
  ENDIF

  *newcat1.e1.nuref.value = (*model_cat.e1.nuref.value)
  *newcat1.e1.numin.value = (*model_cat.e1.numin.value)
  *newcat1.e1.numax.value = (*model_cat.e1.numax.value)

  *newcat1.e2.nuref.value = (*model_cat.e2.nuref.value)
  *newcat1.e2.numin.value = (*model_cat.e2.numin.value)
  *newcat1.e2.numax.value = (*model_cat.e2.numax.value)

  PSM_DEBUG_MESSAGE, 'Implement memory saver by using the same reference frequency for 3rd and 4th emission', routine='PSM_PS_MULTIFREQ_RADIO', infolimit=2, stoplimit=3

  *newcat1.e3.nuref.value = (*model_cat.e3.nuref.value)
  *newcat1.e3.numin.value = (*model_cat.e3.numin.value)
  *newcat1.e3.numax.value = (*model_cat.e3.numax.value)

  *newcat1.e4.nuref.value = (*model_cat.e4.nuref.value)
  *newcat1.e4.numin.value = (*model_cat.e4.numin.value)
  *newcat1.e4.numax.value = (*model_cat.e4.numax.value)

  *newcat1.lon.value = (*model_cat.lon.value)[whstrong]
  *newcat1.lat.value = (*model_cat.lat.value)[whstrong]

  *newcat1.e1.ampl.value = (*model_cat.e1.ampl.value)[whstrong]
  newcat1.e1.ampl.unit = model_cat.e1.ampl.unit
  *newcat1.e1.specind.value = (*model_cat.e1.specind.value)[whstrong]

  *newcat1.e2.ampl.value = (*model_cat.e2.ampl.value)[whstrong]
  newcat1.e2.ampl.unit = model_cat.e2.ampl.unit
  *newcat1.e2.specind.value = (*model_cat.e2.specind.value)[whstrong]

  *newcat1.e3.ampl.value = (*model_cat.e3.ampl.value)[whstrong]
  newcat1.e3.ampl.unit = model_cat.e3.ampl.unit
  *newcat1.e3.specind.value = (*model_cat.e3.specind.value)[whstrong]

  *newcat1.e4.ampl.value = (*model_cat.e4.ampl.value)[whstrong]
  newcat1.e4.ampl.unit = model_cat.e4.ampl.unit
  *newcat1.e4.specind.value = (*model_cat.e4.specind.value)[whstrong]

  filename = FIX_SEPARATOR(GETPSMDIR('ps')) + 'strong'+names.radiops_basename + '.sav'
  WRITE_CATALOG, newcat1, filename, /sav, name='comp'
  DESTROY_DATA, newcat1
ENDIF

;;faint catalogue---------------------
IF nfaint GT 1 THEN BEGIN

  newcat2 = CREATE_PS_MODEL_CAT(nfaint, emlaw_list, name='faint'+names.radiops_basename, id=id, polarised=polarised)
  *newcat2.objectname = (*model_cat.objectname)[whfaint]

  IF polarised THEN BEGIN
     *newcat2.pol_fraction = (*model_cat.pol_fraction)[whfaint]
     *newcat2.pol_angle = (*model_cat.pol_angle)[whfaint]
  ENDIF  

  *newcat2.e1.nuref.value = (*model_cat.e1.nuref.value)
  *newcat2.e1.numin.value = (*model_cat.e1.numin.value)
  *newcat2.e1.numax.value = (*model_cat.e1.numax.value)

  *newcat2.e2.nuref.value = (*model_cat.e2.nuref.value)
  *newcat2.e2.numin.value = (*model_cat.e2.numin.value)
  *newcat2.e2.numax.value = (*model_cat.e2.numax.value)

  *newcat2.e3.nuref.value = (*model_cat.e3.nuref.value)
  *newcat2.e3.numin.value = (*model_cat.e3.numin.value)
  *newcat2.e3.numax.value = (*model_cat.e3.numax.value)

  *newcat2.e4.nuref.value = (*model_cat.e4.nuref.value)
  *newcat2.e4.numin.value = (*model_cat.e4.numin.value)
  *newcat2.e4.numax.value = (*model_cat.e4.numax.value)

  *newcat2.lon.value = (*model_cat.lon.value)[whfaint]
  *newcat2.lat.value = (*model_cat.lat.value)[whfaint]

  *newcat2.e1.ampl.value = (*model_cat.e1.ampl.value)[whfaint]
  newcat2.e1.ampl.unit = model_cat.e1.ampl.unit
  *newcat2.e1.specind.value = (*model_cat.e1.specind.value)[whfaint]

  *newcat2.e2.ampl.value = (*model_cat.e2.ampl.value)[whfaint]
  newcat2.e2.ampl.unit = model_cat.e2.ampl.unit
  *newcat2.e2.specind.value = (*model_cat.e2.specind.value)[whfaint]

  *newcat2.e3.ampl.value = (*model_cat.e3.ampl.value)[whfaint]
  newcat2.e3.ampl.unit = model_cat.e3.ampl.unit
  *newcat2.e3.specind.value = (*model_cat.e3.specind.value)[whfaint]

  *newcat2.e4.ampl.value = (*model_cat.e4.ampl.value)[whfaint]
  newcat2.e4.ampl.unit = model_cat.e4.ampl.unit
  *newcat2.e4.specind.value = (*model_cat.e4.specind.value)[whfaint]

  filename = FIX_SEPARATOR(GETPSMDIR('ps')) + 'faint'+names.radiops_basename + '.sav'
  WRITE_CATALOG, newcat2, filename, /sav, name='comp'
  DESTROY_DATA, newcat2
ENDIF

;; ending the generation of radiosources

  IF runp.verbosity GE 1 THEN PSM_INFO_MESSAGE, 'Radiosource model catalogues written'

  DESTROY_DATA, model_cat
  IF DEFINED(dcat) THEN PTR_FREE, dcat
  IF DEFINED(dcat) THEN PTR_FREE, mcat

  IF DEFINED(comp) THEN DESTROY_DATA, comp

END
