; -----------------------------------------------------------------------------
;
; Copyright (C) 2010
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
;-
FUNCTION SUM_CATALOGS, cat1, cat2, erase=erase

  IF cat1.type NE cat2.type THEN BEGIN
     PSM_INFO_MESSAGE, 'Cannot add catalogs of different types, returning'
     GOTO, closing
  ENDIF
  
  type = cat1.type
  
  CASE type OF
     'obs': BEGIN

        ;; Check that catalogs have the same number of observations
        IF cat1.nobs NE cat2.nobs THEN BEGIN
           PSM_INFO_MESSAGE, 'Cannot add catalogs with different number of observations, returning'
           GOTO, closing
        ENDIF

        ;; Check that catalogs observe in the same bands
        FOR i=1, cat1.nobs DO BEGIN
           ok = EXECUTE('bnd1 = *cat1.obs' +NUMBER2STRING(i)+ '.band')
           ok = EXECUTE('bnd2 = *cat1.obs' +NUMBER2STRING(i)+ '.band')
           IF NOT BANDS_EQUAL(bnd1,bnd2) THEN BEGIN
           PSM_INFO_MESSAGE, 'Cannot add catalogs with different bands of observations, returning'
           GOTO, closing
           ENDIF
        ENDFOR

        nsources = cat1.nsources + cat2.nsources
        band_list = PTRARR(cat1.nobs)
        FOR i=1, cat1.nobs DO ok = EXECUTE('band_list[i-1] = PTR_NEW(*cat1.obs' +NUMBER2STRING(i)+ '.band)')

        ispol = cat1.polarised OR cat2.polarised
        sumcat = CREATE_PS_DATA_CAT(nsources, band_list, polarised=ispol)

        ;; source names
        (*sumcat.objectname)[0:cat1.nsources-1] = *cat1.objectname
        (*sumcat.objectname)[cat1.nsources:*] = *cat2.objectname
        IF KEYWORD_SET(erase) NE 0 THEN BEGIN
           PTR_FREE, cat1.objectname
           PTR_FREE, cat2.objectname
        ENDIF

        ;; coordinates
        (*sumcat.lon.value)[0:cat1.nsources-1] = *cat1.lon.value
        (*sumcat.lon.value)[cat1.nsources:*] = *cat2.lon.value
        IF KEYWORD_SET(erase) NE 0 THEN BEGIN
           PTR_FREE, cat1.lon.value
           PTR_FREE, cat2.lon.value
        ENDIF
        (*sumcat.lat.value)[0:cat1.nsources-1] = *cat1.lat.value
        (*sumcat.lat.value)[cat1.nsources:*] = *cat2.lat.value
        IF KEYWORD_SET(erase) NE 0 THEN BEGIN
           PTR_FREE, cat1.lat.value
           PTR_FREE, cat2.lat.value
        ENDIF

        ;; fluxes
        PSM_DEBUG_MESSAGE, 'Process errors', routine='SUM_CATALOGS'
        FOR i=1, cat1.nobs DO BEGIN
           ok = EXECUTE('(*sumcat.obs'+NUMBER2STRING(i)+'.iflux) = REPLICATE(0*(*cat1.obs'+NUMBER2STRING(i)+'.iflux)[0],sumcat.nsources)')
           ok = EXECUTE('(*sumcat.obs'+NUMBER2STRING(i)+'.iflux)[0:cat1.nsources-1] = *cat1.obs'+NUMBER2STRING(i)+'.iflux')
           ok = EXECUTE('(*sumcat.obs'+NUMBER2STRING(i)+'.iflux)[cat1.nsources:*] = *cat2.obs'+NUMBER2STRING(i)+'.iflux')

           PSM_DEBUG_MESSAGE, 'Check units', routine='SUM_CATALOGS'
           ok = EXECUTE('sumcat.obs'+NUMBER2STRING(i)+'.unit = cat1.obs'+NUMBER2STRING(i)+'.unit')

           IF ispol THEN BEGIN
              ok = EXECUTE('(*sumcat.obs'+NUMBER2STRING(i)+'.qflux) = REPLICATE(0*(*cat1.obs'+NUMBER2STRING(i)+'.iflux)[0],sumcat.nsources)')
              ok = EXECUTE('(*sumcat.obs'+NUMBER2STRING(i)+'.uflux) = REPLICATE(0*(*cat1.obs'+NUMBER2STRING(i)+'.iflux)[0],sumcat.nsources)')
              IF cat1.polarised THEN BEGIN
                 ok = EXECUTE('(*sumcat.obs'+NUMBER2STRING(i)+'.qflux)[0:cat1.nsources-1] = *cat1.obs'+NUMBER2STRING(i)+'.qflux')
                 ok = EXECUTE('(*sumcat.obs'+NUMBER2STRING(i)+'.uflux)[0:cat1.nsources-1] = *cat1.obs'+NUMBER2STRING(i)+'.uflux')
              ENDIF
              IF cat2.polarised THEN BEGIN
                 ok = EXECUTE('(*sumcat.obs'+NUMBER2STRING(i)+'.qflux)[cat1.nsources:*] = *cat2.obs'+NUMBER2STRING(i)+'.qflux')
                 ok = EXECUTE('(*sumcat.obs'+NUMBER2STRING(i)+'.uflux)[cat1.nsources:*] = *cat2.obs'+NUMBER2STRING(i)+'.uflux')
              ENDIF
           ENDIF
        ENDFOR
        IF KEYWORD_SET(erase) NE 0 THEN BEGIN
           DESTROY_DATA, cat1
           DESTROY_DATA, cat2
        ENDIF
        RETURN, sumcat
     END
     ELSE: MESSAGE, 'Case not implemented'
  ENDCASE

closing:
END
