; -----------------------------------------------------------------------------
;
; Copyright (C) 2001-2009
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ADD_POSITIONS_TO_CATALOG
;
; PURPOSE:
;	...
;
;-
; This procedure is called in CREATE_SZ_SIMULATION in order to find
; the cluster position using the box Universe

FUNCTION REPIX, proba, map_dim_x, map_dim_y, underpix_pix
  result = FLTARR(map_dim_x,map_dim_y)
  s = SIZE(proba)
  n = N_ELEMENTS(result)
  pixels = LINDGEN(n)
  new_pixels = UNDERPIX_MAP2MAP(pixels, map_dim_x, map_dim_y, underpix_pix, s[1], s[2], /reverse)
  result[pixels] = proba[new_pixels]
  RETURN, result
END

PRO ADD_POSITIONS_TO_CATALOG, cat, lon_map, lat_map, box, D_g_box, cell_size, bias, show=show, $
		                cosmo=cosmo, map_type=map_type, iseed=iseed

; Written for small maps and without quintessence

IF NOT KEYWORD_SET(iseed) THEN iseed=LONG(systime(1))

IF KEYWORD_SET(show) EQ 0 THEN show=0 ELSE show=1
dimensions = [N_ELEMENTS(box(*,0,0)),N_ELEMENTS(box(0,*,0)),N_ELEMENTS(box(0,0,*))]

tagnames = TAG_NAMES(cat)

mag = FIX(MIN([N_ELEMENTS(lon_map(0,*))/720.,N_ELEMENTS(lon_map(*,0))/900.]))>1

catalog = CREATE_STRUCT(cat(0),'lon',0.,'lat',0.,'pixel',0L)
catalog = REPLICATE(catalog,N_ELEMENTS(cat))
FOR i =0, N_ELEMENTS(tagnames)-1 DO catalog.(i) = cat.(i)

z_cut = 5.
boxpix = cell_size/!dtor/((1+z_cut)*ANG_DIST(z_cut, cosmo=cosmo))
mappix = lon_map[1,0]-lon_map[0,0] ; lon_map[1,0]-lon_map[0,0]=lat_map[0,1]-lat_map[0,0]
underpix_pix = FIX(boxpix/mappix)>1
s = SIZE(lon_map)
map_dim_x = s[1]
map_dim_y = s[2]
underpix_pix = PREVIOUS_EVEN(underpix_pix) ; nx and ny must be both even
underpix_mappix = underpix_pix*mappix
underpix_map_dim_x = CEIL(map_dim_x/FLOAT(underpix_pix))
underpix_map_dim_y = CEIL(map_dim_y/FLOAT(underpix_pix))
underpix_lon_map = (FINDGEN(underpix_map_dim_x)-(underpix_map_dim_x-1.)/2.)# $
                   REPLICATE(underpix_mappix,underpix_map_dim_y)
underpix_lat_map = REPLICATE(underpix_mappix,underpix_map_dim_x)# $
                   (FINDGEN(underpix_map_dim_y)-(underpix_map_dim_y-1.)/2.)

total_pos_map = 0.*lon_map

z_list = cat(UNIQ(cat.z, SORT(cat.z))).z
FOR i=0, N_ELEMENTS(z_list)-1 DO BEGIN
        pos_map = 0.*lon_map
	which_clusters = WHERE(cat.z EQ z_list[i], ndraw)
        redshift = z_list[i]
	masses = cat(which_clusters).mass
        COMOVING2OBSERVABLE, x_map, y_map, z_map, underpix_lon_map, underpix_lat_map, redshift, $
                               cosmo=cosmo, /reverse
	BOX_PIX2COORD, px, py, pz, x_map, y_map, z_map, dimensions, $
                               cell_size, map_type='small-map',/reverse
	pixels = LONARR(ndraw)
	proba = 1+box(px,py,pz)*D_g_box(px,py,pz)
        ;include effect of the clustering bias:
	FOR ncl = 0, ndraw-1 DO BEGIN
	  clustering_bias = bias[which_clusters[ncl]]
          proba_to_use = HOLES_FILLING(1.+(proba-1.)*clustering_bias)
                         ;FILL_VACUUM(1.+(proba-1.)*clustering_bias)
                         ;MAKE_POSITIVE(1.+(proba-1.)*clustering_bias)
          ;wh = WHERE(NOT proba_to_use eq 0, nwh) -> deux lignes peu coherentes retirees
          ;IF nwh EQ 0 then begin -> le 13 dec 2005
          wh = WHERE(proba_to_use LT 0, nwh)
          IF nwh GE 1 then begin
            print, 'attention, proba negative !'
            proba_to_use[*]=1.
          endif
          pixels(ncl) = RANDOM_DRAW(proba_to_use, 1,/silent, iseed=iseed)
	ENDFOR
        real_pix = UNDERPIX_MAP2MAP(pixels, map_dim_x, map_dim_y, $
                           underpix_pix, underpix_map_dim_x, underpix_map_dim_y, iseed=iseed)
	catalog(which_clusters).lon = lon_map(real_pix)
	catalog(which_clusters).lat = lat_map(real_pix)
	catalog(which_clusters).pixel = real_pix
	pos_map(real_pix)=1
	total_pos_map = total_pos_map+pos_map
	  IF show EQ 1 THEN BEGIN
            prob = REPIX(proba, map_dim_x, map_dim_y, underpix_pix)
            IF i eq 0 THEN SIMPLEVISU, 255*pos_map+BYTSCL(prob), $
               mag=mag ELSE SIMPLEVISU, 255*pos_map+BYTSCL(prob), mag=mag, /nonew
	       XYOUTS, 10, 10, 'redshift = '+STRTRIM(STRING(z_list(i)),1), /device
	       wait, .5
          ENDIF
ENDFOR

IF show EQ 1 THEN BEGIN
	simplevisu, total_pos_map, mag=mag, /no
ENDIF
cat = catalog
END
