; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2009
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	CREATE_DETERMINISTIC_SZ_SIMULATION_ALLSKY
;
; PURPOSE:
;	...
;
;-
; BE CAREFUL : This procedure is adapted from CREATE_SZ_SIMULATION_ALLSKY. It
; simulates skies with known clusters only

; March 22 2007
; Written, Jean-Baptiste Melin
; July 24 2008
; xmm, chandra and norm keywords added, JBM
; July 25 2008
; pixel field added
; Dec 10 2008
; nr500 keyword added, JBM
; April 14 2009
; nomap keyword added, JBM
; nstd keyword added, JBM, June 14 2012

PRO CREATE_DETERMINISTIC_SZ_SIMULATION_ALLSKY, map_hd, catalog, y_map, taubeta_map, taubeta2_q_map, taubeta2_u_map, $
			                       cosmo=cosmo, cluster=cluster, $
                                               polar=polar, chandra=chandra, norm=norm, xmm=xmm, nr500=nr500, nomap=nomap, nstd=nstd

IF NOT KEYWORD_SET(nomap) THEN BEGIN

  nside = SXPAR(map_hd, 'NSIDE')
		
  n_i_cat = N_ELEMENTS(catalog)
  tagnames = TAG_NAMES(catalog)
  cat = CREATE_STRUCT(catalog(0),'pixel',0L)
  cat = REPLICATE(cat,n_i_cat)
  FOR i=0, N_ELEMENTS(tagnames)-1 DO cat.(i) = catalog.(i)
  theta = !pi/2.-!dtor*cat.lat
  phi = !dtor*cat.lon
  ang2pix_ring, nside, theta, phi, pixel_cat
  theta = 0B
  phi = 0B
  cat.pixel = pixel_cat
  catalog=cat
  cat=0B

ENDIF

mf_cat = catalog

catalog = CREATE_STRUCT(mf_cat(0),'v_radial',0d,'v_transverse',0d,'angle',0.)
catalog = REPLICATE(catalog,N_ELEMENTS(mf_cat))

tagnames = TAG_NAMES(mf_cat)
nnn = N_ELEMENTS(tagnames)
FOR i =0, nnn-1 DO catalog.(i) = mf_cat.(i)

catalog.v_radial = 0.				
catalog.v_transverse = 0.
catalog.angle = 0.

mf_cat = catalog

;add cluster parameters
;----------------------
PRINT,''
PRINT, '  MODELLING THE CLUSTERS'
model = CLUSTER_MODEL(mf_cat.z, mf_cat.mass, cosmo=cosmo, cluster=cluster, $
	/Te, /ne0, /tau0, /y0, chandra=chandra, norm=norm,xmm=xmm, c500=c500, cm500=cm500, nstd=nstd)
IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN
  IF KEYWORD_SET(nr500) THEN BEGIN
    catalog = CREATE_STRUCT(mf_cat(0),'Te',0d,'ne0',0d,'rs',0d,'rv',0d,'tau0',0d,'y0',0d,'ts',0.,'tv',0.,'cy',0d, $
                            'r500',0d,'t500',0d,'mass500',0d,'cy500',0d, $
                            'rn500',0d,'tn500',0d,'massn500',0d,'cyn500',0d)
  ENDIF ELSE BEGIN
    catalog = CREATE_STRUCT(mf_cat(0),'Te',0d,'ne0',0d,'rs',0d,'rv',0d,'tau0',0d,'y0',0d,'ts',0.,'tv',0.,'cy',0d)
  ENDELSE
ENDIF ELSE BEGIN
  catalog = CREATE_STRUCT(mf_cat(0),'Te',0d,'ne0',0d,'rc',0d,'rv',0d,'tau0',0d,'y0',0d,'tc',0.,'tv',0.,'cy',0d)
ENDELSE
catalog = REPLICATE(catalog,N_ELEMENTS(mf_cat))

tagnames = TAG_NAMES(mf_cat)
nnn = N_ELEMENTS(tagnames)
FOR i=0, nnn-1 DO catalog.(i) = mf_cat.(i)
tagnames = TAG_NAMES(model)
FOR i=0, N_ELEMENTS(tagnames)-1 DO catalog.(i+nnn) = model.(i)

da = ANG_DIST(catalog.z, cosmo=cosmo)

IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN
  catalog.ts = catalog.rs/!dtor/da*60.
ENDIF ELSE BEGIN
  catalog.tc = catalog.rc/!dtor/da*60.
ENDELSE
catalog.tv = catalog.rv/!dtor/da*60.

catalog.cy = FAST_INT_Y(catalog.z, catalog.mass, cosmo=cosmo, cluster=cluster,chandra=chandra, norm=norm, xmm=xmm, nstd=nstd)

IF (KEYWORD_SET(chandra) OR KEYWORD_SET(xmm)) AND KEYWORD_SET(nr500) THEN BEGIN
  catalog.r500 = c500*catalog.rs
  catalog.t500 = c500*catalog.ts
  FAST_CONVERT_MASS_RADIUS, catalog.mass, catalog.z, INTERPOL(cluster.big_delta_c_array, cluster.z_array, catalog.z), REPLICATE(500.,N_ELEMENTS(catalog)), $
                            REPLICATE(cm500,N_ELEMENTS(catalog)),rvir,r500,m500,cosmo=cosmo,/five_hundred
  catalog.mass500 = m500
  catalog.cy500 = FAST_INT_Y(catalog.z, catalog.mass, cosmo=cosmo, cluster=cluster, chandra=chandra, xmm=xmm, norm=norm, nr500=1., nstd=nstd)
  catalog.rn500 = nr500*catalog.r500
  catalog.tn500 = nr500*catalog.t500
  catalog.massn500 = (ALOG(1.+nr500*cm500)-nr500*cm500/(1.+nr500*cm500))/(ALOG(1.+cm500)-cm500/(1.+cm500))*catalog.mass500
  factn500 = FAST_INT_Y(catalog[0].z, catalog[0].mass, cosmo=cosmo, cluster=cluster, chandra=chandra, xmm=xmm, norm=norm, nr500=nr500, nstd=nstd)/catalog[0].cy500
  catalog.cyn500=factn500[0]*catalog.cy500
ENDIF

;mf_cat = catalog
mf_cat = 0B

;make SZ maps
;------------
;PRINT, ''
;PRINT, '  GENERATING THE SZ MAPS'
;z_limits = [0.,10.]
;which = WHERE( (mf_cat.z GE z_limits(0)) AND (mf_cat.z LE z_limits(1)) )
;catalog = mf_cat(which)
;mf_cat = 0B

IF NOT KEYWORD_SET(nomap) THEN BEGIN

CREATE_Y_TAUBETA_MAP_ALLSKY, nside, catalog, y_map, taubeta_map, $
                             taubeta2_q_map, taubeta2_u_map, $
                             cosmo=cosmo, cluster=cluster, $
                             polar=polar,chandra=chandra, norm=norm, xmm=xmm, nr500=nr500, nstd=nstd

ENDIF

END
