; -----------------------------------------------------------------------------
;
; Copyright (C) 2009
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	CREATE_SZ_MAP_ALLSKY
;
; PURPOSE:
;	...
;
;-
; Function to create an sz map
; 
; @categories sz
; 
; @param mapp {in}{required}{type=structure} pixelisation parameters
; @param catalog {in}{required}{type=structure} Number of clusters in the catalogue 
; @param ymap {out}{required}{type=float} SZ y map
; @param taubeta_map {out}{required}{type=float} optical depth times radial velocity map
; @param taubeta2_q_map {out}{required}{type=float} optical depth times transverse velocity square map (Q)
; @param taubeta2_u_map {out}{required}{type=float} optical depth times transverse velocity square map (U)
; @keyword cosmo {in}{required}{type=structure} cosmological parameters
; @keyword cluster {in}{required}{type=structure} cluster parameters
; @keyword polar {in}{optional}{type=boolean} compute polarized maps if set
; @keyword chandra {in}{optional}{type=boolean} set the chandra model
; @keyword xmm {in}{optional}{type=boolean} set the xmm model
; @keyword norm {in}{optional}{type=boolean} model normalized on observations if set
; @keyword nr500 {in}{optional}{type=float} n times r500 
;
; @history <p>14/04/2009: First version, Jacques Delabrouille and Jean-Baptiste Melin</p>
; @history <p>19/05/2009: Adapted to new values in param, Jean-Baptiste Melin</p>
; @history <p>10/05/2012: Implemented first order relativistic correction, Jacques Delabrouille</p>
; @history <p>14/06/2012: NSTD keyword added, Jean-Baptiste Melin</p>

PRO CREATE_SZ_MAP_ALLSKY, catalog, y_map, taubeta_map, taubeta2_q_map, taubeta2_u_map, ythetae_map=ythetae_map, ythetae2_map=ythetae2_map, ythetae3_map=ythetae3_map, ythetae4_map=ythetae4_map, relativistic=relativistic

COMMON psm, param
COMMON sky, model

pixp = SKY_PIX()
szp = model.sz
cp = model.cosmo
cosmo = cp

;; cosmo = DEFINECOSMO(h=cp.h, $
;;                        omega_m=cp.Omega_m, $
;;                        omega_b=cp.Omega_b, $
;;                        lambda=cp.Omega_de, $
;;                       /silent)

cluster = DEFINE_CLUSTER(cosmo=cosmo, t_star=szp.cluster_t_star, /silent)

case strlowcase(szp.cluster_profile) of
  'xmm': begin
           xmm=1
           chandra=0
         end
  'chandra': begin
               xmm=0
               chandra=1
             end
  'beta': begin
            xmm=0
            chandra=0
          end
endcase
case strlowcase(szp.nstd_profile) of
  'no': nstd=0
  'yes':nstd=1
endcase
case strlowcase(szp.norm_profile) of
  'no': norm=0
  'yes':norm=1
endcase
case strlowcase(szp.profile_bounds) of
  '5r500': nr500=5.
  'rvir':              ; default case
endcase

IF (strlowcase(szp.sz_include_polarised) eq 'yes') THEN polar=1 ELSE polar=0

IF pixp.pixtype EQ 'HEALPIX' THEN BEGIN

  nside = pixp.nside

  n_i_cat = N_ELEMENTS(catalog)
  tagnames = TAG_NAMES(catalog)
  cat = CREATE_STRUCT(catalog(0),'pixel',0L)
  cat = REPLICATE(cat,n_i_cat)
  FOR i=0, N_ELEMENTS(tagnames)-1 DO cat.(i) = catalog.(i)
  theta = !pi/2.-!dtor*cat.lat
  phi = !dtor*cat.lon
  IF pixp.ordering EQ 'RING' THEN ang2pix_ring, nside, theta, phi, pixel_cat ELSE MESSAGE, 'Case not implemented'
  theta = 0B
  phi = 0B
  cat.pixel = pixel_cat
  catalog=cat
  cat=0B

  CREATE_Y_TAUBETA_MAP_ALLSKY, nside, catalog, y_map, taubeta_map, $
                               taubeta2_q_map, taubeta2_u_map, $
                               ythetae_map = ythetae_map, ythetae2_map=ythetae2_map, ythetae3_map=ythetae3_map, ythetae4_map=ythetae4_map, $
                               relativistic = relativistic, $
                               cosmo=cosmo, cluster=cluster, $
                               polar=polar,chandra=chandra, norm=norm, xmm=xmm, nr500=nr500, nstd=nstd

ENDIF ELSE BEGIN

  PRINT, 'This pixelisation is not supported for the SZ maps for the moment !'

ENDELSE

END
