; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	FILE_EXTENSION
;
; PURPOSE:
;	Function to get the extension of a file.
;
; CATEGORY:
;	
;
; CALLING SEQUENCE:
;	result = FILE_EXTENSION(File)
;
; INPUTS:
;	File:	A file name.
;
; OUTPUTS:
;	The file extension (substring after the last dot).
;
; RESTRICTIONS:
;	Works for unix/linux OS standard conventions.
;
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Straightforward.
;
; EXAMPLE:
;	> PRINT, FILE_EXTENSION('file.fits')
;
;       (IDL prints 'fits')
;
; MODIFICATION HISTORY:
; 	19/01/2009: 	Jacques Delabrouille, First version.
;	January 2012: 	JD, Updated doc header and PSM license header.
;-
FUNCTION FILE_EXTENSION, file

  IF ISARRAY(file) THEN BEGIN
     nel = N_ELEMENTS(file)
     result = STRARR(nel)
     FOR i=0, nel-1 DO result[i] = FILE_EXTENSION(file[i])
     RETURN, result
  ENDIF ELSE BEGIN
     basename = FILE_BASENAME(file)
     list = STR_SEP(basename,'.')
     nel = N_ELEMENTS(list)
     IF nel LE 1 THEN RETURN, '' ELSE RETURN, list[nel-1]
  ENDELSE
  
END
