; -----------------------------------------------------------------------------
;
; Copyright (C) 2000
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	FIX_SEPARATOR
;
; PURPOSE:
;	This function adds the directory separator at the end of a
;	directory name if it is not already there.
;
; CATEGORY:
;	
;
; CALLING SEQUENCE:
;	result = FIX_SEPARATOR(Dir)
;
; INPUTS:
;	Dir:	An input directory name (string) with or without final
;	        directory separator (e.g. '/' for unix-based OS).
;
; OUTPUTS:
;	Directory name (string) ending with the directory separator
;	for the operating system.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Straightforward.
;
; EXAMPLE:
;	Under UNIX operating system:
;
;	> olddirectoryname = '/Soft/PSM'
;	> PRINT, FIX_SEPARATOR(olddirectoryname)
;
;	(IDL prints: '/Soft/PSM/').
;
; MODIFICATION HISTORY:
;	02/08/2000: 	Jacques Delabrouille, First version.
;	05/01/2012:	JD, Updated doc header and PSM license header.
;-
FUNCTION FIX_SEPARATOR, dir, remove=remove

  CASE !version.os_family OF
     'MacOS' : sep=':'
     'unix' : sep='/'
     ELSE : BEGIN
        PRINT, 'WARNING : program checked for mac and unix only'
        sep=!sep
     END
  ENDCASE
  
  len = STRLEN(dir)
  IF NOT KEYWORD_SET(remove) THEN BEGIN
     IF STRMID(dir,len-1) NE sep THEN RETURN, dir+sep ELSE RETURN, dir
  ENDIF ELSE BEGIN
     IF STRMID(dir,len-1) NE sep THEN RETURN, dir ELSE RETURN, STRMID(dir,0,len-1)
  ENDELSE

END
