; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2011
;   Jacques Delabrouille
;   Marc Betoule
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PSM_TMPFILENAME
;
; PURPOSE:
;	...
;
;-
FUNCTION PSM_TMPFILENAME, extension=extension, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSM_TMPFILENAME: function to get a temporary file name'
     PSM_INFO_MESSAGE, 'SYNTAX: result = PSM_TMPFILENAME(extension=)'
     PSM_INFO_MESSAGE, "extension is the extension to be used for the file (default = '.tmp')."
     PSM_INFO_MESSAGE, "Remember to erase the temporary file after use !"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  IF NOT DEFINED(extension) THEN extension='.tmp'
  IF STRMID(extension,0,1) NE '.' THEN extension = '.'+extension


  try_a_file_name:
  PSM_ID_KEYGEN, key
  filename = FIX_SEPARATOR(GETENV('IDL_TMPDIR')) + 'psm_tmpfile_' + key + extension
  
;; Check where filename already exists, test for duplicates, try again if needed
  notok = FILE_TEST(filename)
  IF notok THEN GOTO, try_a_file_name

  DEFSYSV, '!psm_tmpfilenames', exists=exists
  IF exists THEN BEGIN
     IF PTR_VALID(!psm_tmpfilenames) THEN BEGIN
        files = *(!psm_tmpfilenames)
        wh = WHERE(files EQ filename, nwh)
        IF nwh NE 0 THEN GOTO, try_a_file_name
        files = [files,filename]
        *(!psm_tmpfilenames) = files
     ENDIF ELSE BEGIN
        filenameptr = PTR_NEW([filename])
        DEFSYSV, '!psm_tmpfilenames', filenameptr
     ENDELSE
  ENDIF ELSE BEGIN
     filenameptr = PTR_NEW([filename])
     DEFSYSV, '!psm_tmpfilenames', filenameptr
  ENDELSE

  ;print, TRANSPOSE(*(!psm_tmpfilenames))
  RETURN, filename

END
