; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2011
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PSM_TMPFILES
;
; PURPOSE:
;	...
;
;-
FUNCTION PSM_TMPFILES, allpsm=allpsm, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSM_TMPFILES: function returning the list of current PSM temporary files'
     PSM_INFO_MESSAGE, 'SYNTAX: result = PSM_TMPFILES(/allpsm)'
     PSM_INFO_MESSAGE, "If /allpsm is set, then the function returns all files created by the PSM"
     PSM_INFO_MESSAGE, "otherwise it returns only the list of temporary files created in the present PSM session"
     PSM_INFO_MESSAGE, "/allpsm permits, for instance, to find files remaining after a crash in a previous PSM run"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF


  IF NOT KEYWORD_SET(allpsm) THEN BEGIN
     DEFSYSV, '!psm_tmpfilenames', exists=exists
     IF exists THEN BEGIN
        IF PTR_VALID(!psm_tmpfilenames) THEN BEGIN
           list = *(!psm_tmpfilenames)
           nf = N_ELEMENTS(list)
           fileexist = INTARR(nf)
           FOR i=0, nf-1 DO fileexist[i] = FILE_TEST(list[i])
           whfiles = WHERE(fileexist EQ 1, nwh)
           IF nwh NE 0 THEN list = list[whfiles] ELSE list=''
           RETURN, list
        ENDIF ELSE RETURN, ''
     ENDIF ELSE RETURN, ''
  ENDIF ELSE BEGIN
     list = FILE_SEARCH(FIX_SEPARATOR(GETENV('IDL_TMPDIR')),'psm_tmpfile_*.*')
     RETURN, list
  ENDELSE


END
