PRO HPX_PIX_CONVOL, map_in, map_out, kernelpix=kernelpix, ordering=ordering, silent=silent

  ;; Default values for keywords
  ;;-----------------------------
  IF NOT KEYWORD_SET(kernelpix) THEN kernelpix=[1,1]
  IF NOT KEYWORD_SET(ordering) THEN ordering='RING'

  ;; Create output array
  ;;---------------------
  map_out = map_in
  nside = GET_NSIDE(map_in)

  ;; Make 2-D kernel
  ;;-----------------
  nhalf = N_ELEMENTS(kernelpix)
  maxremote = nhalf-1
  nsidekern = 2*nhalf-1
  kernel2d = FLTARR(nsidekern,nsidekern)
  kernel2d[nhalf-1,nhalf-1] = kernelpix[0]
  IF nhalf GE 2 THEN BEGIN
     FOR i=1,nhalf-1 DO BEGIN 
        kernel2d[nhalf-i-1,nhalf-i-1:nhalf+i-1] = kernelpix[i]
        kernel2d[nhalf+i-1,nhalf-i-1:nhalf+i-1] = kernelpix[i]
        kernel2d[nhalf-i-1:nhalf+i-1,nhalf-i-1] = kernelpix[i]
        kernel2d[nhalf-i-1:nhalf+i-1,nhalf+i-1] = kernelpix[i]
     ENDFOR
  ENDIF

  ;; Smooth all faces
  ;;------------------
  FOR face=0, 11 DO BEGIN
     
     IF NOT KEYWORD_SET(silent) THEN PRINT, 'Convolution for face '+NUMBER2STRING(face)

     ;; Find pixels for face
     pixface = HPX_FACE_PIX(nside, face=face, ordering=ordering)

     ;; Read map_in values
     face_in = map_in[pixface]

     ;; Smooth the face
     face_in = CONVOL(TEMPORARY(face_in),kernel2d)

     ;; Assign values to output map
     map_out[pixface] = TEMPORARY(face_in)

     ;; Take care of the borders, which are wrong for the moment
     IF nhalf GE 2 THEN BEGIN

        ;; Find border pixels, not smoothed in an appropriate way
        borderpix1 = pixface[0:(nhalf-2),0:(nside-nhalf)]
        borderpix2 = pixface[0:(nside-nhalf),(nside-nhalf+1):(nside-1)]
        borderpix3 = pixface[(nside-nhalf+1):(nside-1),(nhalf-1):(nside-1)]
        borderpix4 = pixface[(nhalf-1):(nside-1),0:(nhalf-2)]
        
        borderpix = [borderpix1[*],borderpix2[*],borderpix3[*],borderpix4[*]]
        nb = N_ELEMENTS(borderpix)

        ;; For computing factor for normalisation, to take into account the number
        ;; of pixels in each ring of 'remoteness'
        factor = REPLICATE(1.,nhalf)
        expected = REPLICATE(1.,nhalf) & FOR r=1L,nhalf-1 DO expected[r] = 8.*r

        ;; Loop on border pixels, using hpx_pix_neighbours
        FOR p=0L, nb-1 DO BEGIN
           ipix = borderpix[p]
           HPX_PIX_NEIGHBOURS, ipix, remote, list, maxremote=maxremote, nside=nside, ordering=ordering
           FOR x=1, maxremote DO BEGIN
              wh = WHERE(remote EQ x,nwh)
              factor[x] = expected[x]/nwh
           ENDFOR
           f=factor*kernelpix
           map_out[ipix] = TOTAL(map_in[list]*f[remote])
        ENDFOR

     ENDIF

  ENDFOR

END
