; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Jacques Delabrouille
;   Soumen Basak
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	COS_TRANSITION_MASK
;
; PURPOSE:
;	...
;
;-
FUNCTION COS_TRANSITION_MASK, rangelon, trans, nside=nside, ordering=ordering, coord_map=coord_map, coord_mask=coord_mask, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'COS_TRANSITION_MASK: function to get a latitude mask with a cos squared apodisation'
     PSM_INFO_MESSAGE, 'SYNTAX: result = COS_TRANSITION_MASK(rangelon, trans, nside=nside, ordering=, coord_map=, coord_mask=)'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  
  ;; Set default values for keywords
  ;;---------------------------------
  IF NOT KEYWORD_SET(nside) THEN nside = 2048
  IF NOT KEYWORD_SET(ordering) THEN ordering = 'RING'
  IF KEYWORD_SET(coord_map) EQ 0 THEN coord_map = 'G'
  IF KEYWORD_SET(coord_mask) EQ 0 THEN coord_mask = 'G'


  ;; IF COORDINATE CONVERSION IS NEEDED
  ;;------------------------------------
  IF coord_map NE coord_mask THEN BEGIN

     ;; Compute longitude and latitude of map pixels
     ipix = LINDGEN(LONG(nside)*LONG(nside)*12)
     IF STRUPCASE(ordering) EQ 'NESTED' THEN PIX2ANG_NEST, nside, ipix, lat, phi
     IF STRUPCASE(ordering) EQ 'RING' THEN PIX2ANG_RING, nside, ipix, lat, phi
     ipix = 0B
     lat = FLOAT(90.-TEMPORARY(lat)/!DTOR)
     phi = FLOAT(TEMPORARY(phi)/!DTOR)
     
     ;; Convert to mask coordinates
     CASE coord_map OF
        'G': BEGIN
           IF coord_mask EQ 'E' THEN EULER, phi, lat, phimask, latmask, 6
        END
        'E': BEGIN
           IF coord_mask EQ 'G' THEN EULER, phi, lat, phimask, latmask, 5
        END
     ENDCASE
     lat = TEMPORARY(latmask)
     phimask = 0
     latmask = 0

  ;; IF COORDINATE CONVERSION IS NOT NECESSARY
  ;;-------------------------------------------
  ENDIF ELSE BEGIN

     ;; Compute latitude of map pixels
     ipix = LINDGEN(LONG(nside)*LONG(nside)*12)
     IF STRUPCASE(ordering) EQ 'NESTED' THEN PIX2THETA_NEST, nside, ipix, lat
     IF STRUPCASE(ordering) EQ 'RING' THEN PIX2THETA_RING, nside, ipix, lat
     ipix = 0B
     lat = FLOAT(90.-TEMPORARY(lat)/!DTOR)

  ENDELSE

  phi=0
  lat = abs(TEMPORARY(lat))
  
  mask = lat GT (rangelon+trans)
  
  IF trans GT 0. THEN BEGIN
     select = (lat GE rangelon) * (lat LE (rangelon+trans))
     mask = TEMPORARY(mask) + TEMPORARY(select) * SIN( !pi*(TEMPORARY(lat)-rangelon)/trans/2 ) ^2
     ;mask = mask + $
     ;      (lat GE rangelon) * $
     ;      (lat LE (rangelon+trans)) * $
     ;      SIN( !pi*(lat-rangelon)/trans/2 ) ^2
  ENDIF
  
  
  RETURN, mask
END
