; -----------------------------------------------------------------------------
;
; Copyright (C) 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	WG2_MASK
;
; PURPOSE:
;	...
;
;-
FUNCTION WG2_MASK, small=small, nside=nside, ordering=ordering, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'WG2_MASK: function which returns the WG2-challenge2 large or small galactic mask'
     PSM_INFO_MESSAGE, 'SYNTAX: result = WG2_MASK(/small,nside=,ordering=)'
     PSM_INFO_MESSAGE, "set /small to get the small mask instead of the default large mask"
     PSM_INFO_MESSAGE, "Specify optionally the output nside [defaut=2048] and ordering [default='RING']"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  ;; Set default values for keywords
  ;;---------------------------------
  IF NOT KEYWORD_SET(nside) THEN nside = 2048
  IF NOT KEYWORD_SET(ordering) THEN ordering = 'RING'

  IF NOT KEYWORD_SET(small) THEN BEGIN
     maskfile = GETPSMDIR('psmdata')+'ancillary/masks/wg2/WG2_large_galactic_mask_2048.fits'
  ENDIF ELSE BEGIN
     maskfile = GETPSMDIR('psmdata')+'ancillary/masks/wg2/WG2_small_galactic_mask_2048.fits'
  ENDELSE
  ok = TEST_PSM_DATA(maskfile,/get)
  IF NOT ok THEN BEGIN
     PSM_INFO_MESSAGE, 'Sorry, file for WG2 mask is not available. Returning.'
     RETURN, -1
  ENDIF ELSE BEGIN
     READ_FITS_MAP, maskfile, mask
     CASE ordering OF
        'RING': BEGIN
           IF nside NE 2048 THEN UD_GRADE, mask, mask, nside_out=nside, order_in='RING', order_out='RING'
        END
        'NESTED': BEGIN
           IF nside NE 2048 THEN BEGIN
              UD_GRADE, mask, mask, nside_out=nside, order_in='RING', order_out='NESTED' 
           ENDIF ELSE BEGIN
              mask = REORDER(mask,/r2n)
           ENDELSE
        END
     ENDCASE
  ENDELSE

  mask = (mask EQ 1)
  RETURN, mask

END
