; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	COVARMATRIX
;
; PURPOSE:
;	...
;
;-
FUNCTION COVARMATRIX, x, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'COVARMATRIX: function to compute the covariance matrix of nx vectors'
     PSM_INFO_MESSAGE, 'SYNTAX: result = COVARMATRIX(x)'
     PSM_INFO_MESSAGE, "x is the input data set (dimension [np,nx])"
     PSM_INFO_MESSAGE, "the output is the covariance matrix (dimension [nx,nx])"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

ndim = SIZE(x,/n_dimensions)
IF ndim NE 2 THEN MESSAGE, 'works only for two-dimensional input arrays'
dim = SIZE(x,/dimensions)
nx = dim(ndim-1)

covar = FLTARR(nx,nx)
FOR i=0, nx-1 DO BEGIN
   FOR j=i, nx-1 DO BEGIN
      s = AVG(x[*,i]*x[*,j])
      covar[i,j] = s
      covar[j,i] = s
   ENDFOR
ENDFOR

RETURN, covar
END
