; -----------------------------------------------------------------------------
;
; Copyright (C) 2001, 2010, 2012
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PREVIOUS_EVEN
;
; PURPOSE:
;	This procedure returns the even number preceding the input
;	positive number (i.e. the largest even number equal to or
;	smaller than the input).
;
; CATEGORY:
;	Mathematics
;
; CALLING SEQUENCE:
;	result = PREVIOUS_EVEN(Int)
;
; INPUTS:
;	Int:	An input (long) integer
;
; OUTPUTS:
;	Even number just smaller or equal to the input integer.
;
; SIDE EFFECTS:
;	None.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
; 	October 2001: 	Jean-Baptiste Melin, First version.
;	27/05/2010: 	JD, Now accepts negative number, does not
; 	                return 1 if input < 1
;	04/01/2012: 	JD Some clean-up + updated doc header.
;-

FUNCTION PREVIOUS_EVEN, int, help=help
  inp = LONG(int)
  
  IF (NOT (inp EQ int)) THEN BEGIN
     PRINT, 'The input must be a positive or negative integer !'
     GOTO, closing
  ENDIF
  
  IF inp/2 EQ inp/2. THEN result = inp $
  ELSE result = inp-1
  
  result = LONG(result)
  
  RETURN, result

closing:
END
