; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	COND_MRANDOMN
;
; PURPOSE:
;	...
;
;-
FUNCTION COND_MRANDOMN, covar, covarnoise, obs, seed=seed, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'COND_MRANDOMN: function to generate a constrained realization of a multivariate Gaussian'
     PSM_INFO_MESSAGE, 'SYNTAX: result = COND_MRANDOMN(covar, covarnoise, obs, seed=)'
     PSM_INFO_MESSAGE, "covar is the covariance matrix of the data to be generated (dimension [nx,nx])"
     PSM_INFO_MESSAGE, "covarnoise is the noise covariance matrix of the constraining observations (dimension [nx,nx])"
     PSM_INFO_MESSAGE, "obs is the constraining observations (dimension [np,nx])"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF
  
  ;; The objective is to generate an [np,nx]-dimensional array x, given a noisy observation y

  ;; covartot = Ry = Rx + Rn, and its inverse
  covartot = covar+covarnoise
  invcovartot = INVERT(covartot)

  ;; mean of the constrained random realization
  mu = (covar ## invcovartot) ## obs 

  ;; covariance of the random part to be added to the mean
  covaradd = covar - (covar ## invcovartot) ## covar

  ;; generation of the random deviates, using MRANDOMN from astron
  nrand = N_ELEMENTS(obs[*,0])
  IF NOT DEFINED(seed) THEN delta = MRANDOMN(GETSEED(), covaradd, nrand) $
  ELSE delta = MRANDOMN(seed, covaradd, nrand)

  RETURN, mu+delta

END


