; -----------------------------------------------------------------------------
;
; Copyright (C) 2009,2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ALM_STRUCT2ARRAY
;
; PURPOSE:
;	Convert an a_lm spherical harmonic coefficient structure
;	created with ALM_STRUCTURE into a 3-D array of dimension [N_alm,2,N_fields].
;
; CATEGORY:
;	Spherical transforms
;
; CALLING SEQUENCE:
;	ALM_STRUCT2ARRAY, Almstr, Almarr, Index
;
; INPUTS:
;	Almstr:	The input a_lm structure
;
; KEYWORD PARAMETERS:
;	None
;
; OUTPUTS:
;	Almarr:	The output array
;
; OPTIONAL OUTPUTS:
;	Index:	The corresponding a_lm index (see ALM_INDEX)
;
; COMMON BLOCKS:
;	None
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	29/10/2009:	Jacques Delabrouille, First version.
; 	20/09/2012:	JD, documentation header.
;-
PRO ALM_STRUCT2ARRAY, almstr, almarr, index

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, 'ALM_STRUCT2ARRAY: procedure to convert an alm structure to a table of alm'
     PRINT, 'SYNTAX: ALM_STRUCT2ARRAY, almstr, almarr, [index]'
     PRINT, 'input is a structure created with alm_structure.pro, output an array [n_index,2,n_fields]'
     RETURN
  ENDIF

  nalm = N_ELEMENTS(almstr.field)
  index = ALM_INDEX(almstr.lmin,almstr.lmax)
  ni = N_ELEMENTS(index)
  
  type = SIZE(almstr.a_1,/type)
  IF type EQ 5 THEN double=1 ELSE double=0 
  
  IF double THEN almarr = DBLARR(ni,2,nalm) ELSE almarr = FLTARR(ni,2,nalm)

  FOR i=0, nalm-1 DO ok=EXECUTE('almarr[*,*,i] = almstr.a_'+NUMBER2STRING(i+1))

END

