; -----------------------------------------------------------------------------
;
; Copyright (C) 2011, 2012
;   Soumen Basak
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	CAMB_CL2ALM
;
; PURPOSE:
;	...
;
;-
PRO CAMB_CL2ALM, clfile, cmbalmfile, lmin=lmin, lmax=lmax, potalmfile=potalmfile, polar=polar, double=double

  READ_CAMB_CL, clfile, cl, polar=polar
  IF NOT KEYWORD_SET(lmin) THEN lmin=2L
  IF NOT KEYWORD_SET(lmax) THEN lmax=MAX(cl.l)
  
  nl=(lmax+1L)*(lmax+2L)/2
  nalm=2
  IF KEYWORD_SET(polar) THEN nsig=3 ELSE nsig=1
  ncl=4+3*(nsig-3)/3
  
  llfac=LONARR(lmax+1)
  FOR ll=0L,lmax DO llfac(ll)=ll*(ll+1L)/2
  
  IF KEYWORD_SET(double) THEN cmbcl=DBLARR(lmax+1,ncl) ELSE cmbcl=FLTARR(lmax+1,ncl)
  IF KEYWORD_SET(double) THEN cmbalm=DBLARR(nl,nalm,nsig) ELSE cmbalm=FLTARR(nl,nalm,nsig)
  
  newlmax=MIN([N_ELEMENTS(cl.c_1_1)-1,lmax])
  cmbcl(0:newlmax,0)=cl.c_1_1(0:newlmax)
  IF ncl EQ 4 THEN BEGIN
     cmbcl(0:newlmax,1)=cl.c_2_2(0:newlmax)
     cmbcl(0:newlmax,2)=cl.c_3_3(0:newlmax)
     cmbcl(0:newlmax,3)=cl.c_1_2(0:newlmax)
  ENDIF
  
  IF KEYWORD_SET(potalmfile) THEN BEGIN
     IF KEYWORD_SET(double) THEN potcl=DBLARR(lmax+1,2) ELSE potcl=FLTARR(lmax+1,2)
     IF KEYWORD_SET(double) THEN potalm=DBLARR(nl,nalm,1) ELSE potalm=FLTARR(nl,nalm,1)
     IF ncl EQ 4 THEN BEGIN
        potcl(0:lmax,0)=cl.c_4_4(0:newlmax)
        potcl(0:lmax,1)=cl.c_1_4(0:newlmax)
     ENDIF ELSE BEGIN
        potcl(0:lmax,0)=cl.c_2_2(0:newlmax)
        potcl(0:lmax,1)=cl.c_1_2(0:newlmax)
     ENDELSE
  ENDIF
  
  g1lm=RANDOMN(GETSEED(),nl,nalm,double=double)
  g1lm(llfac,1)=0.0
  g2lm=RANDOMN(GETSEED(),nl,nalm,double=double)
  g2lm(llfac,1)=0.0
  g3lm=RANDOMN(GETSEED(),nl,nalm,double=double)
  g3lm(llfac,1)=0.0
  g4lm=RANDOMN(GETSEED(),nl,nalm,double=double)
  g4lm(llfac,1)=0.0
  
  FOR ll=lmin,newlmax DO BEGIN
     ind=llfac(ll)
     cmbalm(ind,*,0)=g1lm(ind,*)*SQRT(cmbcl(ll,0))
     tamp=SQRT(cmbcl(ll,0)/2)
     FOR mm=1L,ll DO BEGIN
        ind=llfac(ll)+mm
        cmbalm(ind,*,0)=g1lm(ind,*)*tamp
     ENDFOR
  ENDFOR
  
  IF nsig EQ 3 THEN BEGIN
     FOR ll=lmin,newlmax DO BEGIN
        IF cmbcl(ll,0) EQ 0.d0 THEN BEGIN
           tamp=1.d0
        ENDIF ELSE BEGIN
           tamp=cmbcl(ll,0)
        ENDELSE
        corr=cmbcl(ll,3)/tamp
        xamp=SQRT(cmbcl(ll,1)-corr*cmbcl(ll,3))
        bamp=SQRT(cmbcl(ll,2))
        ind=llfac(ll)
        IF KEYWORD_SET(potalmfile) THEN potalm(ind,*,0)=g2lm(ind,*)
        cmbalm(ind,*,1)=corr*cmbalm(ind,*,0)+g2lm(ind,*)*xamp
        cmbalm(ind,*,2)=bamp*g3lm(ind,*)
        xamp=xamp/SQRT(2.d0)
        bamp=bamp/SQRT(2.d0)
        FOR mm=1L,ll DO BEGIN
           ind=llfac(ll)+mm
           IF KEYWORD_SET(potalmfile) THEN potalm(ind,*,0)=g2lm(ind,*)
           cmbalm(ind,*,1)=corr*cmbalm(ind,*,0)+g2lm(ind,*)*xamp
           cmbalm(ind,*,2)=bamp*g3lm(ind,*)
        ENDFOR
     ENDFOR
  ENDIF
  
  IF KEYWORD_SET(potalmfile) THEN BEGIN
     FOR ll=lmin,newlmax DO BEGIN
        IF cmbcl(ll,0) EQ 0.d0 THEN BEGIN
           tamp=1.d0
        ENDIF ELSE BEGIN
           tamp=cmbcl(ll,0)
        ENDELSE
        corr=potcl(ll,1)/tamp
        IF nsig EQ 3 THEN BEGIN
           examp=corr*cmbcl(ll,3)*SQRT(tamp/(cmbcl(ll,1)*tamp-cmbcl(ll,3)*cmbcl(ll,3)))
           xamp=SQRT(MAX([0.d0,potcl(ll,0)-corr*potcl(ll,1)-examp*examp]))
           ind=llfac(ll)
           potalm(ind,*,0)=-examp*potalm(ind,*,0)
           examp=examp/SQRT(2.d0)
        ENDIF ELSE BEGIN
           xamp=SQRT(MAX([0.d0,potcl(ll,0)-corr*potcl(ll,1)]))
        ENDELSE 
        ind=llfac(ll)
        potalm(ind,*,0)=potalm(ind,*,0)+corr*cmbalm(ind,*,0)+g4lm(ind,*)*xamp
        xamp=xamp/SQRT(2.d0)
        FOR mm=1L,ll DO BEGIN
           ind=llfac(ll)+mm
           IF nsig EQ 3 THEN potalm(ind,*,0)=-examp*potalm(ind,*,0)
           potalm(ind,*,0)=potalm(ind,*,0)+corr*cmbalm(ind,*,0)+g4lm(ind,*)*xamp
        ENDFOR
     ENDFOR
  ENDIF
  
  WRITE_ALM, cmbalm, cmbalmfile
  
  IF KEYWORD_SET(potalmfile) THEN WRITE_ALM, potalm, potalmfile
  
END
