; -----------------------------------------------------------------------------
;
; Copyright (C) 2008, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	NALM2LMAX
;
; PURPOSE:
;	Function that computes lmax given a number of alm
;
; CATEGORY:
;	Spherical transforms
;
; CALLING SEQUENCE:
;	Result = NALM2LMAX(Nalm)
;
; INPUTS:
;	Nalm:	Number of alm coefficients
;
; OUTPUTS:
;	Result:   The corresponding maximum harmonic mode lmax.
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
; 	05 February 2009:	Jacques Delabrouille, First version
; 	18 January 2012:	JD, transformed from procedure to
; 	                        function (while I still can do it,
; 	                        before the code is distributed too
; 	                        widely and it impacts too much other
; 	                        independent code).
;-
FUNCTION NALM2LMAX, nalm, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'NALM2LMAX: procedure to calculate lmax given the number of alm, assuming lmin=0'
     PSM_INFO_MESSAGE, 'SYNTAX: result = NALM2LMAX(nalm)'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     GOTO, closing
  ENDIF

  RETURN, LONG((SQRT(8.*nalm+1)-3.)/2.)

closing:
END
