; -----------------------------------------------------------------------------
;
; Copyright (C) 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PSM_MAP2CL
;
; PURPOSE:
;	...
;
;-
PRO PSM_MAP2CL, map, cl, lmin=lmin, lmax=lmax, debeam=debeam, double=double, weighted=weighted, ordering=ordering, help=help, iter=iter

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, ''
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSM_ALM2CL: procedure to calculate empirical cl from a map between lmin and lmax'
     PSM_INFO_MESSAGE, 'SYNTAX: PSM_MAP2CL, map, cl, lmin=, lmax=, ordering=, /weighted, /double'
     PSM_INFO_MESSAGE, 'Set /debeam to correct from the effect of the beam'
     PSM_INFO_MESSAGE, 'set /double to force double precision for output cl'
     PSM_INFO_MESSAGE, "Set ordering to 'RING' or 'NESTED' (default ='RING')"
     PSM_INFO_MESSAGE, 'map can be either a file, or an array [n_index,n_fields]'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  PSM_MAP2ALM,  map, alm, double=double, weighted=weighted, ordering=ordering, lmax=lmax, iter=iter
  PSM_ALM2CL, alm, cl, lmin=lmin, lmax=lmax, double=double

  IF KEYWORD_SET(debeam) NE 0 THEN BEGIN
     IF ISFITSFILE(map) THEN BEGIN
        filebeam = FITSFILE_BEAM(map)
        cllmax = MAX(cl.l)
        nfield = N_ELEMENTS(cl.field)
        bls = PSM_GET_BL(filebeam, lmax=cllmax, dim=nfield)
        FOR i=1, nfield DO FOR j=i, nfield DO $
           ok = EXECUTE("cl.c_"+NUMBER2STRING(i)+"_"+NUMBER2STRING(j)+"[cl.l]="+"cl.c_"+NUMBER2STRING(i)+"_"+NUMBER2STRING(j)+"[cl.l] / bls[cl.l,i-1] / bls[cl.l,j-1]")
     ENDIF
  ENDIF

END
