; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	STRUCT_EQUAL
;
; PURPOSE:
;	...
;
;-
FUNCTION STRUCT_EQUAL, struct1, struct2, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'STRUCT_EQUAL: function which tests whether two structures are equal'
     PSM_INFO_MESSAGE, 'SYNTAX: result = STRUCT_EQUAL(struct1,struct2)'
     PSM_INFO_MESSAGE, 'Function returns 1 if same tag tree and equal values for all tags, 0 otherwise'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  ;; Get all tag names for the structures (recursive)
  ;;--------------------------------------------------
  tags1 = RECURSIVE_TAG_NAMES(struct1)
  tags2 = RECURSIVE_TAG_NAMES(struct2)

  ;; Make sure the tag names are the same
  ;;--------------------------------------
  
     ;; First compare number of tags
     n1 = N_ELEMENTS(tags1) & n2 = N_ELEMENTS(tags2)
     IF n1 NE n2 THEN RETURN, 0B

     ;; Now compare tag_names
     iseq=1B
     FOR i=0, n1-1 DO iseq = (iseq AND (tags1[i] EQ tags2[i]))
     IF NOT iseq THEN RETURN, 0B

     ;; Finally compare tag values
     FOR i=0, n1-1 DO BEGIN
        ok = EXECUTE('a = struct1.'+tags1[i] )
        ok = EXECUTE('b = struct2.'+tags1[i] )
        ;; same number of elements?
        na = N_ELEMENTS(a)
        nb = N_ELEMENTS(b)
        IF na NE nb THEN RETURN, 0B
        ;; compare all elements
        iseq = 1B
        FOR j=0L, na-1 DO BEGIN
           typa = SIZE(a[j],/type)
           typb = SIZE(b[j],/type)

           IF ((typa GE 9) OR (typb GE 9)) THEN BEGIN
   
              IF (typa EQ 10) AND (typb EQ 10) THEN BEGIN
                 IF PTR_VALID(a[j]) AND PTR_VALID(b[j]) THEN BEGIN
                    typaptd = SIZE(*(a[j]),/type)
                    typbptd = SIZE(*(b[j]),/type)
                    IF (typaptd EQ 8) AND (typbptd EQ 8) THEN BEGIN
                       IF NOT STRUCT_EQUAL(*(a[j]),*(b[j])) THEN RETURN, 0B
                    ENDIF ELSE MESSAGE, 'Case not implemented'
                 ENDIF ELSE BEGIN
                    IF NOT PTR_VALID(a[j]) AND PTR_VALID(b[j]) THEN RETURN, 0B
                    IF PTR_VALID(a[j]) AND NOT PTR_VALID(b[j]) THEN RETURN, 0B
                 ENDELSE
              ENDIF ELSE MESSAGE, 'Case not implemented'
           ENDIF 

           IF ((typa GE 8) AND (typb NE typa)) THEN RETURN, 0B
           IF ((typb GE 8) AND (typb NE typa)) THEN RETURN, 0B

           IF ((typa EQ 8) AND (typb EQ 8)) THEN BEGIN
              ;; IF NOT STRUCT_EQUAL(a,b) THEN RETURN, 0B
              PSM_DEBUG_MESSAGE, 'Check this', routine='STRUCT_EQUAL'
              IF NOT STRUCT_EQUAL(a,b) THEN RETURN, 0B
           ENDIF
           
           IF ((typa EQ 7) AND (typb NE 7)) THEN RETURN, 0B
           IF ((typa NE 7) AND (typb EQ 7)) THEN RETURN, 0B

           IF NOT (a[j] EQ b[j]) THEN RETURN, 0B

        ENDFOR

     ENDFOR

     ;; If all above comparisons are OK
     RETURN, 1B

END
